
/*******************************************************************************
DISCLAIMER

THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.

WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
INTELLECTUAL PROPERTY INFRINGIMENT.

TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.

BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
THIS SOFTWARE.
BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.

IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
File:       comm_thread_entry.c
Project:    BLAC_S5D9
Revision:   1.0
Date:       28-08-17 (DD-MM-YYYY)
Author:     Andrea Ghirardello
Contents:   Communication Thread entry function
*******************************************************************************/

#define _COMM_THREAD_ENTRY_C    // insert define BEFORE including header files

#include "comm_thread.h"
#include "motor_control\i_userif.h"

#define RTX_BUF_DIM ( 280 )
#define RX_DELAY    (  10 )

typedef enum
{
    RX_WAIT     = 0,
    RX_STARTED  = 1,
    RX_FINISHED = 2
}   rtx_sta_t;

rtx_sta_t
    rtx_status = RX_WAIT;

uint8_t
    u8v_rtx_buf[RTX_BUF_DIM];

uint16_t
    u16_bytes_received,
    u16_bytes_to_receive;

uint32_t
    errors_cnt = 0;
ssp_err_t
    comm_error = SSP_SUCCESS;

void comm_thread_entry(void);


/* Comm Thread entry function */
void comm_thread_entry(void)
{
    /* TODO: add your own code here */
    while (1)
    {

        // slave is waiting to receive a request from the master
        // the request is organized as described by the protocol, so the first received byte
        // represent the total number of bytes which are contained in the request
        switch (rtx_status)
        {
            case RX_WAIT:
                comm_error = g_sf_comms0.p_api->read(g_sf_comms0.p_ctrl, &u8v_rtx_buf[0], 1, TX_WAIT_FOREVER);
                if(SSP_SUCCESS != comm_error)
                {
                    comm_error = g_sf_comms0.p_api->close (g_sf_comms0.p_ctrl);
                    comm_error = g_sf_comms0.p_api->open (g_sf_comms0.p_ctrl, g_sf_comms0.p_cfg);
                    errors_cnt++;
                    u16_bytes_to_receive = 0;
                    u16_bytes_received = 0;
                    rtx_status = RX_WAIT;
                    break;
                }
                else
                {
                    u16_bytes_received = 1;
                    if(0 < u8v_rtx_buf[0])
                    {
                        u16_bytes_to_receive = u8v_rtx_buf[0] - 1;
                    }
                    else
                    {
                        u16_bytes_to_receive = 0;
                    }
                    rtx_status = RX_STARTED;
                    //no break;
                }
            //no break;
            case RX_STARTED:
                while(0 < u16_bytes_to_receive)
                {
                    comm_error = g_sf_comms0.p_api->read(g_sf_comms0.p_ctrl, &u8v_rtx_buf[u16_bytes_received], 1, RX_DELAY);
                    if(SSP_SUCCESS != comm_error)
                    {
                        comm_error = g_sf_comms0.p_api->close (g_sf_comms0.p_ctrl);
                        comm_error = g_sf_comms0.p_api->open (g_sf_comms0.p_ctrl, g_sf_comms0.p_cfg);
                        errors_cnt++;
                        u16_bytes_to_receive = 0;
                        u16_bytes_received = 0;
                        rtx_status = RX_WAIT;
                        break;
                    }
                    else
                    {
                        u16_bytes_received++;
                        u16_bytes_to_receive--;
                    }
                }
                rtx_status = RX_FINISHED;
                //no break;
            case RX_FINISHED:
                decode_rx(u8v_rtx_buf);
                if(0 < u8v_rtx_buf[0])
                {
                    g_sf_comms0.p_api->write(g_sf_comms0.p_ctrl, u8v_rtx_buf, u8v_rtx_buf[0], TX_WAIT_FOREVER);
                }
                //no break;
            default:
                u16_bytes_to_receive = 0;
                u16_bytes_received = 0;
                rtx_status = RX_WAIT;
                break;
        }
        tx_thread_sleep(1);
    }

}
