
/*******************************************************************************
DISCLAIMER

THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.

WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
INTELLECTUAL PROPERTY INFRINGIMENT.

TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.

BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
THIS SOFTWARE.
BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.

IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
File:		gpt_pwm.h
Project:	S5D9_MotorControl
Revision:	1.0
Date:		01-05-17 (DD-MM-YYYY)
Author:		Andrea Ghirardello
Contents:	Prototypes of GPT functions used to implement three-phases pwm
*******************************************************************************/

#ifndef _GPT_PWM_H
#define _GPT_PWM_H


/***********************************************************************************************************************
 * Includes
 **********************************************************************************************************************/

#include "mc_timer_api.h"


/***********************************************************************************************************************
 * Macro definitions
 **********************************************************************************************************************/

#define GPT_PWM_CODE_VERSION_MAJOR (1)
#define GPT_PWM_CODE_VERSION_MINOR (2)


/***********************************************************************************************************************
 * Typedef definitions
 **********************************************************************************************************************/

// Level of GPT pin
typedef enum e_gpt_pwm_pin_level
{
    GPT_PWM_PIN_LEVEL_LOW      = 0,     ///< Pin level low
    GPT_PWM_PIN_LEVEL_HIGH     = 1,     ///< Pin level high
    GPT_PWM_PIN_LEVEL_RETAINED = 2      ///< Pin level retained
}   gpt_pwm_pin_level_t;

// Output level used when selecting what happens at compare match or cycle end
typedef enum e_gpt_pwm_output
{
    GPT_PWM_OUTPUT_RETAINED = 0,        ///< Output retained
    GPT_PWM_OUTPUT_LOW      = 1u,       ///< Output low
    GPT_PWM_OUTPUT_HIGH     = 2u,       ///< Output high
    GPT_PWM_OUTPUT_TOGGLED  = 3u        ///< Output toggled
}   gpt_pwm_output_t;

// Initial output levels
typedef enum e_gpt_pwm_initial_output
{
    GPT_PWM_INITIAL_OUTPUT_LOW  = 0,
    GPT_PWM_INITIAL_OUTPUT_HIGH = 1u,
}   gpt_pwm_initial_output_t;

// Configurations for output pins
typedef struct s_gpt_pwm_output_pin
{
    bool                        output_enabled;     ///< Set to true to enable output, false to disable output
    gpt_pwm_pin_level_t         stop_level;         ///< Select a stop level from ::gpt_pwm_pin_level_t
    gpt_pwm_output_t            match_level;        ///< Select a match level from ::gpt_pwm_output_t
    gpt_pwm_output_t            cycle_end_level;    ///< Select a cycle end level from ::gpt_pwm_output_t
    gpt_pwm_initial_output_t    initial_level;      ///< Select an initial level from ::gpt_pwm_initial_level_t
}   gpt_pwm_out_pin_t;

// GPT extension configures the output pins for GPT; same configuration is applied to all the three channels
typedef struct st_pwm_timer_on_gpt_cfg
{
    gpt_pwm_out_pin_t   gtioca;         ///< Configuration for GPT I/O pin A
    gpt_pwm_out_pin_t   gtiocb;         ///< Configuration for GPT I/O pin B
}   gpt_pwm_out_cfg_t;

typedef enum e_gpt_pwm_buffer_operation
{
    GPT_PWM_NO_BUFFER = 0,
    GPT_PWM_SINGLE_BUFFER = 1u,
    GPT_PWM_DOUBLE_BUFFER = 2u
}	gpt_pwm_buffer_operation_t;


/**********************************************************************************************************************
 * Exported global variables
 **********************************************************************************************************************/

// Filled in Interface API structure for this Instance
extern const mc_timer_api_t g_pwm_timer_on_gpt;

#include "hw_gpt_private.h"


__STATIC_INLINE bool HW_GPT_PWM_GTIOCPinOutputEnabled (gtior_t * const p_reg)
{
	if (0 != p_reg->GTIOR_b.OAE)	{ return ( 1 ); }
	else							{ return ( 0 ); }
}

__STATIC_INLINE void HW_GPT_PWM_CompareMatchGet (GPT_BASE_PTR p_gpt_base, uint32_t * value)
{
	*value = p_gpt_base->GTCCRC;	// GTCCRA buffer register
}

__STATIC_INLINE void HW_GPT_PWM_Start (uint32_t channelMask)
{
	R_GPTA0->GTSTR |= channelMask;
}

__STATIC_INLINE void HW_GPT_PWM_SetPOEGGA (uint32_t value)
{
    R_POEG->POEGGA = value;
}

__STATIC_INLINE void HW_GPT_PWM_SetPOEGGB (uint32_t value)
{
    R_POEG->POEGGB = value;
}

__STATIC_INLINE void HW_GPT_PWM_SetAdcStartRequest (GPT_BASE_PTR p_gpt_base, uint32_t value)
{
    p_gpt_base->GTADTRA = value;
}

__STATIC_INLINE void HW_GPT_PWM_SetSkippingCountSelect (GPT_BASE_PTR p_gpt_base, uint32_t value)
{
    p_gpt_base->GTITC = value;
}

/*
__STATIC_INLINE void HW_GPT_PWM_GTIOCPinOutputEnable (gtior_t * const p_reg, bool enable)
{
    p_reg->GTIOR_b.OAE = (uint16_t) (enable & 1U);
}
*/

/*
__STATIC_INLINE void HW_GPT_PWM_GTIOCCompareMatchOutputSet (gtior_t * const p_reg, gpt_pwm_output_t const output)
{
    p_reg->GTIOR_b.GTIOCM = output;
}
*/

/*
__STATIC_INLINE void HW_GPT_PWM_GTIOCCycleEndOutputSet (gtior_t * const p_reg, gpt_pwm_output_t const output)
{
    p_reg->GTIOR_b.GTIOCE = output;
}
*/

/*
__STATIC_INLINE void HW_GPT_PWM_GTIOCInitialOutputSet (gtior_t * const p_reg, gpt_pwm_initial_output_t const output)
{
    p_reg->GTIOR_b.GTIOCI = output;
}
*/

/*
__STATIC_INLINE void HW_GPT_PWM_GTIOCPinLevelStoppedSet (gtior_t * const p_reg, gpt_pin_level_t const level)
{
    p_reg->GTIOR_b.OADFLT = level;
}
*/

__STATIC_INLINE void HW_GPT_PWM_Gtccr_SetBufferMode (GPT_BASE_PTR p_gpt_base, const gpt_gtioc_t gtio, const gpt_pwm_buffer_operation_t value)
{
	if (GPT_GTIOCA == gtio)		{ p_gpt_base->GTBER_b.CCRA = value; }
	else						{ p_gpt_base->GTBER_b.CCRB = value; }
}

__STATIC_INLINE void HW_GPT_PWM_Gtpr_SetBufferMode (GPT_BASE_PTR p_gpt_base, const gpt_pwm_buffer_operation_t value)
{
    p_gpt_base->GTBER_b.PR = value;
}

__STATIC_INLINE void HW_GPT_PWM_Gtccr_BufferEnable (GPT_BASE_PTR p_gpt_base, const bool value)
{
   if(true == value)	{ p_gpt_base->GTBER_b.BD &= 0x2u; }
   else					{ p_gpt_base->GTBER_b.BD |= 0x1u; }
}

__STATIC_INLINE void HW_GPT_PWM_Gtpr_BufferEnable (GPT_BASE_PTR p_gpt_base, const bool value)
{
   if(true == value)	{ p_gpt_base->GTBER_b.BD &= 0x1u; }
   else					{ p_gpt_base->GTBER_b.BD |= 0x2u; }
}


__STATIC_INLINE void HW_GPT_PWM_SetDeadTime(GPT_BASE_PTR p_gpt_base, uint32_t const value)
{
   p_gpt_base->GTDVU = value;
}

/*
__STATIC_INLINE void HW_GPT_PWM_InitialCompareMatchSet (GPT_BASE_PTR p_gpt_base, gpt_gtioc_t const gtio, uint32_t const value)
{
	if (GPT_GTIOCA == gtio)		{ p_gpt_base->GTCCRA = value; }
	else						{ p_gpt_base->GTCCRB = value; }
}
*/

/*
__STATIC_INLINE void HW_GPT_PWM_CompareMatchSet (GPT_BASE_PTR p_gpt_base, gpt_gtioc_t const gtio, uint32_t const value)
{
	if (GPT_GTIOCA == gtio)		{ p_gpt_base->GTCCRC = value; }
	else						{ p_gpt_base->GTCCRD = value; }
}
*/

__STATIC_INLINE void HW_GPT_PWM_ADReqEnable (GPT_BASE_PTR p_gpt_base, bool enable )
{
	if(0 == enable)		{ p_gpt_base->GTINTAD &= 0xFFFEFFFF; }	// B16
	else				{ p_gpt_base->GTINTAD |= 0x00010000; }	// B16
}

__STATIC_INLINE void HW_GPT_PWM_POE_PowerOn (void)
{
    R_BSP_RegisterProtectDisable(BSP_REG_PROTECT_OM_LPC_BATT);
    R_MSTP->MSTPCRD_b.MSTPD5 = 0;       // GPT 0-7
    R_MSTP->MSTPCRD_b.MSTPD6 = 0;       // GPT 8-13
    R_MSTP->MSTPCRD_b.MSTPD14 = 0;      // POE
    R_BSP_RegisterProtectEnable(BSP_REG_PROTECT_OM_LPC_BATT);
}

#endif	// ifndef _GPT_PWM_H
