
/*******************************************************************************
DISCLAIMER

THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.

WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
INTELLECTUAL PROPERTY INFRINGIMENT.

TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.

BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
THIS SOFTWARE.
BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.

IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
File:		gui_defs.h
Project:	SK_S7G2_MotorControl
Revision:	1.0
Date:		01-06-16 (DD-MM-YYYY)
Author:		Andrea Ghirardello
Contents:	GUI definitions
*******************************************************************************/

#ifndef _GUI_DEFS_H
#define _GUI_DEFS_H

#include "typedefine.h"
#include "mask.h"

/******************************************************************************
Macro definitions
******************************************************************************/
// IMPORTANT NOTE: depending on the parameters type and of the ram_tab elements type (16 or 32 bits),
// to comply with the protocol requirements, the following relations MUST be verified:
// if type is 32bit then: NUM_PAR<MAX_NLD, N_RAM_READ<MAX_NLD, N_RAM_WRITE<MAX_NLD
// if type is 16bit then: NUM_PAR<MAX_NWD, N_RAM_READ<MAX_NWD, N_RAM_WRITE<MAX_NWD
// in every case: MAX_SAM_CNT<MAX_NWD

#define N_RAM_READ		(  32 )
#define N_RAM_WRITE		(   8 )
#define MAX_SAM_CNT		( 120 )
#define OUTBUF_XDIM		( 128 )
#define OUTBUF_SDIM		OUTBUF_XDIM
#define OUTBUF_CDIM		(OUTBUF_XDIM * 2)


/******************************************************************************
Typedef definitions
******************************************************************************/
typedef union
{
	uint32_t		ram_tab[N_RAM_READ];
	struct
	{
		float32_t	rif;		//  0	internal speed reference [rpm]
		float32_t	rpm;		//  1	measured speed [rpm]
		float32_t	fre;		//  2	imposed frequency [Hz]
		float32_t	id;			//  3	direct current [A]
		float32_t	iq;			//  4	quadrature current [A]
		float32_t	vd;			//  5	direct voltage [V]
		float32_t	vq;			//  6	quadrature voltage [V]
		float32_t	vb;			//  7	bus voltage [V]
		uint32_t	all;		//  8	alarm
		uint32_t	flg;		//  9	flags
		float32_t	toti;		// 10	current vector amplitude [V]
		float32_t	totv;		// 11	voltage vector amplitude [V]
		float32_t	du0;		// 12
		float32_t	du1;		// 13
		float32_t	du2;		// 14
		float32_t	du3;		// 15
		uint32_t	mod;		// 16	mode
		float32_t	rs;			// 17	stator resistance [Ohm]
		float32_t	ls;			// 18	synchronous inductance [Hen]
		float32_t	fl;			// 19	permanent magnet flux [Web]
		float32_t	kpi;		// 20	current loop kp [Ohm]
		float32_t	kii;		// 21	current loop ki (referred to continuous time) [Ohm/sec]
		float32_t	pwmf;		// 22	pwm frequency [Hz]
		float32_t	sf;			// 23	sampling frequency [Hz]
		uint32_t	ena;		// 24	extra features enable flags
		float32_t	du4;		// 25
		float32_t	du5;		// 26
		float32_t	du6;		// 27
		float32_t	du7;		// 28
		float32_t	du8;		// 29
		float32_t	du9;		// 30
		float32_t	du10;		// 31
	}	var;
}	UIF_R_t;

typedef union
{
	uint32_t		ram_tab[N_RAM_WRITE];
	struct
	{
		uint32_t	trg;		//  0	trigger
		uint32_t	mod;		//  1	mode
		float32_t	rif;		//  2	speed reference [rpm]
		float32_t	cra;		//  3	current ratio [%]
		uint32_t	sel;		//  4	variable and time scale selection
		float32_t	du0;		//  5
		float32_t	du1;		//  6
		float32_t	du2;		//  7
	}	var;
}	UIF_W_t;

typedef union
{
	int16_t		ss[OUTBUF_SDIM];
	uint8_t		uc[OUTBUF_CDIM];
}	OUTB_t;


/******************************************************************************
Variables definitions/declarations
******************************************************************************/

#ifdef _MOTORCONTROL_C
UIF_R_t					UIF_R;
UIF_W_t					UIF_W;
OUTB_t					outbuf, outbuf1;
uint16_t				*au16_wbuf = ((uint16_t *)(outbuf1.ss));
uint16_t				*au16_rbuf = ((uint16_t *)(outbuf.ss));
uint16_t				*au16_pbuf = ((uint16_t *)(outbuf.ss));
u8_bitflags_t			u8b_flg0;
#else	// ifdef _MOTORCONTROL_C
extern UIF_R_t			UIF_R;
extern UIF_W_t			UIF_W;
extern OUTB_t			outbuf, outbuf1;
extern uint16_t			*au16_wbuf;
extern uint16_t			*au16_rbuf;
extern uint16_t			*au16_pbuf;
extern u8_bitflags_t	u8b_flg0;
#endif	// else ifdef _MOTORCONTROL_C

// global flags (used for synchronization)
#define	TRGED			(0 != u8b_flg0.bit.b0)
#define	RBUSY			(0 != u8b_flg0.bit.b1)
#define	CSELD			(0 != u8b_flg0.bit.b2)
#define	FREE_03			(0 != u8b_flg0.bit.b3)
#define	FREE_04			(0 != u8b_flg0.bit.b4)
#define	FREE_05			(0 != u8b_flg0.bit.b5)
#define	FREE_06			(0 != u8b_flg0.bit.b6)
#define	FREE_07			(0 != u8b_flg0.bit.b7)

#define	NOT_TRGED		(0 == u8b_flg0.bit.b0)
#define	NOT_RBUSY		(0 == u8b_flg0.bit.b1)
#define	NOT_CSELD		(0 == u8b_flg0.bit.b2)
#define	NOT_FREE_03		(0 == u8b_flg0.bit.b3)
#define	NOT_FREE_04		(0 == u8b_flg0.bit.b4)
#define	NOT_FREE_05		(0 == u8b_flg0.bit.b5)
#define	NOT_FREE_06		(0 == u8b_flg0.bit.b6)
#define	NOT_FREE_07		(0 == u8b_flg0.bit.b7)

#define	SET_TRGED		{ u8b_flg0.bit.b0 = 1; }
#define	SET_RBUSY		{ u8b_flg0.bit.b1 = 1; }
#define	SET_CSELD		{ u8b_flg0.bit.b2 = 1; }
#define	SET_FREE_03		{ u8b_flg0.bit.b3 = 1; }
#define	SET_FREE_04		{ u8b_flg0.bit.b4 = 1; }
#define	SET_FREE_05		{ u8b_flg0.bit.b5 = 1; }
#define	SET_FREE_06		{ u8b_flg0.bit.b6 = 1; }
#define	SET_FREE_07		{ u8b_flg0.bit.b7 = 1; }

#define	RES_TRGED		{ u8b_flg0.bit.b0 = 0; }
#define	RES_RBUSY		{ u8b_flg0.bit.b1 = 0; }
#define	RES_CSELD		{ u8b_flg0.bit.b2 = 0; }
#define	RES_FREE_03		{ u8b_flg0.bit.b3 = 0; }
#define	RES_FREE_04		{ u8b_flg0.bit.b4 = 0; }
#define	RES_FREE_05		{ u8b_flg0.bit.b5 = 0; }
#define	RES_FREE_06		{ u8b_flg0.bit.b6 = 0; }
#define	RES_FREE_07		{ u8b_flg0.bit.b7 = 0; }

/*	--- Special Working Modes ---

	Special working modes are controlled trough UIF_W.var.mod:
	if it is 0 then normal mode is used, other values stay for
	special modes. A feedback of the working mode is given in UIF_R.var.mod,
	which is equal to the working mode.
	If the working mode requires a trigger from the GUI to execute some
	operations, UIF_W.var.trg bit0 is used (it is automatically cleared after
	being detected); the feedback regarding the requested operation status
	is given by some bits in UIF_R.var.flg: bit9=1 means BUSY, while if it is 0 means
	READY, bitA=1 means last operation was ENDED_NOK, while if it is 0 means ENDED_OK.
	Trigger requests are not accepted if UIF_R.var.sta is not ready. It means that the
	GUI has to stay in polling for this flag after a trigger.
	Other important flags are bit7 (alarm), and bit8 which indicates that the motor
	is driven.
	Working status can be changed only when the motor in not driven (UIF_R.var.flg
	has bit8 equal to 0).
*/

#define ENA_CURPI_TUN	{ UIF_R.var.ena |= LSET00; }
#define ENA_CURPI_AUT	{ UIF_R.var.ena |= LSET01; }
#define ENA_AUTO_IDEN	{ UIF_R.var.ena |= LSET02; }
#define ENA_OSCI_WIND	{ UIF_R.var.ena |= LSET03; }

#define NORM_MODE_CODE	( 0 )	// normal inverter behaviour
#define CURPI_TUN_CODE	( 1 )	// current PI gains manual tuning mode
#define CURPI_AUT_CODE	( 2 )	// current PI gains automatic detection mode
#define AUTO_IDEN_CODE	( 3 )	// motor parameters auto-identification mode

#define NORM_MODE_REQ	( NORM_MODE_CODE == UIF_W.var.mod )
#define CURPI_TUN_REQ	( CURPI_TUN_CODE == UIF_W.var.mod )
#define CURPI_AUT_REQ	( CURPI_AUT_CODE == UIF_W.var.mod )
#define AUTO_IDEN_REQ	( AUTO_IDEN_CODE == UIF_W.var.mod )

#define NORM_MODE		( NORM_MODE_CODE == UIF_R.var.mod )
#define CURPI_TUN		( CURPI_TUN_CODE == UIF_R.var.mod )
#define CURPI_AUT		( CURPI_AUT_CODE == UIF_R.var.mod )
#define AUTO_IDEN		( AUTO_IDEN_CODE == UIF_R.var.mod )

#define NOT_NORM_MODE	( NORM_MODE_CODE != UIF_R.var.mod )
#define NOT_CURPI_TUN	( CURPI_TUN_CODE != UIF_R.var.mod )
#define NOT_CURPI_AUT	( CURPI_AUT_CODE != UIF_R.var.mod )
#define NOT_AUTO_IDEN	( AUTO_IDEN_CODE != UIF_R.var.mod )

#define SET_NORM_MODE	{ UIF_R.var.mod = NORM_MODE_CODE; }
#define SET_CURPI_TUN	{ UIF_R.var.mod = CURPI_TUN_CODE; }
#define SET_CURPI_AUT	{ UIF_R.var.mod = CURPI_AUT_CODE; }
#define SET_AUTO_IDEN	{ UIF_R.var.mod = AUTO_IDEN_CODE; }

#define ALRM_ON			( 0 != ( UIF_R.var.flg & LSET07 ) )
#define ALRM_OFF		( 0 == ( UIF_R.var.flg & LSET07 ) )
#define COM_ON			( 0 != ( UIF_R.var.flg & LSET08 ) )
#define COM_OFF			( 0 == ( UIF_R.var.flg & LSET08 ) )
#define STA_BSY			( 0 != ( UIF_R.var.flg & LSET09 ) )
#define STA_RDY			( 0 == ( UIF_R.var.flg & LSET09 ) )
#define END_NOK			( 0 != ( UIF_R.var.flg & LSET0A ) )
#define END_OK			( 0 == ( UIF_R.var.flg & LSET0A ) )
#define ICOM_ON			( 0 != ( UIF_R.var.flg & LSET0B ) )
#define ICOM_OFF		( 0 == ( UIF_R.var.flg & LSET0B ) )
#define OUT_ENA			( 0 != ( UIF_R.var.flg & LSET0C ) )
#define NOT_OUT_ENA		( 0 == ( UIF_R.var.flg & LSET0C ) )
#define SET_ALRM_ON		{ UIF_R.var.flg |= LSET07; }
#define RES_ALRM_ON		{ UIF_R.var.flg &= LCLR07; }
#define SET_COM_ON		{ UIF_R.var.flg |= LSET08; }
#define RES_COM_ON		{ UIF_R.var.flg &= LCLR08; }
#define SET_STA_BSY		{ UIF_R.var.flg |= LSET09; }
#define RES_STA_BSY		{ UIF_R.var.flg &= LCLR09; }
#define SET_END_NOK		{ UIF_R.var.flg |= LSET0A; }
#define RES_END_NOK		{ UIF_R.var.flg &= LCLR0A; }
#define SET_ICOM_ON		{ UIF_R.var.flg |= LSET0B; }
#define RES_ICOM_ON		{ UIF_R.var.flg &= LCLR0B; }
#define SET_OUT_ENA		{ UIF_R.var.flg |= LSET0C; }
#define RES_OUT_ENA		{ UIF_R.var.flg &= LCLR0C; }

#define GUI_TRI			( 0 != ( UIF_W.var.trg & LSET00 ) )
#define NOT_GUI_TRI		( 0 == ( UIF_W.var.trg & LSET00 ) )
#define RES_GUI_TRI		{ UIF_W.var.trg &= LCLR00; }
#define GUI_STP			( 0 != ( UIF_W.var.trg & LSET01 ) )
#define NOT_GUI_STP		( 0 == ( UIF_W.var.trg & LSET01 ) )
#define RES_GUI_STP		{ UIF_W.var.trg &= LCLR01; }

#endif	// _GUI_DEFS_H

// EOF gui_defs.h

