
/*******************************************************************************
DISCLAIMER

THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.

WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
INTELLECTUAL PROPERTY INFRINGIMENT.

TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.

BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
THIS SOFTWARE.
BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.

IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
File:		par_defines.h
Project:	S5D9_MotorControl
Revision:	1.0
Date:		01-06-16 (DD-MM-YYYY)
Author:		Andrea Ghirardello
Contents:	Parameter values macro definitions
*******************************************************************************/

#ifndef _PAR_DEFINES_H
#define _PAR_DEFINES_H

// Minimum values
#define RPM_MIN_MIN		(      1.0f     )	//  1 RPM_MIN	minimum speed				rpm
#define RPM_MAX_MIN		(    100.0f     )	//  2 RPM_MAX	maximum speed				rpm
#define R_ACC_MIN		(      1.0f     )	//  3 R_ACC		acceleration				rpm/sec
#define R_DEC_MIN		(      1.0f     )	//  4 R_DEC		deceleration				rpm/sec
#define C_POLI_MIN		(      1.0f     )	//  5 C_POLI	polar couples
#define I_START_MIN		(      0.0f     )	//  6 I_START	startup current				Ampere
#define I_MAX_MIN		(      0.0f     )	//  7 I_MAX		maximum current				Ampere
#define R_STA_MIN		(      0.0f     )	//  8 R_STA		stator resistance			Ohm
#define L_SQ_MIN		(      0.0f     )	//  9 L_SQ		q synchronous inductance		Henry
#define PM_FLX_MIN		(      0.0f     )	// 10 PM_FLX	permanent magnets flux		Weber
#define KP_CUR_MIN		(      0.0f     )	// 11 KP_CUR	current loop KP				Ohm
#define KI_CUR_MIN		(      0.0f     )	// 12 KI_CUR	current loop KI (analog)	Ohm/sec
#define KP_VEL_MIN		(      0.0f     )	// 13 KP_VEL	speed loop KP				Ampere*sec/rad
#define KI_VEL_MIN		(      0.0f     )	// 14 KI_VEL	speed loop KI				Ampere/rad
#define FB_GAIN_MIN		(      0.0f     )	// 15 FB_GAIN	flux amp. feedback gain
#define PHA_OFF_MIN		(      0.0f     )	// 16 PHA_OFF	phase offset				deg
#define SUP_TIM_MIN		(      0.0f     )	// 17 SUP_TIM	startup acc. time			sec
#define FL_FTAU_MIN		(      0.0f     )	// 18 FL_FTAU	flux est. filter tau		sec
#define SAM_FRE_MIN		(   2000.0f     )	// 19 SAM_FRE	sampling frequency			Hz
#define F_RATIO_MIN		(      1.0f     )	// 20 F_RATIO	pwm_freq/sam_freq ratio

// Default values: an external file which can be generated by the GUI will be included instead
/*
#define RPM_MIN_DEF		(    300.0f     )	//  1 RPM_MIN	minimum speed				rpm
#define RPM_MAX_DEF		(   8000.0f     )	//  2 RPM_MAX	maximum speed				rpm
#define R_ACC_DEF		(   2000.0f     )	//  3 R_ACC		acceleration				rpm/sec
#define R_DEC_DEF		(   2000.0f     )	//  4 R_DEC		deceleration				rpm/sec
#define C_POLI_DEF		(      4.0f     )	//  5 C_POLI	polar couples
#define I_START_DEF		(      0.5f     )	//  6 I_START	startup current				Ampere
#define I_MAX_DEF		(      2.5f     )	//  7 I_MAX		maximum current				Ampere
#define R_STA_DEF		(      0.885f   )	//  8 R_STA		stator resistance			Ohm
#define L_SQ_DEF		(      0.0012f )	//  9 L_SQ		q synchronous inductance	Henry
#define PM_FLX_DEF		(      0.0055f  )	// 10 PM_FLX	permanent magnets flux		Weber
#define KP_CUR_DEF		(      3.5f     )	// 11 KP_CUR	current loop KP				Ohm
#define KI_CUR_DEF		(   3500.0f     )	// 12 KI_CUR	current loop KI (analog)	Ohm/sec
#define KP_VEL_DEF		(      0.002f   )	// 13 KP_VEL	speed loop KP				Ampere*sec/rad
#define KI_VEL_DEF		(      0.1f     )	// 14 KI_VEL	speed loop KI				Ampere/rad
#define FB_GAIN_DEF		(     50.0f     )	// 15 FB_GAIN	flux amp. feedback gain
#define PHA_OFF_DEF		(      0.0f     )	// 16 PHA_OFF	phase offset				deg
#define SUP_TIM_DEF		(      0.8f     )	// 17 SUP_TIM	startup acc. time			sec
#define FL_FTAU_DEF		(      0.05f    )	// 18 FL_FTAU	flux est. filter tau		sec
#define SAM_FRE_DEF		(   8000.0f     )	// 19 SAM_FRE	sampling frequency			Hz
#define F_RATIO_DEF		(      1.0f     )	// 20 F_RATIO	pwm_freq/sam_freq ratio
*/

// Maximum values
#define RPM_MIN_MAX		(   5000.0f     )	//  1 RPM_MIN	minimum speed				rpm
#define RPM_MAX_MAX		(  99999.0f     )	//  2 RPM_MAX	maximum speed				rpm
#define R_ACC_MAX		(  99999.0f     )	//  3 R_ACC		acceleration				rpm/sec
#define R_DEC_MAX		(  99999.0f     )	//  4 R_DEC		deceleration				rpm/sec
#define C_POLI_MAX		(     36.0f     )	//  5 C_POLI	polar couples
#define I_START_MAX		(    200.0f     )	//  6 I_START	startup current				Ampere
#define I_MAX_MAX		(    200.0f     )	//  7 I_MAX		maximum current				Ampere
#define R_STA_MAX		(   1000.0f     )	//  8 R_STA		stator resistance			Ohm
#define L_SQ_MAX		(     10.0f     )	//  9 L_SQ		q synchronous inductance		Henry
#define PM_FLX_MAX		(     10.0f     )	// 10 PM_FLX	permanent magnets flux		Weber
#define KP_CUR_MAX		(  99999.0f     )	// 11 KP_CUR	current loop KP				Ohm
#define KI_CUR_MAX		( 999999.0f     )	// 12 KI_CUR	current loop KI (analog)	Ohm/sec
#define KP_VEL_MAX		(  99999.0f     )	// 13 KP_VEL	speed loop KP				Ampere*sec/rad
#define KI_VEL_MAX		( 999999.0f     )	// 14 KI_VEL	speed loop KI				Ampere/rad
#define FB_GAIN_MAX		(   1000.0f     )	// 15 FB_GAIN	flux amp. feedback gain
#define PHA_OFF_MAX		(    359.9999f  )	// 16 PHA_OFF	phase offset				deg
#define SUP_TIM_MAX		(    100.0f     )	// 17 SUP_TIM	startup acc. time			sec
#define FL_FTAU_MAX		(      1.0f     )	// 18 FL_FTAU	flux est. filter tau		sec
#define SAM_FRE_MAX		(  16000.0f     )	// 19 SAM_FRE	sampling frequency			Hz
#define F_RATIO_MAX		(      6.0f     )	// 20 F_RATIO	pwm_freq/sam_freq ratio

// Unused parameters defines
//#define PAR_00_MIN	(   0x00000000  )	// 00 PAR_00
#define PAR_01_MIN		(      0.0f     )	// 01 PAR_01	
#define PAR_02_MIN		(      0.0f     )	// 02 PAR_02	
#define PAR_03_MIN		(      0.0f     )	// 03 PAR_03	
#define PAR_04_MIN		(      0.0f     )	// 04 PAR_04	
#define PAR_05_MIN		(      0.0f     )	// 05 PAR_05	
#define PAR_06_MIN		(      0.0f     )	// 06 PAR_06	
#define PAR_07_MIN		(      0.0f     )	// 07 PAR_07	
#define PAR_08_MIN		(      0.0f     )	// 08 PAR_08	
#define PAR_09_MIN		(      0.0f     )	// 09 PAR_09	
#define PAR_10_MIN		(      0.0f     )	// 10 PAR_10	
#define PAR_11_MIN		(      0.0f     )	// 11 PAR_11	
#define PAR_12_MIN		(      0.0f     )	// 12 PAR_12	
#define PAR_13_MIN		(      0.0f     )	// 13 PAR_13	
#define PAR_14_MIN		(      0.0f     )	// 14 PAR_14	
#define PAR_15_MIN		(      0.0f     )	// 15 PAR_15	
#define PAR_16_MIN		(      0.0f     )	// 16 PAR_16	
#define PAR_17_MIN		(      0.0f     )	// 17 PAR_17	
#define PAR_18_MIN		(      0.0f     )	// 18 PAR_18	
#define PAR_19_MIN		(      0.0f     )	// 19 PAR_19	
#define PAR_20_MIN		(      0.0f     )	// 20 PAR_20	
#define PAR_21_MIN		(      0.0f     )	// 21 PAR_21	
#define PAR_22_MIN		(      0.0f     )	// 22 PAR_22	
#define PAR_23_MIN		(      0.0f     )	// 23 PAR_23	
#define PAR_24_MIN		(      0.0f     )	// 24 PAR_24	
#define PAR_25_MIN		(      0.0f     )	// 25 PAR_25	
#define PAR_26_MIN		(      0.0f     )	// 26 PAR_26    
#define PAR_27_MIN		(      0.0f     )	// 27 PAR_27	
#define PAR_28_MIN		(      0.0f     )	// 28 PAR_28	
#define PAR_29_MIN		(      0.0f     )	// 29 PAR_29	
#define PAR_30_MIN		(      0.0f     )	// 30 PAR_30	
#define PAR_31_MIN		(      0.0f     )	// 31 PAR_31	
//#define PAR_00_DEF	(   0x00000000  )	// 00 PAR_00
#define PAR_01_DEF		(   PAR_01_MIN  )	// 01 PAR_01	
#define PAR_02_DEF		(   PAR_02_MIN  )	// 02 PAR_02	
#define PAR_03_DEF		(   PAR_03_MIN  )	// 03 PAR_03	
#define PAR_04_DEF		(   PAR_04_MIN  )	// 04 PAR_04	
#define PAR_05_DEF		(   PAR_05_MIN  )	// 05 PAR_05	
#define PAR_06_DEF		(   PAR_06_MIN  )	// 06 PAR_06	
#define PAR_07_DEF		(   PAR_07_MIN  )	// 07 PAR_07	
#define PAR_08_DEF		(   PAR_08_MIN  )	// 08 PAR_08	
#define PAR_09_DEF		(   PAR_09_MIN  )	// 09 PAR_09	
#define PAR_10_DEF		(   PAR_10_MIN  )	// 10 PAR_10	
#define PAR_11_DEF		(   PAR_11_MIN  )	// 11 PAR_11	
#define PAR_12_DEF		(   PAR_12_MIN  )	// 12 PAR_12	
#define PAR_13_DEF		(   PAR_13_MIN  )	// 13 PAR_13	
#define PAR_14_DEF		(   PAR_14_MIN  )	// 14 PAR_14	
#define PAR_15_DEF		(   PAR_15_MIN  )	// 15 PAR_15	
#define PAR_16_DEF		(   PAR_16_MIN  )	// 16 PAR_16	
#define PAR_17_DEF		(   PAR_17_MIN  )	// 17 PAR_17	
#define PAR_18_DEF		(   PAR_18_MIN  )	// 18 PAR_18	
#define PAR_19_DEF		(   PAR_19_MIN  )	// 19 PAR_19	
#define PAR_20_DEF		(   PAR_20_MIN  )	// 20 PAR_20	
#define PAR_21_DEF		(   PAR_21_MIN  )	// 21 PAR_21	
#define PAR_22_DEF		(   PAR_22_MIN  )	// 22 PAR_22	
#define PAR_23_DEF		(   PAR_23_MIN  )	// 23 PAR_23	
#define PAR_24_DEF		(   PAR_24_MIN  )	// 24 PAR_24	
#define PAR_25_DEF		(   PAR_25_MIN  )	// 25 PAR_25	
#define PAR_26_DEF		(   PAR_26_MIN  )	// 26 PAR_26    
#define PAR_27_DEF		(   PAR_27_MIN  )	// 27 PAR_27	
#define PAR_28_DEF		(   PAR_28_MIN  )	// 28 PAR_28	
#define PAR_29_DEF		(   PAR_29_MIN  )	// 29 PAR_29	
#define PAR_30_DEF		(   PAR_30_MIN  )	// 30 PAR_30	
#define PAR_31_DEF		(   PAR_31_MIN  )	// 31 PAR_31	
//#define PAR_00_MAX	(   0xFFFFFFFF  )	// 00 PAR_00
#define PAR_01_MAX		( 999999.99999f )	// 01 PAR_01	
#define PAR_02_MAX		( 999999.99999f )	// 02 PAR_02	
#define PAR_03_MAX		( 999999.99999f )	// 03 PAR_03	
#define PAR_04_MAX		( 999999.99999f )	// 04 PAR_04	
#define PAR_05_MAX		( 999999.99999f )	// 05 PAR_05	
#define PAR_06_MAX		( 999999.99999f )	// 06 PAR_06	
#define PAR_07_MAX		( 999999.99999f )	// 07 PAR_07	
#define PAR_08_MAX		( 999999.99999f )	// 08 PAR_08	
#define PAR_09_MAX		( 999999.99999f )	// 09 PAR_09	
#define PAR_10_MAX		( 999999.99999f )	// 10 PAR_10	
#define PAR_11_MAX		( 999999.99999f )	// 11 PAR_11	
#define PAR_12_MAX		( 999999.99999f )	// 12 PAR_12	
#define PAR_13_MAX		( 999999.99999f )	// 13 PAR_13	
#define PAR_14_MAX		( 999999.99999f )	// 14 PAR_14	
#define PAR_15_MAX		( 999999.99999f )	// 15 PAR_15	
#define PAR_16_MAX		( 999999.99999f )	// 16 PAR_16	
#define PAR_17_MAX		( 999999.99999f )	// 17 PAR_17	
#define PAR_18_MAX		( 999999.99999f )	// 18 PAR_18	
#define PAR_19_MAX		( 999999.99999f )	// 19 PAR_19	
#define PAR_20_MAX		( 999999.99999f )	// 20 PAR_20	
#define PAR_21_MAX		( 999999.99999f )	// 21 PAR_21	
#define PAR_22_MAX		( 999999.99999f )	// 22 PAR_22	
#define PAR_23_MAX		( 999999.99999f )	// 23 PAR_23	
#define PAR_24_MAX		( 999999.99999f )	// 24 PAR_24	
#define PAR_25_MAX		( 999999.99999f )	// 25 PAR_25	
#define PAR_26_MAX		( 999999.99999f )	// 26 PAR_26    
#define PAR_27_MAX		( 999999.99999f )	// 27 PAR_27	
#define PAR_28_MAX		( 999999.99999f )	// 28 PAR_28	
#define PAR_29_MAX		( 999999.99999f )	// 29 PAR_29	
#define PAR_30_MAX		( 999999.99999f )	// 30 PAR_30	
#define PAR_31_MAX		( 999999.99999f )	// 31 PAR_31	

#endif	// _PAR_DEFINES_H

// EOF par_defines.h

