
/*******************************************************************************
DISCLAIMER

THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.

WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
INTELLECTUAL PROPERTY INFRINGIMENT.

TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.

BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
THIS SOFTWARE.
BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.

IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
File:		parameters.h
Project:	S5D9_MotorControl
Revision:	1.0
Date:		01-06-16 (DD-MM-YYYY)
Author:		Andrea Ghirardello
Contents:	Preliminary parameter definition file
*******************************************************************************/

#ifndef	_PARAMETERS_H
#define _PARAMETERS_H

#include "globdef.h"
#include "par_defines.h"	// minimimum and maximum values macros
#include "defpar.h"			// default values macros (can be automatically generated by the GUI)

typedef union
{
	float32_t	f32;
	uint32_t	u32;
}	param_t;

// IMPORTANT NOTE: depending on the parameters type and of the ram_tab elements type (16 or 32 bits),
// to comply with the protocol requirements, the following relations MUST be verified:
// if type is 32bit then: NUM_PAR<MAX_NLD, N_RAM_READ<MAX_NLD, N_RAM_WRITE<MAX_NLD
// if type is 16bit then: NUM_PAR<MAX_NWD, N_RAM_READ<MAX_NWD, N_RAM_WRITE<MAX_NWD
// in every case: MAX_SAM_CNT<MAX_NWD

#define NUM_PAR 		( 32 )		// parameters number

#define RIP_DEF 		( 33 )		// default parameters setting password
#define DEB_OPN			( 55 )		// open loop operation debug mode password
#define DCCUR_S			( 66 )		// dc current vector setting debug mode password

#define DEB_OPNLP		(DEB_OPN == SEL_OP)
#define DCCUR_SET		(DCCUR_S == SEL_OP)

#define SEL_OP			(p_vec[ 0]).u32	// 00 operation select
#define RPM_MIN			(p_vec[ 1]).f32	// 01 minimum speed				rpm
#define RPM_MAX			(p_vec[ 2]).f32	// 02 maximum speed				rpm
#define R_ACC			(p_vec[ 3]).f32	// 03 acceleration				rpm/sec
#define R_DEC			(p_vec[ 4]).f32	// 04 deceleration				rpm/sec
#define C_POLI			(p_vec[ 5]).f32	// 05 polar couples
#define I_START			(p_vec[ 6]).f32	// 06 startup current			Ampere
#define I_MAX			(p_vec[ 7]).f32	// 07 maximum current			Ampere
#define R_STA			(p_vec[ 8]).f32	// 08 stator resistance			Ohm
#define L_SQ			(p_vec[ 9]).f32	// 09 synchronous inductance	Henry
#define PM_FLX			(p_vec[10]).f32	// 10 permanent magnets flux	Weber
#define KP_CUR			(p_vec[11]).f32	// 11 current loop KP			Ohm
#define KI_CUR			(p_vec[12]).f32	// 12 current loop KI (analog)	Ohm/sec
#define KP_VEL			(p_vec[13]).f32	// 13 speed loop KP				Ampere*sec/rad
#define	KI_VEL			(p_vec[14]).f32	// 14 speed loop KI				Ampere/rad
#define	FB_GAIN			(p_vec[15]).f32	// 15 flux amp. feedback gain
#define PHA_OFF			(p_vec[16]).f32	// 16 phase offset				deg
#define	SUP_TIM			(p_vec[17]).f32	// 17 startup acc. time			sec
#define FL_FTAU			(p_vec[18]).f32	// 18 flux est. filter tau		sec
#define SAM_FRE			(p_vec[19]).f32	// 19 sampling frequency		Hz
#define F_RATIO			(p_vec[20]).f32	// 20 pwm_freq/sam_freq ratio
#define PAR_21			(p_vec[21]).f32	// 21
#define PAR_22			(p_vec[22]).f32	// 22
#define PAR_23			(p_vec[23]).f32	// 23
#define PAR_24			(p_vec[24]).f32	// 24
#define PAR_25			(p_vec[25]).f32	// 25
#define PAR_26			(p_vec[26]).f32	// 26
#define PAR_27			(p_vec[27]).f32	// 27
#define PAR_28			(p_vec[28]).f32	// 28
#define PAR_29			(p_vec[29]).f32	// 29
#define PAR_30			(p_vec[30]).f32	// 30
#define PAR_31			(p_vec[31]).f32	// 31

#define SEL_OP_CPY		(p_cpy[ 0]).u32	// default parameters setting
#define RPM_MIN_CPY		(p_cpy[ 1]).f32	// minimum speed			rpm
#define RPM_MAX_CPY		(p_cpy[ 2]).f32	// maximum speed			rpm
#define R_ACC_CPY		(p_cpy[ 3]).f32	// acceleration				rpm/s
#define R_DEC_CPY		(p_cpy[ 4]).f32	// deceleration				rpm/s
#define C_POLI_CPY		(p_cpy[ 5]).f32	// polar couples
#define I_START_CPY		(p_cpy[ 6]).f32	// startup current			Ampere/AMP_RES
#define I_MAX_CPY		(p_cpy[ 7]).f32	// maximum current			Ampere/AMP_RES
#define R_STA_CPY		(p_cpy[ 8]).f32	// stator resistance		Ohm/OHM_RES
#define L_SQ_CPY		(p_cpy[ 9]).f32	// synchronous inductance	Henry/HEN_RES
#define PM_FLX_CPY		(p_cpy[10]).f32	// permanent magnets flux	Weber/WEB_RES
#define KP_CUR_CPY		(p_cpy[11]).f32	// current loop KP
#define KI_CUR_CPY		(p_cpy[12]).f32	// current loop KI
#define KP_VEL_CPY		(p_cpy[13]).f32	// speed loop KP
#define	KI_VEL_CPY		(p_cpy[14]).f32	// speed loop KI
#define	FB_GAIN_CPY		(p_cpy[15]).f32	// flux amp. feedback gain
#define PHA_OFF_CPY		(p_cpy[16]).f32	// phase offset				deg
#define	SUP_TIM_CPY		(p_cpy[17]).f32	// startup acc. time		sec/SEC_RES
#define FL_FTAU_CPY	    (p_cpy[18]).f32	// flux est. filter tau		sec/SEC_RES
#define SAM_FRE_CPY		(p_cpy[19]).f32	// sampling frequency		Hz
#define F_RATIO_CPY		(p_cpy[20]).f32	// pwm_freq/sam_freq ratio
#define PAR_21_CPY		(p_cpy[21]).f32	//
#define PAR_22_CPY		(p_cpy[22]).f32	//
#define PAR_23_CPY		(p_cpy[23]).f32	//
#define PAR_24_CPY		(p_cpy[24]).f32	//
#define PAR_25_CPY		(p_cpy[25]).f32	//
#define PAR_26_CPY		(p_cpy[26]).f32	//
#define PAR_27_CPY		(p_cpy[27]).f32	//
#define PAR_28_CPY		(p_cpy[28]).f32	//
#define PAR_29_CPY		(p_cpy[29]).f32	//
#define PAR_30_CPY		(p_cpy[30]).f32	//
#define PAR_31_CPY		(p_cpy[31]).f32	//

#ifdef _GES_EQP_C
const param_t
	p_min[NUM_PAR] = {				// MINIMUM VALUES VECTOR
		{.u32 = 0UL},				//  0 SEL_OP	operation select (minimum, default and maximum values not considered)
		RPM_MIN_MIN,				//  1 RPM_MIN	minimum speed				rpm
		RPM_MAX_MIN,				//  2 RPM_MAX	maximum speed				rpm
		R_ACC_MIN,					//  3 R_ACC		acceleration				rpm/sec
		R_DEC_MIN,					//  4 R_DEC		deceleration				rpm/sec
		C_POLI_MIN,					//  5 C_POLI	polar couples
		I_START_MIN,				//  6 I_START	startup current				Ampere
		I_MAX_MIN,					//  7 I_MAX		maximum current				Ampere
		R_STA_MIN,					//  8 R_STA		stator resistance			Ohm
		L_SQ_MIN,					//  9 L_SQ		q synchronous inductance	Henry
		PM_FLX_MIN,					// 10 PM_FLX	permanent magnets flux		Weber
		KP_CUR_MIN,					// 11 KP_CUR	current loop KP				Ohm
		KI_CUR_MIN,					// 12 KI_CUR	current loop KI (analog)	Ohm/sec
		KP_VEL_MIN,					// 13 KP_VEL	speed loop KP				Ampere*sec/rad
		KI_VEL_MIN,					// 14 KI_VEL	speed loop KI				Ampere/rad
		FB_GAIN_MIN,				// 15 FB_GAIN	flux amp. feedback gain
		PHA_OFF_MIN,				// 16 PHA_OFF	phase offset				deg
		SUP_TIM_MIN,				// 17 SUP_TIM	startup acc. time			sec
		FL_FTAU_MIN,				// 18 FL_FTAU	flux est. filter tau		sec
		SAM_FRE_MIN,				// 19 SAM_FRE	sampling frequency			Hz
		F_RATIO_MIN,				// 20 F_RATIO	pwm_freq/sam_freq ratio
		PAR_21_MIN,					// 21 PAR_21	
		PAR_22_MIN,					// 22 PAR_22	
		PAR_23_MIN,					// 23 PAR_23	
		PAR_24_MIN,					// 24 PAR_24	
		PAR_25_MIN,					// 25 PAR_25	
		PAR_26_MIN,					// 26 PAR_26    
		PAR_27_MIN,					// 27 PAR_27	
		PAR_28_MIN,					// 28 PAR_28	
		PAR_29_MIN,					// 29 PAR_29	
		PAR_30_MIN,					// 30 PAR_30	
		PAR_31_MIN},				// 31 PAR_31	
	p_def[NUM_PAR] = {				// DEFAULT VALUES VECTOR
		{.u32 = 0UL},				//  0 SEL_OP	operation select (minimum, default and maximum values not considered)
		RPM_MIN_DEF,				//  1 RPM_MIN	minimum speed				rpm
		RPM_MAX_DEF,				//  2 RPM_MAX	maximum speed				rpm
		R_ACC_DEF,					//  3 R_ACC		acceleration				rpm/sec
		R_DEC_DEF,					//  4 R_DEC		deceleration				rpm/sec
		C_POLI_DEF,					//  5 C_POLI	polar couples
		I_START_DEF,				//  6 I_START	startup current				Ampere
		I_MAX_DEF,					//  7 I_MAX		maximum current				Ampere
		R_STA_DEF,					//  8 R_STA		stator resistance			Ohm
		L_SQ_DEF,					//  9 L_SQ		q synchronous inductance	Henry
		PM_FLX_DEF,					// 10 PM_FLX	permanent magnets flux		Weber
		KP_CUR_DEF,					// 11 KP_CUR	current loop KP				Ohm
		KI_CUR_DEF,					// 12 KI_CUR	current loop KI (analog)	Ohm/sec
		KP_VEL_DEF,					// 13 KP_VEL	speed loop KP				Ampere*sec/rad
		KI_VEL_DEF,					// 14 KI_VEL	speed loop KI				Ampere/rad
		FB_GAIN_DEF,				// 15 FB_GAIN	flux amp. feedback gain
		PHA_OFF_DEF,				// 16 PHA_OFF	phase offset				deg
		SUP_TIM_DEF,				// 17 SUP_TIM	startup acc. time			sec
		FL_FTAU_DEF,				// 18 FL_FTAU	flux est. filter tau		sec
		SAM_FRE_DEF,				// 19 SAM_FRE	sampling frequency			Hz
		F_RATIO_DEF,				// 20 F_RATIO	pwm_freq/sam_freq ratio
		PAR_21_DEF,					// 21 PAR_21	
		PAR_22_DEF,					// 22 PAR_22	
		PAR_23_DEF,					// 23 PAR_23	
		PAR_24_DEF,					// 24 PAR_24	
		PAR_25_DEF,					// 25 PAR_25	
		PAR_26_DEF,					// 26 PAR_26    
		PAR_27_DEF,					// 27 PAR_27	
		PAR_28_DEF,					// 28 PAR_28	
		PAR_29_DEF,					// 29 PAR_29	
		PAR_30_DEF,					// 30 PAR_30	
		PAR_31_DEF},				// 31 PAR_31	
	p_max[NUM_PAR] = {				// MAXIMUM VALUES VECTOR
		{.u32 = 0xFFFFFFFF},		//  0 SEL_OP	operation select (minimum, default and maximum values not considered)
		RPM_MIN_MAX,				//  1 RPM_MIN	minimum speed				rpm
		RPM_MAX_MAX,				//  2 RPM_MAX	maximum speed				rpm
		R_ACC_MAX,					//  3 R_ACC		acceleration				rpm/sec
		R_DEC_MAX,					//  4 R_DEC		deceleration				rpm/sec
		C_POLI_MAX,					//  5 C_POLI	polar couples
		I_START_MAX,				//  6 I_START	startup current				Ampere
		I_MAX_MAX,					//  7 I_MAX		maximum current				Ampere
		R_STA_MAX,					//  8 R_STA		stator resistance			Ohm
		L_SQ_MAX,					//  9 L_SQ		q synchronous inductance	Henry
		PM_FLX_MAX,					// 10 PM_FLX	permanent magnets flux		Weber
		KP_CUR_MAX,					// 11 KP_CUR	current loop KP				Ohm
		KI_CUR_MAX,					// 12 KI_CUR	current loop KI (analog)	Ohm/sec
		KP_VEL_MAX,					// 13 KP_VEL	speed loop KP				Ampere*sec/rad
		KI_VEL_MAX,					// 14 KI_VEL	speed loop KI				Ampere/rad
		FB_GAIN_MAX,				// 15 FB_GAIN	flux amp. feedback gain
		PHA_OFF_MAX,				// 16 PHA_OFF	phase offset				deg
		SUP_TIM_MAX,				// 17 SUP_TIM	startup acc. time			sec
		FL_FTAU_MAX,				// 18 FL_FTAU	flux est. filter tau		sec
		SAM_FRE_MAX,				// 19 SAM_FRE	sampling frequency			Hz
		F_RATIO_MAX,				// 20 F_RATIO	pwm_freq/sam_freq ratio
		PAR_21_MAX,					// 21 PAR_21	
		PAR_22_MAX,					// 22 PAR_22	
		PAR_23_MAX,					// 23 PAR_23	
		PAR_24_MAX,					// 24 PAR_24	
		PAR_25_MAX,					// 25 PAR_25	
		PAR_26_MAX,					// 26 PAR_26    
		PAR_27_MAX,					// 27 PAR_27	
		PAR_28_MAX,					// 28 PAR_28	
		PAR_29_MAX,					// 29 PAR_29	
		PAR_30_MAX,					// 30 PAR_30	
		PAR_31_MAX};				// 31 PAR_31
param_t
	p_vec[NUM_PAR],					// parameters vector
	p_cpy[NUM_PAR];					// parameters vector copy
#else   // ifdef _GES_EQP_C
extern const param_t
	p_min[NUM_PAR],
	p_max[NUM_PAR],
	p_def[NUM_PAR];
extern param_t
	p_vec[NUM_PAR],
    p_cpy[NUM_PAR];					// parameters vector copy
#endif  // else ifdef _GES_EQP_C
#endif	// _PARAMETERS_H

// EOF parameters.h
