
/*******************************************************************************
DISCLAIMER

THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.

WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
INTELLECTUAL PROPERTY INFRINGIMENT.

TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.

BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
THIS SOFTWARE.
BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.

IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
File:		pws_EBR_S5D9_0.h
Project:	S5D9_MotorControl
Revision:	1.0
Date:		01-06-16 (DD-MM-YYYY)
Author:		Andrea Ghirardello
Contents:	Internal power stage related definitions
*******************************************************************************/

#ifndef	_PWS_EBR_S5D9_0_H
#define _PWS_EBR_S5D9_0_H

//#include "typedefine.h"

// current conversion: reading of shunt voltage (VIN) through a resistance (RIN);
// pin is also connected to an offset (VOFF) through a resistance (ROFF);
// internal amplifier gain is K;
// VOUT=K*(ROFF*VIN+RIN*VOFF)/(RIN+ROFF)=K*VIN*ROFF/(RIN+ROFF)+K*VOFF*RIN/(RIN+ROFF)=GIN*VIN+GOFF*VOFF
// if we want GIN=2.8, GOFF=0.5, we get: GIN/GOFF=5.6->ROFF/RIN=5.6, 2.8=K*5.6/6.6->K=3.3
// the nearest possible choice for K is 3.333, and keeping RIN=220R, ROFF=1K21 we get: GIN=2.82023, GOFF=0.51277
// with K=3.077, and keeping RIN=220R, ROFF=1K21 we would get: GIN=2.6036, GOFF=0.4734
// since VOFF=ADVCC, internal offset will be ADFS*GOFF, while the total current gain will be RHUNT_OHM*GIN*ADFS/ADVCC
// sign will be reversed
// value to set in the gain register to obtain K=3.333 is 0101b=0x05 (see hardware manual)
#define PGA_GAIN			( 0x05 )
#define RSHUNT_OHM			(    0.1 )
#define RSGAIN				(   -2.82023 )

// DC bus voltage conversion
#define RVBUSH_OHM		(    82000.0 )		// high side split resistor [Ohm]
#define RVBUSL_OHM		(     6800.0 )		// low side split resistor [Ohm]

// deadtimes to be inserted by MTU
#define DEADTIM_NS			( 1000 )		// deatimes [ns]

// minimum on time of low arms (one half of), to guarantee good current reading
// this time should guarantee that the lower switch commutates effectively for enough time to
// sample the current for A/D conversion (added to deadtimes, it gives the minimum duty-cycle value)
#define MIN_TON_LOW_NS		(  300 )		// [ns]

// current conversion delay respect pwm trough
#define CON_DELAY_NS		(   30 )		// [ns] lesser than MIN_TON_LOW

// voltages
#define VB_NOM_VOL			(   24 )		// 220 * sqrt(2) nominal bus voltage [V]

#endif	// _PWS_EBR_S5D9_0_H

// EOF pws_EBR_S5D9_0.h
