
/*******************************************************************************
DISCLAIMER

THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.

WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
INTELLECTUAL PROPERTY INFRINGIMENT.

TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.

BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
THIS SOFTWARE.
BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.

IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
File:		pws_PS60A.h
Project:	YRotateIt-RX23T (three shunts)
Revision:	2.0
Date:		01-06-16 (DD-MM-YYYY)
Author:		Andrea Ghirardello
Contents:	External power stage (PS60A) related definitions
*******************************************************************************/

#ifndef	_PWS_PS60A_H
#define _PWS_PS60A_H

//#include "typedefine.h"

// current conversion
// NOTE: the current is considered positive if going out from the inverter
#define PGA_GAIN			( 0x05 )		// NOT USED
#define RATIOX				(2000.0 / (1000.0 + 2000.0))	// interface partition
#define RSHUNT_OHM			(     0.001 )		// shunt value [Ohm]
#define RSGAIN					(31.0 * RATIOX)	// circuit gain
//#define RSHUNT_OHM			(0.625 / 20)	// LEM gain HXS 20-NP
//#define RSHUNT_OHM			(0.625 / 50)	// LEM gain HXS 50-NP
//#define RSGAIN				(RATIOX)		// circuit gain if LEM used

// DC bus voltage conversion
#define RVBUSH_OHM			( 82000.0 )		// high side split resistor [Ohm]
//#define RVBUSL_OHM			(  6200.0 )		// low side split resistor [Ohm]
#define RVBUSL_OHM			((6200.0 * 10000.0) / (6200.0 + 10000.0))		// low side split resistor [Ohm]

// deadtimes to be inserted by MTU
#define DEADTIM_NS			(  1000 )		// deatimes [ns]

// minimum on time of low arms (one half of), to guarantee good current reading
// this time should guarantee that the lower switch commutates effectively for enough time to
// sample the current for A/D conversion (added to deadtimes, it gives the minimum duty-cycle value)
#define MIN_TON_LOW_NS		(  300 )		// [ns]

// current conversion delay respect pwm trough
#define CON_DELAY_NS		(   30 )		// [ns] lesser than MIN_TON_LOW

// voltages
#define VB_NOM_VOL			(   48 )		// nominal bus voltage [V]

#endif	// _PWS_PS60A_H

// EOF pws_PS60A.h
