
/*******************************************************************************
DISCLAIMER

THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.

WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
INTELLECTUAL PROPERTY INFRINGIMENT.

TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.

BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
THIS SOFTWARE.
BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.

IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
File:		gpt_pwm_defs.c
Project:	S5D9_MotorControl
Revision:	1.0
Date:		01-05-17 (DD-MM-YYYY)
Author:		Andrea Ghirardello
Contents:	pwm timer on gpt structures define
*******************************************************************************/

#define _GPT_PWM_DEFS_C	// insert define BEFORE including header files


#include "gpt_pwm.h"

static mc_timer_ctrl_t
	g_mc_timer0_ctrl,
	g_mc_timer1_ctrl;

	static const gpt_pwm_out_cfg_t g_mc_timer0_extend =
{
	.gtioca =
	{
		.output_enabled		= false,
		.stop_level			= GPT_PWM_PIN_LEVEL_LOW,
		.match_level		= GPT_PWM_OUTPUT_TOGGLED,
		.cycle_end_level	= GPT_PWM_OUTPUT_RETAINED,
		.initial_level		= GPT_PWM_INITIAL_OUTPUT_LOW
	},
	.gtiocb =
	{
		.output_enabled		= false,
		.stop_level			= GPT_PWM_PIN_LEVEL_LOW,
		.match_level		= GPT_PWM_OUTPUT_TOGGLED,
		.cycle_end_level	= GPT_PWM_OUTPUT_RETAINED,
		.initial_level		= GPT_PWM_INITIAL_OUTPUT_HIGH
	}
};

static const gpt_pwm_out_cfg_t g_mc_timer1_extend =
{
	.gtioca =
	{
		.output_enabled		= false,
		.stop_level			= GPT_PWM_PIN_LEVEL_LOW,
		.match_level		= GPT_PWM_OUTPUT_TOGGLED,
		.cycle_end_level	= GPT_PWM_OUTPUT_RETAINED,
		.initial_level		= GPT_PWM_INITIAL_OUTPUT_LOW
	},
	.gtiocb =
	{
		.output_enabled		= false,
		.stop_level			= GPT_PWM_PIN_LEVEL_LOW,
		.match_level		= GPT_PWM_OUTPUT_TOGGLED,
		.cycle_end_level	= GPT_PWM_OUTPUT_RETAINED,
		.initial_level		= GPT_PWM_INITIAL_OUTPUT_HIGH
	}
};

/* the following are only a dummy initialization; the configuration should be updated
 * following user parameters before opening the timer
 */
mc_timer_par_cfg_t g_mc_timer0_par_cfg =
{
	.fre_ratio	= 1,
    .halfperiod = 5000,
    .deadtime	= 300,
	.con_delay	= 30
};

mc_timer_par_cfg_t g_mc_timer1_par_cfg =
{
	.fre_ratio	= 1,
    .halfperiod = 5000,
    .deadtime	= 300,
	.con_delay	= 30
};

mc_timer_cfg_t g_mc_timer0_cfg =
{
	.channels		= { 1, 2, 3 },
	.p_param		= &g_mc_timer0_par_cfg,
	.p_extend		= &g_mc_timer0_extend
};

mc_timer_cfg_t g_mc_timer1_cfg =
{
	.channels		= { 4, 5, 7 },
	.p_param		= &g_mc_timer1_par_cfg,
	.p_extend		= &g_mc_timer1_extend
};

// Instance structure to use this module
const mc_timer_instance_t g_mc_timer0 =
{
     .p_ctrl	= &g_mc_timer0_ctrl,
     .p_cfg		= &g_mc_timer0_cfg,
     .p_api		= &g_pwm_timer_on_gpt
};

const mc_timer_instance_t g_mc_timer1 =
{
     .p_ctrl	= &g_mc_timer1_ctrl,
     .p_cfg		= &g_mc_timer1_cfg,
     .p_api		= &g_pwm_timer_on_gpt
};

