
/*******************************************************************************
DISCLAIMER

THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.

WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
INTELLECTUAL PROPERTY INFRINGIMENT.

TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.

BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
THIS SOFTWARE.
BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.

IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
File:		mc_timer_api.h
Project:	S5D9_MotorControl
Revision:	1.0
Date:		01-05-17 (DD-MM-YYYY)
Author:		Andrea Ghirardello
Contents:	PWM timer API;  allows for pwm generation for motor control
*******************************************************************************/

#ifndef DRV_MC_TIMER_API_H
#define DRV_MC_TIMER_API_H


/***********************************************************************************************************************
 * Includes
 **********************************************************************************************************************/

/* Includes board and MCU related header files. */
#include "r_gpt.h"
#include "bsp_api.h"


/**********************************************************************************************************************
 * Macro definitions
 **********************************************************************************************************************/

/* Leading zeroes removed to avoid coding standard violation. */
#define MC_TIMER_API_VERSION_MAJOR ((uint8_t)1)
#define MC_TIMER_API_VERSION_MINOR ((uint8_t)0)


/**********************************************************************************************************************
 * Typedef definitions
 **********************************************************************************************************************/

typedef struct st_mc_timer_instance_ctrl
{
	void			*	p_reg;        ///< Base register for this channel
	uint8_t				channel;      ///< Channel number.
	timer_variant_t		variant;      ///< Timer variant
} mc_timer_instance_ctrl_t;

typedef union un_mc_timer_ctrl
{
	struct st_channel_instances
	{
		mc_timer_instance_ctrl_t	u;					// gpt_instance_ctrl_t defined in r_gpt.h
		mc_timer_instance_ctrl_t	v;
		mc_timer_instance_ctrl_t	w;
	}								chn;
	mc_timer_instance_ctrl_t		channels[3];
}	mc_timer_ctrl_t;

typedef union
{
    timer_size_t		duties[3];					// timer_size_t defined in r_timer_api.h
	struct
	{
        timer_size_t	du;
        timer_size_t	dv;
        timer_size_t	dw;
	}	phases;
}	mc_timer_dutycycles;

typedef struct
{
	uint8_t			fre_ratio;
	timer_size_t	deadtime;
	timer_size_t	halfperiod;
	timer_size_t	con_delay;
}	mc_timer_par_cfg_t;

// User configuration structure, used in open function
typedef struct st_mc_timer_cfg
{
    uint8_t                     channels[3];	// Select the channels corresponding to the channel number of the hardware
    mc_timer_par_cfg_t          * p_param;		// User configuration parameters
    void           				* p_extend;     // Ouput pins specification
}	mc_timer_cfg_t;

// Timer API structure
typedef struct st_mc_timer_api
{
    ssp_err_t (* open)			(	mc_timer_ctrl_t			* const p_ctrl,
									mc_timer_cfg_t const	* const p_cfg);
    ssp_err_t (* stop)			(	mc_timer_ctrl_t			* const p_ctrl);
    ssp_err_t (* start)			(	mc_timer_ctrl_t			* const p_ctrl);
    ssp_err_t (* enableOut)		(	mc_timer_ctrl_t			* const p_ctrl);
    ssp_err_t (* disableOut)	(	mc_timer_ctrl_t			* const p_ctrl);
    ssp_err_t (* outEnabled)	(	mc_timer_ctrl_t			* const p_ctrl,
									bool					* enabled);
    ssp_err_t (* dutyCyclesSet)	(	mc_timer_ctrl_t			* const p_ctrl,
									mc_timer_dutycycles		* const duty_cycles);
    ssp_err_t (* dutyCyclesGet)	(	mc_timer_ctrl_t			* const p_ctrl,
									mc_timer_dutycycles		* const duty_cycles);
    ssp_err_t (* close)			(	mc_timer_ctrl_t			* const p_ctrl);
    ssp_err_t (* versionGet)	(	ssp_version_t			* const p_version);
}	mc_timer_api_t;

// This structure encompasses everything that is needed to use an instance of this interface
typedef struct st_mc_timer_instance
{
    mc_timer_ctrl_t       * p_ctrl;    ///< Pointer to the control structure for this instance
    mc_timer_cfg_t  const * p_cfg;     ///< Pointer to the configuration structure for this instance
    mc_timer_api_t  const * p_api;     ///< Pointer to the API structure for this instance
}	mc_timer_instance_t;


#endif // ifndef DRV_MC_TIMER_API_H

