
/*******************************************************************************
DISCLAIMER

THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.

WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
INTELLECTUAL PROPERTY INFRINGIMENT.

TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.

BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
THIS SOFTWARE.
BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.

IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
File:		customize.h
Project:	SK_S7G2_MotorControl
Revision:	1.0
Date:		01-06-16 (DD-MM-YYYY)
Author:		Andrea Ghirardello
Contents:	Custom values definition
*******************************************************************************/

#ifndef _CUSTOMIZE_H
#define _CUSTOMIZE_H

#include "typedefine.h"
#include "parameters.h"
//#include "pws_EBR_6140.h"
#include "pws_EBR_S5D9_0.h"

// eeprom (comment if not used)
#define EEPROM_USED

// debug (comment after first control)
//#define FIRST_DEBUG
//#define MACRO_DEBUG
//#define INTERRUPT_TIME_DEBUG    // do not uncomment at the same time of the next one
//#define MAINLOOP_TIME_DEBUG     // do not uncomment at the same time of the previous one

// open loop debug (comment in final release)
// if not commented it can be enabled writing 55 in par. 0
//#define DEBUG_OPENLOOP

// DC current vector setting debug mode (comment if not used)
// if not commented it can be enabled writing 66 in par. 0
//#define DCCUR_SETTING

// ADDITIONAL FEATURE: current PI manual tuning (comment if not used)
#define CPI_TUN

// ADDITIONAL FEATURE: current PI automatic tuning (comment if not used)
#define CPI_AUT

// ADDITIONAL FEATURE: motor parameters auto-identification (comment if not used)
#define MOT_IDENT

// ADDITIONAL FEATURE: oscilloscope window in GUI management (comment if not used)
#define OSC_WIN

// motor parameters default values include
#include "defpar.h"						// parameters limits and deafult values

// other definitions related to startup
#define STUP_CUR_RT_S		( 0.2f )	// current rising time during startup alignment [sec]
#define STUP_CUR_FT_S		( 1.0f )	// d current falling time after startup [sec] (assuming id=i_start)
#define ISTART_PH			( 0.523598775f )	// pi/6

// rotation direction
//#define ROT_REV

// current offsets refresh when motor not driven
#define OFF_REFRESH

// overmodulation (adjust to obtain overmodulation)
//#define K_OVERMOD			(  1.0f )	// 1 -> no overmod, 1.1547=2/sqrt(3) -> full hex sat, <1 to limit out vol
#define K_OVERMOD				(  1.1547f )// 1 -> no overmod, 1.1547=2/sqrt(3) -> full hex sat, <1 to limit out vol

// other control parameters
#define K_MARGV_DQ			(   0.9f )	// max_d_volt/max_volt ratio
#define MAXERR_RPM			( 300.0f )	// maximum error for ramp limitation and PI clamping

// speed PI error clamping
#define SPEEDPI_ERRCLAMP				// input to speed PI is clamped to MAXERR_RPM

// ramp clamp (comment if you don't want it)
//#define RAMP_CL0						// clamp if heavy speed error

// DC bus voltage limit for ramp limitation
//#define VB_RAMP						// comment this to avoid ramp limitation due to vbus
#define K_RAMPLIM			(   0.9f )	// ramp limitation gain
#define HVBUS_LIMH			((float32_t)(1.25 * VB_NOM_VOL))	// high level DC bus voltage limit [V]
#define HVBUS_LIML			((float32_t)(1.10 * VB_NOM_VOL))	// high level DC bus voltage limit reset [V]

// flux weakening
#define FLUXWEAK						// comment to avoid flux weakening
#define K_MARGV_FW			(   0.85f )	// percent of available voltage at which FW begins
#define K_MARGI_FW			(   0.8f )	// max_d_cur/max_cur ratio
#define DCUR_DT				(   2.0f )	// d current variation rate [A/s] during FW

// trip control
#define TRIP_CONTROL					// comment to avoid trip control
#define T_MARGL				(   0.5f )	// minimum speed trip (ratio of expected speed or of minimum speed)
#define T_MARGH				(   1.3f )	// maximum speed trip (ratio of expected speed or of maximum speed)
#define MAX_TRIP			( 500 )		// trip counter threshold

// speed low-pass filter cut-off frequency f0 (time constant will be tau=(1/w0)=(1/(2PIf0)))
#define	SPEED_LP_CUTOFF_HZ	( 100.0f )	// cut-off frequency [Hz]

// flux estimation method selection (default is exact bemf integration with amplitude feddback)
//#define APP_INT							// uncomment this if you prefer approximated integration

#endif	// _CUSTOMIZE_H

// EOF customize.h

