
/*******************************************************************************
DISCLAIMER

THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.

WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
INTELLECTUAL PROPERTY INFRINGIMENT.

TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.

BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
THIS SOFTWARE.
BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.

IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
File:		ges_eqp.c
Project:	YRotateIt-RX23T (three shunts)
Revision:	2.0
Date:		01-06-16 (DD-MM-YYYY)
Author:		Andrea Ghirardello
Contents:	Eeprom management functions
*******************************************************************************/

#define _GES_EQP_C			// insert define BEFORE including header files

#include "globdef.h"		// include globdef.h BEFORE including ges_eqp.h
#include "i_ges_eqp.h"

#define MAX_WAIT		( 50 )			// 50 ms
#define	NL_W_EQP		(  5 )			// 5*10ms=50ms max

#define	NUM_FIFO_EQP	(NUM_PAR + 2)	// eqp fifo dimension

param_t
    eqp_aux;

#ifdef EEPROM_USED

int16_t
	modif = -1;
uint16_t
	mas_ind,				// fifo master index
	sla_ind,				// fifo slave index
	eqp_all,
	eqp_wrt,
	eqp_bsy,
	eqp_add[NUM_FIFO_EQP];	// fifo of addresses of eeprom values to be updated
uint32_t
	eqp_fifo[NUM_FIFO_EQP];	// fifo of eeprom values to be updated
ioport_level_t
	sda_level;

/* management functions for iic eeprom ****************************************/

/* Hardware notes **************************************************************
	Since used micro's ports are input or push_pull outputs, to interface an
	IIC eeprom the following strategy is used:
	_	a port of the micro (EEPSCL) is used as clock output and is directly
		connected with the SCL input of the eeprom;
	_	the SDA I/O of the eeprom is connected to a pull-up resistor and to a
		port (EEPSDA) of the micro;
	_	to read a data bit or to write a HIGH data bit the EEPSDA port is
		configured as input.
	_	to write a LOW data bit the EEPSDA port is configured as output and
		it is forced to zero.
*******************************************************************************/

#define EEP_SDA			((ioport_port_pin_t)IOPORT_PORT_04_PIN_02)
#define SET_SDA_DIR_IN	{   g_ioport.p_api->pinDirectionSet(EEP_SDA, IOPORT_DIRECTION_INPUT);	}
#define SET_SDA_DIR_OUT	{   g_ioport.p_api->pinDirectionSet(EEP_SDA, IOPORT_DIRECTION_OUTPUT);	}
#define SET_SDA_OUT     {   g_ioport.p_api->pinWrite(EEP_SDA, IOPORT_LEVEL_HIGH);	}
#define RES_SDA_OUT     {   g_ioport.p_api->pinWrite(EEP_SDA, IOPORT_LEVEL_LOW);	}

#define EEP_SCL			((ioport_port_pin_t)IOPORT_PORT_04_PIN_03)
#define SET_SCL_DIR_IN	{   g_ioport.p_api->pinDirectionSet(EEP_SCL, IOPORT_DIRECTION_INPUT);	}
#define SET_SCL_DIR_OUT	{   g_ioport.p_api->pinDirectionSet(EEP_SCL, IOPORT_DIRECTION_OUTPUT);	}
#define SET_SCL_OUT     {   g_ioport.p_api->pinWrite(EEP_SCL, IOPORT_LEVEL_HIGH);	}
#define RES_SCL_OUT     {   g_ioport.p_api->pinWrite(EEP_SCL, IOPORT_LEVEL_LOW);	}
#define READ_SDA		{	g_ioport.p_api->pinRead(EEP_SDA, &sda_level);	}
#define	SDA_IN_H		(IOPORT_LEVEL_LOW != sda_level)
#define	SDA_IN_L		(IOPORT_LEVEL_LOW == sda_level)

#define	WAIT_SCL		DELAY2US		    // clock half period: minimum 1.25 microsec

#define	CODE_ADD_W		( 0xA0U )			// 1010=CODE 000=ADD 0=WRITE
#define	CODE_ADD_R		( 0xA1U )			// 1010=CODE 000=ADD 1=READ

// If the eeprom has a 2 bytes addressing please remove the comment in the following
// define, else if the eeprom has a 1 byte addressing please comment it
//#define	ADD2BYTES

// Routines are called with SCL=0 and SDA=1; they leave the same situation on exit

/*******************************************************************************
Function:		EQP_HwIni
Description:	hardware initialization
Input:			nothing
Output:			nothing
*******************************************************************************/
static void EQP_HwIni(void)
{
	// I/O settings
	SET_SCL_DIR_OUT
	RES_SCL_OUT
	SET_SDA_DIR_IN		// SET_SDA
}

/*******************************************************************************
Function:		EQP_SendStart
Description:	sends start sequence
Input:			nothing
Output:			nothing
*******************************************************************************/
static void EQP_SendStart(void)
{
	RES_SCL_OUT							// for safety
	SET_SDA_DIR_IN		// SET_SDA		// for safety
	WAIT_SCL
	SET_SCL_OUT
	WAIT_SCL
	SET_SDA_DIR_OUT		// configure SDA as output, to be able to clear it
	RES_SDA_OUT			// CLR_SDA
	WAIT_SCL
	RES_SCL_OUT
	SET_SDA_DIR_IN		// SET_SDA
}

/*******************************************************************************
Function:		EQP_SendStop
Description:	sends stop sequence
Input:			nothing
Output:			nothing
*******************************************************************************/
static void EQP_SendStop(void)
{
	RES_SCL_OUT			// for safety
	SET_SDA_DIR_OUT		// configure SDA as output, to be able to clear it
	RES_SDA_OUT			// CLR_SDA
	WAIT_SCL
	SET_SCL_OUT
	WAIT_SCL
	SET_SDA_DIR_IN		// SET_SDA
	WAIT_SCL
	RES_SCL_OUT
}

/*******************************************************************************
Function:		EQP_SendByte
Description:	sends a byte
Input:			byte to send
Output:			=0 -> OK, !=0 -> NOK
*******************************************************************************/
static uint16_t EQP_SendByte(uint8_t byt)
{
	uint8_t i;

	RES_SCL_OUT			// for safety
	for (i = 0; i < 8; i++)
	{
		if (byt & 0x80)
		{
			SET_SDA_DIR_IN		// SET_SDA
		}
		else
		{
			SET_SDA_DIR_OUT		// configure SDA as output, to be able to clear it
			RES_SDA_OUT			// CLR_SDA
		}
		WAIT_SCL
		SET_SCL_OUT
		WAIT_SCL
		RES_SCL_OUT
		byt = byt << 1;
	}
	SET_SDA_DIR_IN		// SET_SDA
	WAIT_SCL
	SET_SCL_OUT
	WAIT_SCL
	READ_SDA
	if (SDA_IN_L)
	{
		RES_SCL_OUT
		return (0);
	}
	else
	{
		RES_SCL_OUT
		EQP_SendStop();
		return (1);
	}
}

/*******************************************************************************
Function:		EQP_ReadByte
Description:	receive a byte
Input:			ack status
Output:			read value
*******************************************************************************/
static uint8_t EQP_ReadByte(uint8_t ack)
{
	uint8_t i, msk, val;

	RES_SCL_OUT						// for safety
	SET_SDA_DIR_IN		// SET_SDA	// for safety
	msk = 0x80;
	val = 0;
	for (i = 0; i < 8; i++)
	{
		WAIT_SCL
		SET_SCL_OUT
		WAIT_SCL
		READ_SDA
		if (SDA_IN_H)
		{
			val |= msk;
		}
		RES_SCL_OUT
		msk = msk >> 1;
	}
	if (ack)
	{
		SET_SDA_DIR_OUT		// configure SDA as output, to be able to clear it
		RES_SDA_OUT			// CLR_SDA
	}
	else
	{
		SET_SDA_DIR_IN		// SET_SDA
	}
	WAIT_SCL
	SET_SCL_OUT
	WAIT_SCL
	RES_SCL_OUT
	SET_SDA_DIR_IN			// SET_SDA
	return (val);
}

/*******************************************************************************
Function:		EQP_WriteWord
Description:	write a word data in eeprom
Input:			eeprom address to be written to, value to write
Output:			=0 -> OK, !=0 -> NOK
*******************************************************************************/
static uint16_t EQP_WriteWord(uint16_t add, uint16_t val)
{
	uint8_t msb, lsb;

	if ((NUM_PAR - 1) < add)
	{
		return (1);
	}
	add = add << 1;				// word access
	EQP_SendStart();
	if (0 != EQP_SendByte(CODE_ADD_W))
	{
		return (1);
	}
	msb = add >> 8;				// MSB of address
	lsb = add & 0x00FF;			// LSB of address
#ifdef	ADD2BYTES
	if(0 != EQP_SendByte(msb))
	{
		return (1);
	}
#endif
	if (0 != EQP_SendByte(lsb))
	{
		return (1);
	}
	msb = val >> 8;				// MSB of value
	lsb = val & 0x00FF;			// LSB of value
	if (0 != EQP_SendByte(msb))
	{
		return (1);
	}
	if (0 != EQP_SendByte(lsb))
	{
		return (1);
	}
	EQP_SendStop();
	return (0);
}

/*******************************************************************************
Function:		EQP_ReadWord
Description:	read a word data from eeprom
Input:			eeprom address to be read to, address of the variable which will
				contain the read value
Output:			=0 -> OK, !=0 -> NOK
*******************************************************************************/
static uint16_t EQP_ReadWord(uint16_t add, uint16_t *val)
{
	uint16_t	result;
	uint8_t		msb, lsb;

	if ((NUM_PAR - 1) < add)
	{
		return (1);
	}
	add = add << 1;				// word access
	EQP_SendStart();
	if (0 != EQP_SendByte(CODE_ADD_W))
	{
		return (1);
	}
	msb = add >> 8;				// MSB of address
	lsb = add & 0x00FF;			// LSB of address
#ifdef	ADD2BYTES
	if (0 != EQP_SendByte(msb))
	{
		return (1);
	}
#endif
	if (0 != EQP_SendByte(lsb))
	{
		return (1);
	}
	EQP_SendStart();
	if (0 != EQP_SendByte(CODE_ADD_R))
	{
		return (1);
	}
	msb = EQP_ReadByte(1);			// MSB of value
	lsb = EQP_ReadByte(0);			// LSB of value
	EQP_SendStop();
	result = msb;
	result = result << 8;
	result = result | lsb;
	(*val) = result;
	return (0);
}

/*******************************************************************************
Function:		EQP_WriteLong
Description:	write a word data in eeprom
Input:			eeprom address to be written to, value to write
Output:			=0 -> OK, !=0 -> NOK
*******************************************************************************/
static uint16_t EQP_WriteLong(uint16_t add, uint32_t val)
{
	uint16_t	msw, lsw;
	uint8_t		msb, lsb;

	if ((NUM_PAR - 1) < add)
	{
		return (1);
	}
	add = add << 2;				// long access
	EQP_SendStart();
	if (0 != EQP_SendByte(CODE_ADD_W))
	{
		return (1);
	}
	msb = add >> 8;				// MSB of address
	lsb = add & 0x00FF;			// LSB of address
#ifdef	ADD2BYTES
	if(0 != EQP_SendByte(msb))
	{
		return (1);
	}
#endif
	if (0 != EQP_SendByte(lsb))
	{
		return (1);
	}
	msw = val >> 16;			// MSW of value
	lsw = val & 0x0000FFFF;		// LSW of value
	msb = msw >> 8;				// MSB of MSW
	if (0 != EQP_SendByte(msb))
	{
		return (1);
	}
	lsb = msw & 0x00FF;			// LSB of MSW
	if (0 != EQP_SendByte(lsb))
	{
		return (1);
	}
	msb = lsw >> 8;				// MSB of LSW
	if (0 != EQP_SendByte(msb))
	{
		return (1);
	}
	lsb = lsw & 0x00FF;			// LSB of LSW
	if (0 != EQP_SendByte(lsb))
	{
		return (1);
	}
	EQP_SendStop();
	return (0);
}

/*******************************************************************************
Function:		EQP_ReadLong
Description:	read a long data from eeprom
Input:			eeprom address to be read to, address of the variable which will
				contain the read value
Output:			=0 -> OK, !=0 -> NOK
*******************************************************************************/
static uint32_t EQP_ReadLong(uint16_t add, uint32_t *val)
{
	uint32_t	result;
	uint16_t	msw, lsw;
	uint8_t		msb, lsb;

	if ((NUM_PAR - 1) < add)
	{
		return (1);
	}
	add = add << 2;				// long access
	EQP_SendStart();
	if (0 != EQP_SendByte(CODE_ADD_W))
	{
		return (1);
	}
	msb = add >> 8;				// MSB of address
	lsb = add & 0x00FF;			// LSB of address
#ifdef	ADD2BYTES
	if (0 != EQP_SendByte(msb))
	{
		return (1);
	}
#endif
	if (0 != EQP_SendByte(lsb))
	{
		return (1);
	}
	EQP_SendStart();
	if (0 != EQP_SendByte(CODE_ADD_R))
	{
		return (1);
	}
	msb = EQP_ReadByte(1);			// MSB of MSW
	lsb = EQP_ReadByte(1);			// LSB of MSW
	msw = msb;
	msw = msw << 8;
	msw = msw | lsb;
	msb = EQP_ReadByte(1);			// MSB of LSW
	lsb = EQP_ReadByte(0);			// LSB of LSW
	lsw = msb;
	lsw = lsw << 8;
	lsw = lsw | lsb;
	EQP_SendStop();
	result = msw;
	result = result << 16;
	result = result | lsw;
	(*val) = result;
	return (0);
}

/*******************************************************************************
Function:		EQP_EqpBusy
Description:	test busy
Input:			nothing
Output:			=0 -> OK, !=0 -> busy or NOK
*******************************************************************************/
static uint16_t EQP_EqpBusy(void)
{
	EQP_SendStart();
	if (0 != EQP_SendByte(CODE_ADD_W))
	{
		return (1);
	}
	EQP_SendStop();
	return (0);
}

/* end of management functions for iic_eeprom *********************************/

/*******************************************************************************
Function:		GE_IniEqp
Description:	hardware init if needed
Input:			nothing
Output:			nothing
*******************************************************************************/
void GE_IniEqp(void)
{
	mas_ind = 0;
	sla_ind = 0;
	eqp_all = 0;
	eqp_wrt = 0;
	eqp_bsy = 0;
	EQP_HwIni();
}

/******************************************************************************
Function:		GE_DefToRam
Description:	copies default parameters into ram
Input:			nothing
Output:			=0 -> OK, !=0 -> NOK
Modifies:		internal parameters vector
******************************************************************************/
uint16_t GE_DefToRam(void)
{
	uint16_t i;

	(p_vec[0]).u32 = 0;
	for (i = 1; i < NUM_PAR; i++)
	{
/*	EQP_DEBUG
		if ((p_min[i]).u32 > (p_max[i]).u32)
		{
			(p_vec[i]).f32 = (p_def[i]).f32;
			eqp_all = 1;
		}
		else if ((p_def[i]).f32 < (p_min[i]).u32)
		{
			(p_vec[i]).f32 = (p_min[i]).u32;
			eqp_all = 1;
		}
		else if ((p_vec[i]).f32 > (p_max[i]).u32)
		{
			(p_vec[i]).f32 = (p_max[i]).u32;
			eqp_all = 1;
		}
EQP_DEBUG */
		(p_vec[i]).f32 = (p_def[i]).f32;
	}
	return (eqp_all = 0);
}

/******************************************************************************
Function:		GE_WaitBusy
Description:	wait if eeprom is busy, for a max time
Input:			max delay in milliseconds
Output:			=0 -> OK, !=0 -> NOK
******************************************************************************/
uint16_t GE_WaitBusy(uint16_t max_time_ms)
{
	if (eqp_all)
	{
		return (eqp_all);
	}
	while (EQP_EqpBusy())
	{
		if (0 != max_time_ms)
		{
			DELAY1MS(1);
			max_time_ms--;
		}
		else
		{
			return (eqp_all = 1);
		}
	}
	return (0);
}

/******************************************************************************
Function:		GE_RamToEqp
Description:	copies all the parameters from ram to eeprom
Input:			nothing
Output:			=0 -> OK, !=0 -> NOK
Modifies:		eeprom content
Note:			to be used out of main loop, due to the high delay
******************************************************************************/
uint16_t GE_RamToEqp(void)
{
	static uint16_t i;

	// wait if busy
	if (GE_WaitBusy(MAX_WAIT))
	{
		return (eqp_all);
	}

	// read/write cycle
	for (i = 0; i < NUM_PAR; i++)
	{
		if (EQP_ReadLong(i, &(eqp_aux.u32)))
		{
			return (eqp_all = 1);
		}
		if ((p_vec[i]).u32 != eqp_aux.u32)
		{
			if (EQP_WriteLong(i, (p_vec[i]).u32))
			{
				return (eqp_all = 1);
			}
			if (GE_WaitBusy(MAX_WAIT))
			{
				return (eqp_all);
			}
			if (EQP_ReadLong(i, &(eqp_aux.u32)))
			{
				return (eqp_all = 1);
			}
			if ((p_vec[i]).u32 != eqp_aux.u32)
			{
				return (eqp_all = 1);
			}
		}
	}
	return (eqp_all = 0);
}

/******************************************************************************
Function:		GE_EqpToRam
Description:	if par0!=RIP_DEF copies all parameters from eeprom to
				ram, else copies all default values in ram parameter
				table, then copies all the parameters from ram to eeprom
Input:			nothing
Output:			=0 -> OK, !=0 -> NOK
Modifies:		internal parameters vector or eeprom content
Note:			to be used out of main loop, due to the high delay
******************************************************************************/
uint16_t GE_EqpToRam(void)
{
	uint16_t i;

	// wait if busy
	if (GE_WaitBusy(MAX_WAIT))
	{
		return (eqp_all);
	}

	// par 0 reading
	if (EQP_ReadLong(0, &(eqp_aux.u32)))
	{
		return (eqp_all = 1);
	}

	// default values restoration if requested, or special modes reset
	if ((0xFFFFFFFF == eqp_aux.u32) || (RIP_DEF == eqp_aux.u32))
	{
		(p_vec[0]).u32 = 0;
		for (i = 1; i < NUM_PAR; i++)
		{
			(p_vec[i]).u32 = (p_def[i]).u32;
		}
		if (GE_RamToEqp())
		{
			return (eqp_all = 1);
		}
	}
	else if (0 != eqp_aux.u32)
	{
		eqp_aux.u32 = 0;
		if (EQP_WriteLong(0, eqp_aux.u32))
		{
			return (eqp_all = 1);
		}
		if (GE_WaitBusy(MAX_WAIT))
		{
			return (eqp_all);
		}
		if (EQP_ReadLong(0, &(eqp_aux.u32)))
		{
			return (eqp_all = 1);
		}
		if (0 != eqp_aux.u32)
		{
			return (eqp_all = 1);
		}
	}
	(p_vec[0]).u32 = eqp_aux.u32;

	// parameters reading
	for (i = 1; i < NUM_PAR; i++)
	{
		if (EQP_ReadLong(i, &(eqp_aux.u32)))
		{
			return (eqp_all = 1);
		}
		if ((p_min[i]).f32 > eqp_aux.f32)
		{
			return (eqp_all = 1);
		}
		if ((p_max[i]).f32 < eqp_aux.f32)
		{
			return (eqp_all = 1);
		}
		(p_vec[i]).f32 = eqp_aux.f32;
	}
	return (eqp_all);
}

/******************************************************************************
Function:		GE_UpdEqp
Description:	copies a modified parameter from ram to eeprom
Input:			nothing
Output:			=0 -> OK, !=0 -> NOK
Modifies:		eeprom content
Note:			to be called in main loop
******************************************************************************/
uint16_t GE_UpdEqp(void)
{
	if (0 <= modif)
	{
		if (0 < eqp_bsy)
		{
			if (EQP_EqpBusy())
			{
				eqp_bsy--;
				if (0 == eqp_bsy)
				{
					eqp_all = 1;		// too long busy
					modif = -1;			// finished
				}
				else
				{
										// still waiting
				}
			}
			else
			{
				eqp_bsy = 0;
				eqp_aux.u32 = 0;
				if (EQP_ReadLong(modif, &(eqp_aux.u32)))
				{
					eqp_all = 1;		// reading failed
				}
				if ((p_vec[modif]).u32 != eqp_aux.u32)
				{
					eqp_all = 1;		// still not equal -> writing failed
				}
				modif = -1;				// finished
			}
		}
		else if ( (0 == modif) || ( (NUM_PAR > modif) && (0 == eqp_all) ) )
		{
			eqp_aux.u32 = 0;
			if (EQP_ReadLong(modif, &(eqp_aux.u32)))
			{
				eqp_all = 1;			// reading failed
				modif = -1;				// finished
			}
			else if ((p_vec[modif]).u32 != eqp_aux.u32)
			{
				eqp_aux.u32 = (p_vec[modif]).u32;
				if (EQP_WriteLong(modif, eqp_aux.u32))
				{
					eqp_all = 1;		// writing failed
					modif = -1;			// finished
				}
				eqp_bsy = NL_W_EQP;		// init delay counter
			}
		}
	}
	return (eqp_all);
}

/******************************************************************************
Function:		GE_SetPar
Description:	insertion of a parameter in eqp fifo, to be updated
Input:			parameter value, parameter address in the vector
Output:			nothing
Modifies:		internal fifo
******************************************************************************/
void GE_SetPar(uint32_t val, uint16_t add)
{
	if(val != (p_vec[add]).u32)
	{
		eqp_fifo[mas_ind] = val;
		eqp_add[mas_ind++] = add;
		if (NUM_FIFO_EQP <= mas_ind)
		{
			mas_ind = 0;
		}
	}
	return;
}

/******************************************************************************
Function:		GE_EqpMan
Description:	eeprom parameters modification management
Input:			parameter value, parameter address in the vector
Output:			=0 -> OK, !=0 -> NOK
Modifies:		internal fifo, eeprom content
Note:			to be called in main loop
******************************************************************************/
uint16_t GE_EqpMan()
{
	static uint16_t	stato, eqpbusy1;
	uint16_t		indice;
	param_t			pnum;

#define	NOR_0	(  0 )
#define	MOD_0	( 10 )

	switch(stato)
	{
		case NOR_0:
			if (sla_ind != mas_ind)
			{
				if (0 == eqp_add[sla_ind])
				{
					(p_vec[0]).u32 = eqp_fifo[sla_ind];
					eqp_fifo[sla_ind] = 0;
					if ((0 == (p_vec[0]).u32) || (RIP_DEF == (p_vec[0]).u32))
					{
						modif = 0;
						eqpbusy1 = 4;
						stato = MOD_0;
					}
					else
					{
						// modif not touched, no eeprom operations
					}
				}
				else if (0 == eqp_all)
				{
					pnum.u32 = eqp_fifo[sla_ind];
					indice = eqp_add[sla_ind];
					eqp_fifo[sla_ind] = 0;
					eqp_add[sla_ind] = 0;
					if (((p_min[indice]).f32 <= pnum.f32) && ((p_max[indice]).f32 >= pnum.f32))
					{
						(p_vec[indice]).f32 = pnum.f32;
						modif = indice;
						eqpbusy1 = 4;
						stato = MOD_0;
					}
					else
					{
						// modif not touched, no eeprom operations
					}
				}
				else // (0 != eqp_all)
				{
					eqp_fifo[sla_ind] = 0;
					eqp_add[sla_ind] = 0;
				}
				if ((NUM_FIFO_EQP - 1) > sla_ind)
				{
					sla_ind++;
				}
				else
				{
					sla_ind = 0;
				}
			}
			break;
		case MOD_0:
			if (0 <= modif)
			{
				if (0 < eqpbusy1)
				{
					eqpbusy1--;
					break;
				}
				else
				{
					return (eqp_all = 1);
				}
			}
			// no break;
		default:
			stato = NOR_0;
			break;
	}

	// eeprom management
	return (GE_UpdEqp());

}

#else	// ifdef EEPROM_USED
void GE_IniEqp(void)
{
	return;
}
uint16_t GE_RamToEqp(void)
{
	return (0);
}
uint16_t GE_EqpToRam(void)
{
	return (0);
}
uint16_t GE_EqpMan(void)
{
	return (0);
}
uint16_t GE_DefToRam(void)
{
	uint16_t i;

	for (i = 0; i < NUM_PAR; i++)
	{
		(p_vec[i]).u32 = (p_def[i]).u32;
	}
	return (0);
}
void GE_SetPar(uint32_t dta, uint16_t add)
{
	if(0 == add)
	{
		(p_vec[0]).u32 = dta;
	}
	else
	{
		eqp_aux.u32 = dta;
		if (((p_min[add]).f32 > eqp_aux.f32) || ((p_max[add]).f32 < eqp_aux.f32))
		{
			return;
		}
		(p_vec[add]).f32 = eqp_aux.f32;
	}
}
#endif 	// else ifdef EEPROM_USED

// EOF ges_eqp.c

