
/*******************************************************************************
DISCLAIMER

THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.

WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
INTELLECTUAL PROPERTY INFRINGIMENT.

TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.

BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
THIS SOFTWARE.
BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.

IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
File:		globdef.h
Project:	S5D9_MotorControl
Revision:	1.0
Date:		01-06-16 (DD-MM-YYYY)
Author:		Andrea Ghirardello
Contents:	Global definitions
*******************************************************************************/

#ifndef	_GLOBDEF_H
#define	_GLOBDEF_H

// inclusions
#include <math.h>
#include "typedefine.h"
#include "mask.h"
#include "..\synergy\ssp\src\bsp\cmsis\Device\RENESAS\S5D9\Include\S5D9.h"
#include "hal_data.h"
#include "common_data.h"
#include "main_thread.h"
#include "const_def.h"
#include "gui_defs.h"
#include "parameters.h"

// Delay
#define US10			(   60 )
#define US100			(  700 )
#define US1000			( 7000 )
#define DELAY(X, Y)		{	volatile uint16_t x, y, z;			\
							for(x = ( X ); x > 0; x--) {		\
								for(y = ( Y ); y > 0; y--) {	\
									z++;	}	}	}
#define DELAY2US		DELAY(1, 1)
#define DELAY10US(X)	DELAY(X, US10)
#define DELAY100US(X)	DELAY(X, US100)
#define DELAY1MS(X)		DELAY(X, US1000)

#define MYONLYLED       ((ioport_port_pin_t)IOPORT_PORT_03_PIN_07)
#define LED_ON          ( IOPORT_LEVEL_HIGH )
#define LED_OFF         ( IOPORT_LEVEL_LOW )
#define TURN_ON_LED     {   g_ioport.p_api->pinWrite(MYONLYLED, LED_ON);    }
#define TURN_OFF_LED    {   g_ioport.p_api->pinWrite(MYONLYLED, LED_OFF);   }
#define TOGGLE_LED      {   ioport_level_t level;                           \
                            g_ioport.p_api->pinRead(MYONLYLED, &level);     \
                            if (LED_ON == level) { TURN_OFF_LED }           \
                            else                 { TURN_ON_LED }            }

#define LIM_OFF_OUT     ((ioport_port_pin_t)IOPORT_PORT_03_PIN_06)
#define LIM_OFF_ON      ( IOPORT_LEVEL_HIGH )
#define LIM_OFF_OFF     ( IOPORT_LEVEL_LOW )
#define SET_LIM_OFF     {   g_ioport.p_api->pinWrite(LIM_OFF_OUT, LIM_OFF_ON);	}
#define RES_LIM_OFF     {   g_ioport.p_api->pinWrite(LIM_OFF_OUT, LIM_OFF_OFF);	}

/*
	R_S12ADC0->ADPGACR = 0x9000;			// A/D Programmable Gain Amplifier Control Register (word access)
		// B15 write 1
		// B14, B13 write 0
		// B12 write 1
	R_S12ADC0->ADPGACR_b.P002GEN = 0;		// B11 enable amplifier gain setting for PGA002
	R_S12ADC0->ADPGACR_b.P002ENAMP = 0;		// B10 enable amplifier for PGA002
	R_S12ADC0->ADPGACR_b.P002SEL1 = 0;		// B09 enable transit through amplifier for PGA002
	R_S12ADC0->ADPGACR_b.P002SEL0 = 0;		// B08 enable bypassing amplifier for PGA002
	R_S12ADC0->ADPGACR_b.P001GEN = 0;		// B07 enable amplifier gain setting for PGA001
	R_S12ADC0->ADPGACR_b.P001ENAMP = 0;		// B06 enable amplifier for PGA001
	R_S12ADC0->ADPGACR_b.P001SEL1 = 0;		// B05 enable transit through amplifier for PGA001
	R_S12ADC0->ADPGACR_b.P001SEL0 = 0;		// B04 enable bypassing amplifier for PGA001
	R_S12ADC0->ADPGACR_b.P000GEN = 0;		// B03 enable amplifier gain setting for PGA000
	R_S12ADC0->ADPGACR_b.P000ENAMP = 0;		// B02 enable amplifier for PGA000
	R_S12ADC0->ADPGACR_b.P000SEL1 = 0;		// B01 enable transit through amplifier for PGA000
	R_S12ADC0->ADPGACR_b.P000SEL0 = 0;		// B00 enable bypassing amplifier for PGA000

	R_S12ADC0->ADPGAGS0 = 0x0000;			// A/D Programmable Gain Amplifier Gain Setting Register 0 (word access)
		// B15, .., B12 write 0
	R_S12ADC0->ADPGAGS0_b.P002GAIN = 0;		// B11, .., B08 gain setting bits for PGA002
	R_S12ADC0->ADPGAGS0_b.P001GAIN = 0;		// B07, .., B04 gain setting bits for PGA001
	R_S12ADC0->ADPGAGS0_b.P000GAIN = 0;		// B03, .., B00 gain setting bits for PGA000
*/

#define ENABLE_PGA_M0						{ R_S12ADC0->ADPGACR = 0x9EEE; R_S12ADC0->ADPGADCR0 = 0x0000; }
#define ENABLE_PGA0_M0						{ R_S12ADC0->ADPGACR |= 0x000E; R_S12ADC0->ADPGADCR0 &= 0x0FF0; }
#define ENABLE_PGA1_M0						{ R_S12ADC0->ADPGACR |= 0x00E0; R_S12ADC0->ADPGADCR0 &= 0x0F0F; }
#define ENABLE_PGA2_M0						{ R_S12ADC0->ADPGACR |= 0x0E00; R_S12ADC0->ADPGADCR0 &= 0x00FF; }
#define SET_PGA_GAIN_M0( X )				{ R_S12ADC0->ADPGAGS0_b.P000GAIN = R_S12ADC0->ADPGAGS0_b.P001GAIN = R_S12ADC0->ADPGAGS0_b.P002GAIN = ( X ); }
#define SET_PGA0_GAIN_M0( X )				{ R_S12ADC0->ADPGAGS0_b.P000GAIN = ( X ); }
#define SET_PGA1_GAIN_M0( X )				{ R_S12ADC0->ADPGAGS0_b.P001GAIN = ( X ); }
#define SET_PGA2_GAIN_M0( X )				{ R_S12ADC0->ADPGAGS0_b.P002GAIN = ( X ); }
#define BYPASS_PGA_M0                       { R_S12ADC0->ADPGACR = 0x9999; R_S12ADC0->ADPGADCR0 = 0x0000; }
#define BYPASS_PGA0_M0                      { R_S12ADC0->ADPGACR |= 0x0009; R_S12ADC0->ADPGADCR0 &= 0x0FF0; }
#define BYPASS_PGA1_M0                      { R_S12ADC0->ADPGACR |= 0x0090; R_S12ADC0->ADPGADCR0 &= 0x0F0F; }
#define BYPASS_PGA2_M0                      { R_S12ADC0->ADPGACR |= 0x0900; R_S12ADC0->ADPGADCR0 &= 0x00FF; }

#define BYPASS_PGA_M1                       { R_S12ADC1->ADPGACR = 0x9999; R_S12ADC0->ADPGADCR0 = 0x0000; }
#define BYPASS_PGA0_M1                      { R_S12ADC1->ADPGACR |= 0x0009; R_S12ADC0->ADPGADCR0 &= 0x0FF0; }
#define BYPASS_PGA1_M1                      { R_S12ADC1->ADPGACR |= 0x0090; R_S12ADC0->ADPGADCR0 &= 0x0F0F; }
#define BYPASS_PGA2_M1                      { R_S12ADC1->ADPGACR |= 0x0900; R_S12ADC0->ADPGADCR0 &= 0x00FF; }

#endif	// _GLOBDEF_H

// EOF globdef.h
