
/*******************************************************************************
DISCLAIMER

THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.

WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
INTELLECTUAL PROPERTY INFRINGIMENT.

TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.

BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
THIS SOFTWARE.
BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.

IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
File:		ges_eqp.h
Project:	YRotateIt-RX23T (three shunts)
Revision:	2.0
Date:		01-06-16 (DD-MM-YYYY)
Author:		Andrea Ghirardello
Contents:	Eeprom management functions
*******************************************************************************/

#ifndef	_GES_EQP_H
#define _GES_EQP_H

#include "parameters.h"

/*******************************************************************************
Function:		GE_IniEqp
Description:	hardware init if needed
Input:			nothing
Output:			nothing
*******************************************************************************/
void GE_IniEqp(void);

/******************************************************************************
Function:		GE_DefToRam
Description:	copies default parameters into ram
Input:			nothing
Output:			=0 -> OK, !=0 -> NOK
Modifies:		internal parameters vector
******************************************************************************/
uint16_t GE_DefToRam(void);

/******************************************************************************
Function:		GE_RamToEqp
Description:	copies all the parameters from ram to eeprom
Input:			nothing
Output:			=0 -> OK, !=0 -> NOK
Modifies:		eeprom content
Note:			to be used out of main loop, due to the high delay
******************************************************************************/
uint16_t GE_RamToEqp(void);

/******************************************************************************
Function:		GE_EqpToRam
Description:	if par0!=RIP_DEF copies all parameters from eeprom to
				ram, else copies all default values in ram parameter
				table, then copies all the parameters from ram to eeprom
Input:			nothing
Output:			=0 -> OK, !=0 -> NOK
Modifies:		internal parameters vector or eeprom content
Note:			to be used out of main loop, due to the high delay
******************************************************************************/
uint16_t GE_EqpToRam(void);

/******************************************************************************
Function:		GE_SetPar
Description:	insertion of a parameter in eqp fifo, to be updated
Input:			parameter value, parameter address in the vector
Output:			nothing
Modifies:		internal fifo
******************************************************************************/
void GE_SetPar(uint32_t val, uint16_t add);

/******************************************************************************
Function:		GE_EqpMan
Description:	eeprom parameters modification management
Input:			parameter value, parameter address in the vector
Output:			=0 -> OK, !=0 -> NOK
Modifies:		internal fifo, eeprom content
Note:			to be called in main loop
******************************************************************************/
uint16_t GE_EqpMan(void);

#endif	// _GES_EQP_H

// EOF ges_eqp.h

