
/*******************************************************************************
DISCLAIMER

THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.

WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
INTELLECTUAL PROPERTY INFRINGIMENT.

TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.

BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
THIS SOFTWARE.
BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.

IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
File:		i_motorcontrol.h
Project:	S5D9_MotorControl
Revision:	1.0
Date:		01-06-16 (DD-MM-YYYY)
Author:		Andrea Ghirardello
Contents:	Public interface header for motorcontrol.c
*******************************************************************************/

#ifndef _I_MOTORCONTROL_H
#define _I_MOTORCONTROL_H

#include "typedefine.h"

#ifdef MACRO_DEBUG
void MC_ConstDebug(void);
#endif	// MACRO_DEBUG

/******************************************************************************
Function:       MC_Debug
Description:    debug function
Input:          nothing
Output:         nothing
******************************************************************************/
void MC_Debug(void);

/******************************************************************************
Function:		MC_WaitSync
Description:	time synchronization control function
Input:			nothing
Output:			0 if main loop time is elapsed, 1 if not
Modifies:		internal status variable
******************************************************************************/
uint16_t MC_WaitSync(void);

/******************************************************************************
Function:       MC_SetOff
Description:    A/D init and current offsets measurement
Input:          nothing
Output:         nothing
Modifies:       current reading offsets, conversion constants
******************************************************************************/
void MC_SetOff(void);

#ifdef _MOTORCONTROL_C
/*******************************************************************************
Function:		MC_SetPwmConst
Description:	calculation of constants related to sampling (and pwm) freq.
Input:			nothing
Output:			nothing
Modifies:		many global variables related to sampling frequency and used as
				constants
*******************************************************************************/
void MC_SetPwmConst(void);

/******************************************************************************
Function:		MC_ADMan
Description:	management of triggered A/D conversions;
				three shunts current reading;
Input:			nothing
Output:			nothing
******************************************************************************/
void MC_ADMan(void);
#endif	// ifdef _MOTORCONTROL_C

/******************************************************************************
Function:		MC_StopOut
Description:	disable pwm outputs
Input:			nothing
Output:			nothing
******************************************************************************/
void MC_StopOut(void);

/******************************************************************************
Function:		MC_StartOut
Description:	enables pwm outputs
Input:			nothing
Output:			nothing
******************************************************************************/
void MC_StartOut(void);

/******************************************************************************
Function:       MC_IniPWM
Description:    This function initializes the GPT module
Input:          nothing
Output:         nothing
******************************************************************************/
void MC_IniPWM(void);

/******************************************************************************
Function: 		MC_StartPWM
Description: 	This function initializes the GPT module and the POEG module
Input:			nothing
Output:			nothing
******************************************************************************/
void MC_StartPWM(void);

/******************************************************************************
Function:		MC_PWMGen
Description:	output duties calc
Input:			nothing
Note:			uses global variables vuc, vvc and vwc (requested voltages),
				global variable IU, IV, IW (currents),
				global variable vbus (bus voltage)
Output:			nothing
Modifies:		global variables duty_u, duty_v, duty_w, vuc, vvc, vwc
******************************************************************************/
void MC_PWMGen(void);

/******************************************************************************
Function:		MC_VecComp
Description:	calculation of the second component of a vector (absolute value)
Input:			vector amplitude amp (greater than zero)
				first component cmp
Output:			second component
Note:			result is positive or zero
******************************************************************************/
float32_t MC_VecComp(float32_t amp, float32_t cmp);

/******************************************************************************
Function:		MC_IniPar
Description:	internal constants initialization, using eeprom values
Input:			nothing
Output:			nothing
Modifies:		many internal variables, using eeprom parameters values
******************************************************************************/
void MC_IniPar(void);

/******************************************************************************
Function:		MC_UpdPar
Description:	internal constants update, if parameters have been changed
Input:			nothing
Output:			nothing
Modifies:		many internal variables
******************************************************************************/
void MC_UpdPar(void);

/******************************************************************************
Function:		MC_ConFil
Description:	management of measured values, to give them to the GUI
Input:			nothing
Output:			nothing
Modifies:		interface variables defined in userif.h
******************************************************************************/
void MC_ConFil(void);

/******************************************************************************
Function:		MC_SpeedPI
Description:	Speed PI control
Input:			reference speed f32wr,
				measured speed f32wm,
				maximum output f32imx,
				integral term memory address af32i
Output:			control output
Modifies:		integral memory *af32i
******************************************************************************/
float32_t MC_SpeedPI(float32_t f32wr, float32_t f32wm, float32_t f32imx, float32_t *af32i);

/******************************************************************************
Function:		MC_IdIqPI
Description:	Current PI control
Input:			reference direct current f32ird,
				measured direct current f32imd,
				reference quadrature current f32irq,
				measured quadrature current f32imq,
				maximum allowable output f32vmx,
				direct output address af32vd,
				quadrature output address af32vq,
				direct integral memory address af32iid,
				quadrature integral memory address af32iiq
Output:			control output
Modifies:		integral memories *af32iid, *af32iiq
******************************************************************************/
void MC_IdIqPI(	float32_t f32ird, float32_t f32imd, float32_t f32irq, float32_t f32imq, float32_t f32vmx,
				float32_t *af32vd, float32_t *af32vq, float32_t *af32iid, float32_t *af32iiq);

/******************************************************************************
Function:		MC_WorkMan
Description:	Working modes management function (speed ramps etc.)
Input:			nothing
Output:			nothing
Modifies:		internal speed reference
******************************************************************************/
void MC_WorkMan(void);

#endif	// _I_MOTORCONTROL_H

// EOF i_motorcontrol.h

