
/*******************************************************************************
DISCLAIMER

THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.

WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
INTELLECTUAL PROPERTY INFRINGIMENT.

TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.

BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
THIS SOFTWARE.
BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.

IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
File:		motorcontrol.c
Project:	S5D9_MotorControl
Revision:	1.0
Date:		01-06-16 (DD-MM-YYYY)
Author:		Andrea Ghirardello
Contents:	Motor control specific functions
*******************************************************************************/

#define _MOTORCONTROL_C		// insert define BEFORE including header files

/*******************************************************************************
File include
*******************************************************************************/
#include "globdef.h"
#include "i_ges_eqp.h"
#include "i_motorcontrol.h"
#include ".\library\i_mcrplibf.h"
#ifdef CPI_AUT
#include ".\library\i_PI_tun.h"
#endif	// ifdef CPI_AUT
#ifdef	MOT_IDENT
#include ".\library\i_mot_ident.h"
#endif	// ifdef MOT_IDENT
#include "..\mc_timer\gpt_pwm_defs.h"
#include "..\mc_timer\mc_timer_api.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/

// flags
#define	RAMPLIM			(0 != u8b_flgx.bit.b0)
#define	STARTED			(0 != u8b_flgx.bit.b1)
#define	STOPPED			(0 != u8b_flgx.bit.b2)
#define	DECEL			(0 != u8b_flgx.bit.b3)
#define	FREE_X4			(0 != u8b_flgx.bit.b4)
#define	FREE_X5			(0 != u8b_flgx.bit.b5)
#define	FREE_X6			(0 != u8b_flgx.bit.b6)
#define	FREE_X7			(0 != u8b_flgx.bit.b7)
#define	NOT_RAMPLIM		(0 == u8b_flgx.bit.b0)
#define	NOT_STARTED		(0 == u8b_flgx.bit.b1)
#define	NOT_STOPPED		(0 == u8b_flgx.bit.b2)
#define	NOT_DECEL		(0 == u8b_flgx.bit.b3)
#define	NOT_FREE_X4		(0 == u8b_flgx.bit.b4)
#define	NOT_FREE_X5		(0 == u8b_flgx.bit.b5)
#define	NOT_FREE_X6		(0 == u8b_flgx.bit.b6)
#define	NOT_FREE_X7		(0 == u8b_flgx.bit.b7)
#define	SET_RAMPLIM		{u8b_flgx.bit.b0 = 1;}
#define	SET_STARTED		{u8b_flgx.bit.b1 = 1;}
#define	SET_STOPPED		{u8b_flgx.bit.b2 = 1;}
#define	SET_DECEL		{u8b_flgx.bit.b3 = 1;}
#define	SET_FREE_X4		{u8b_flgx.bit.b4 = 1;}
#define	SET_FREE_X5		{u8b_flgx.bit.b5 = 1;}
#define	SET_FREE_X6		{u8b_flgx.bit.b6 = 1;}
#define	SET_FREE_X7		{u8b_flgx.bit.b7 = 1;}
#define	RES_RAMPLIM		{u8b_flgx.bit.b0 = 0;}
#define	RES_STARTED		{u8b_flgx.bit.b1 = 0;}
#define	RES_STOPPED		{u8b_flgx.bit.b2 = 0;}
#define	RES_DECEL		{u8b_flgx.bit.b3 = 0;}
#define	RES_FREE_X4		{u8b_flgx.bit.b4 = 0;}
#define	RES_FREE_X5		{u8b_flgx.bit.b5 = 0;}
#define	RES_FREE_X6		{u8b_flgx.bit.b6 = 0;}
#define	RES_FREE_X7		{u8b_flgx.bit.b7 = 0;}

// internal filters (main interrupt) time constants	[sec]
#define VISU_KT			( 0.3f )	// visualization filters
#define	VB_KT_FAST		( 0.002f )	// bus voltage filter fast
#define	VB_KT_SLOW		( 0.2f )	// bus voltage filter slow
#define	OMR_KT			( 0.05f )	// speed reference filter
#define	OFF_KT			( 0.5f )	// A/D offsets filters


/*******************************************************************************
Type Defines
*******************************************************************************/
typedef enum
{
	STOP_0 = 0,
	STOP_W = 1,
	START_P = 2,
	START_N = 3,
	GO_P = 4,
	GO_N = 5
}   ramp_sta_t;

/*******************************************************************************
Internal variables
*******************************************************************************/

uint32_t
	u32_align_dmy1;

u8_bitflags_t
	u8b_flgx,
	u8b_flgy,
	u8b_flgw,
	u8b_flgz;

adc_data_size_t
	adc_iu,				// IU analog channel sample
	adc_iv,				// IV analog channel sample
	adc_iw,				// IW analog channel sample
	adc_vb;				// VB analog channel sample

uint16_t
    u16_cnt_int,        // synchronization  counter
	u16_startup_cnt,	// counter in startup management
	u16_trip_cnt;		// counter in trip alarm management

ramp_sta_t
	ramp_status;		// speed ramp state machine status

ssp_err_t
    ssp_err;			// error from BSP APIs

int32_t
	s32_duty_u,			// u phase duty cycle
	s32_duty_v,			// v phase duty cycle
	s32_duty_w;			// w phase duty cycle

float32_t
	f32_iu_off,			// IU analog channel offset
	f32_iv_off,			// IV analog channel offset
	f32_iw_off,			// IW analog channel offset
	f32_r_sta,			// stator resistance [Ohm]
	f32_l_sq,			// synchronous inductance [Hen]
	f32_pm_fl,			// permanent magnets flux [Web]
	f32_fb_gain,		// flux amplitude feedback gain
	f32_fl_lpco_hz,		// approx. flux estimation filter cutoff frequency [Hz]
	f32_c_poli,			// number of polar couples
	f32_krpmocp,		// conversion constant between mechanical speed and electrical speed
	f32_ukrpmocp,		// 1/f32_krpmocp
	f32_i_start,		// startup current (peak) [A]
	f32_delta_om,		// speed variation (during startup) [rad/s]
	f32_is_rup,			// startup current increasing rate [A]
	f32_is_rdw,			// startup current decreasing rate [A]
	f32_speed_ref,		// speed reference [rpm]
	f32_vbus,			// bus voltage [V]
	f32_vb_fil,			// filtered bus voltage [V]
	f32_vbus_minf,		// filtered available bus voltage (minimum value) [V]
	f32_vfmax,			// max phase voltage (star) [V]
	f32_i_max,			// maximum total current [A]
	f32_iqmax,			// maximum q current [A]
	f32_vdc,			// applied d voltage [V]
	f32_vqc,			// applied q voltage [V]
	f32_vdx,			// applied d voltage [V]
	f32_vqx,			// applied q voltage [V]
	f32_vdm,			// d voltage (applied at previous step) [V]
	f32_vqm,			// q voltage (applied at previous step) [V]
	f32_vdmf,			// applied d voltage (filtered) [V]
	f32_vqmf,			// applied q voltage (filtered) [V]
	f32_vac,			// applied alpha voltage (star) [V]
	f32_vbc,			// applied beta voltage (star) [V]
	f32_vax,			// applied alpha voltage [V]
	f32_vbx,			// applied beta voltage [V]
	f32_vam,			// alpha voltage (applied at previous step) [V]
	f32_vbm,			// beta voltage (applied at previous step) [V]
	f32_vuc,			// applied u voltage (star) [V]
	f32_vvc,			// applied v voltage (star) [V]
	f32_vwc,			// applied w voltage (star) [V]
	f32_vux,			// applied u voltage (star) [V]
	f32_vvx,			// applied v voltage (star) [V]
	f32_vwx,			// applied w voltage (star) [V]
	f32_ium,			// u current (measured) [A]
	f32_ivm,			// v current (measured) [A]
	f32_iwm,			// w current (measured) [A]
	f32_iam,			// alpha current (measured) [A]
	f32_ibm,			// beta current (measured) [A]
	f32_idm,			// d current (measured) [A]
	f32_iqm,			// q current (measured) [A]
	f32_idmf,			// d current (filtered) [A]
	f32_iqmf,			// q current (filtered) [A]
	f32_itr,			// total current reference (used in startup) [A]
	f32_idr,			// d current reference [A]
	f32_iqr,			// q current reference [A]
	f32_iar,			// alpha current reference [A]
	f32_ibr,			// beta current reference [A]
	f32_idint,			// d current error integral value [A]
	f32_iqint,			// q current error integral value [A]
	f32_iaint,			// intermediate variable [A]
	f32_ibint,			// intermediate variable [A]
	f32_idrvar,			// d current reference [A]
	f32_errint,			// speed error integral memory [rad/sec]
	f32_kp_cur,			// current PI proportional gain
	f32_ki_cur,			// current PI integral gain
	f32_kp_vel,			// speed PI proportional gain
	f32_ki_vel,			// speed PI integral gain
	f32_rpm_min,		// minimum reference speed [rpm]
	f32_rpm_max,		// maximum reference speed [rpm]
	f32_rpmrif_x,		// reference speed (ramp input) [rpm]
	f32_rpmrif_y,		// reference speed (ramp output) [rpm]
#ifdef SPEEDPI_ERRCLAMP
	f32_maxerr,			// maximum speed error [rad/s]
#endif	// ifdef SPEEDPI_ERRCLAMP
	f32_r_acc,			// acceleration ramp [rpm/main_loop_duration]
	f32_r_dec,			// deceleration ramp [rpm/main_loop_duration]
	f32_min_speed,		// minimum speed [rad/s]
	f32_max_speed,		// maximum speed [rad/s]
	f32_min_s_trip,		// minimum speed (for trip detection) [rad/s]
	f32_max_s_trip,		// maximum speed (for trip detection) [rad/s]
#ifdef FLUXWEAK
	f32_intid_rup,		// id integral variation ramp up in FW [A/sampling_period]
	f32_intid_rdw,		// id integral variation ramp dw in FW [A/sampling_period]
	f32_amp_v,			// voltage vector amplitude
	f32_id_max,			// maximum d current [A]
	f32_vmarg,			// voltage margin for flux weakening [V]
#endif	// ifdef FLUXWEAK
	f32_Speed_est,		// estimated speed [rad/s]
	f32_omrif,			// reference speed [rad/s]
	f32_omrif_abs,		// |reference speed| [rad/s]
	f32_f_omrif,		// filtered reference speed [rad/s]
	f32_omegae,			// electrical angular speed [rad/s]
	f32_omf,			// electrical angular speed (filtered) [rad/s]
	f32_omf_abs,		// |electrical angular speed (filtered)| [rad/s]
	f32_delta_ph,		// phase variation (during startup) [rad]
	f32_SystemPhase,	// rotating reference system angular position [rad]
	f32_Phase_est,		// estimated electrical phase [rad]
	f32_Phase_offset,	// optional phase offset [rad]
	f32_FluxAmp,		// flux vector amplitude estimation [web]
	f32_FluxPhase,		// flux vector phase first estimation (not compensated) [rad]
	f32_f_amp_i,		// filtered current vector amplitude (derived from idmf, iqmf)
	f32_f_arg_i,		// filtered current vector phase (derived from idmf, iqmf)
	f32_f_amp_v,		// filtered voltage vector amplitude (derived from vdmf, vqmf)
	f32_f_arg_v,		// filtered voltage vector phase (derived from vdmf, vqmf)
	f32_freq,			// electrical frequency [Hz/10]
	f32_mec_rpm;		// mechanical speed [rpm]

// variables involved in sampling frequency adaptation
float32_t
	f32_pwm_fre_Hz,		// true pwm frequency [Hz]
	f32_sam_per_s,		// true sampling period [s]
	f32_sam_per_ms,		// true sampling period [ms]
	f32_sam_fre_Hz,		// true sampling frequency [Hz]
	f32_sam_fre_kHz,	// true sampling frequency [kHz]
	f32_mloop_per_s,	// true main loop period [s]
	f32_mloop_fre_Hz,	// true main loop frequency [Hz]
	f32_comptime,		// phase compensation time
	f32_u_halfper,		// 1/u32_halfper
	f32_k_vout,			// keeps into account modulation voltage losses
	f32_visu_ukt,		// 1.0 / (1.0 + (VISU_KT / SAM_PER_S))
	f32_vb_ukt_fast,	// 1.0 / (1.0 + (VB_KT_FAST / SAM_PER_S))
	f32_vb_ukt_slow,	// 1.0 / (1.0 + (VB_KT_SLOW / SAM_PER_S))
	f32_omr_ukt,		// 1.0 / (1.0 + (OMR_KT / SAM_PER_S))
	f32_off_ukt;		// 1.0 / (1.0 + (OFF_KT / SAM_PER_S))

uint16_t
	u16_fre_ratio,		// ratio between pwm frequency and sampling frequency
	u16_num_int;		// number of control interrupts for each main loop

uint32_t
	u32_halfper,		// half pwm period in GPT timer ticks
	u32_pwm_per_int,	// pwm period in GPT timer ticks
	u32_sam_per_int,	// sampling period in GPT timer ticks
	u32_deadtim,		// deadtime in GPT timer ticks
	u32_con_delay,		// A/D start conversion delay in GPT timer ticks
	u32_min_ton_low,	// minimum on time of the low arms in GPT timer ticks
	u32_dmin,			// minimum duty in GPT timer ticks
	u32_delmed,			// maximum delta between maximum duty cycle and medium one
	u32_delmax;			// maximum delta between maximum duty cycle and minimum one
// end of variables involved in sampling frequency adjustment

mc_timer_dutycycles
	dutycycle;			// dutycycles structure

#ifdef CPI_TUN
// current PI tuning
uint16_t
	u16_pit_s,			// counter in tuning state machine
	u16_pit_c;			// counter in tuning state machine
float32_t
	f32_ratio,			// ratio between maximum current and current step amplitude
	f32_cur_ampf;		// filtered measured current amplitude
#endif	// ifdef CPI_TUN

#ifdef CPI_AUT
// automatic current PI tuning
#define I_PIT_RATIO		( 0.8f )	// 80% of i_max
#define R_UP			( 0 )
#define R_MS			( 1 )
#define R_DW			( 2 )
#define R_ST			( 3 )
#define R_EN			( 4 )
float32_t
	f32_v_req;			// required voltage
uint16_t
	u16_pitun_status;	// state machine status
#endif	// ifdef CPI_AUT

// motor parameters identification
#ifdef MOT_IDENT
#define IDENT_SPEED_HZ  ( 20.0f )
//#define IDENT_SPEED_HZ  (100.0f )
#define X_TIM1			((int32_t)(ID_TIM1_MS * f32_sam_fre_kHz))	// current rising + alignment + stabilization
#define X_TIM2			((int32_t)(ID_TIM2_MS * f32_sam_fre_kHz))	// first measurement step
#define X_TIM3			((int32_t)(ID_TIM3_MS * f32_sam_fre_kHz))	// current falling + stabilization
#define X_TIM4			((int32_t)(ID_TIM4_MS * f32_sam_fre_kHz))	// second measurement step
#define X_TIM5			((int32_t)(ID_TIM5_MS * f32_sam_fre_kHz))	// speed rising + stabilization
#define X_TIM6			((int32_t)(ID_TIM6_MS * f32_sam_fre_kHz))	// third measurement step
#define X_TIM7			((int32_t)(ID_TIM7_MS * f32_sam_fre_kHz))	// current falling + stabilization
#define X_TIM8			((int32_t)(ID_TIM8_MS * f32_sam_fre_kHz))	// fourth measurement step
#define X_TIM9			((int32_t)(ID_TIM9_MS * f32_sam_fre_kHz))	// final wait
#define INI_IDENT		{s32_x_cnt = 0; MC_Ident_IniVar(f32_sam_fre_Hz);}
#define IDENT_ENDED		(X_TIM9 < s32_x_cnt)
int32_t
	s32_x_cnt;			// identification synchronization counter
float32_t
	f32_iden_rs,		// identification stator resistance
	f32_iden_ls,		// identification synchronous inductance
	f32_iden_fl,		// identification permanent magnets flux
	f32_iden_om,		// identification speed
	f32_iden_or,		// identification speed ramp
	f32_ial_1,			// identification current 1
	f32_ial_2,			// identification current 2
	f32_ial_3,			// identification current 3
	f32_ial_r;			// identification current ramp
#endif	// ifdef MOT_IDENT

// oscilloscope window in GUI
#ifdef OSC_WIN
float32_t
	f32_imax1000,		// maximum current divided by 1000
	f32_phmem;			// angular position memory
int16_t
	s16_sample,			// measurement sample
	s16_selmem,			// variable selection memory
	s16_bcntm,			// counter memory
	s16_bcnt0,			// counter
	s16_bcnt1;			// counter
#endif	// ifdef OSC_WIN

#ifdef MACRO_DEBUG
float32_t
	pf32_kfo,			// Omega[rad/sec]=KFO*Freq[Hertz]
	pf32_kof,			// Freq[Hertz]=KOF*Omega[rad/sec]
	pf32_krpmo,			// Omega[rad/sec]=KRPMO*velocita[rpm]
	pf32_korpm,			// speed[rpm]=KORPM*Omega[rad/sec]
	pf32_kadi,			// current A/D conversion constant
	pf32_kadv;			// voltage A/D conversion constant
void MC_ConstDebug(void)
{
	pf32_kfo = KFO;
	pf32_kof = KOF;
	pf32_krpmo = KRPMO;
	pf32_korpm = KORPM;
	pf32_kadi = KADI;
	pf32_kadv = KADV;
}
#endif	// ifdef MACRO_DEBUG

// sampling frequency related constants calculation
/*******************************************************************************
Function:		MC_SetPwmConst
Description:	calculation of constants related to sampling (and pwm) freq.
Input:			nothing
Output:			nothing
Modifies:		many global variables related to sampling frequency and used as
				constants
*******************************************************************************/
void MC_SetPwmConst(void)
{
	// getting parameters values
	f32_sam_fre_Hz = SAM_FRE;
	u16_fre_ratio = F_RATIO;
	f32_c_poli = C_POLI;

	// Read the current PCLK frequency from the clock module
	uint32_t gpt_fre_Hz = 0;
	g_cgc_on_cgc.systemClockFreqGet(CGC_SYSTEM_CLOCKS_PCLKD, &gpt_fre_Hz);

	// converts all important time constants in peripheral ticks unit
	u32_deadtim = gpt_fre_Hz * ((float32_t)(DEADTIM_NS / 1000000000.0));
	if(1u > u32_deadtim)
	{
		u32_deadtim = 1u;
	}
	u32_min_ton_low = gpt_fre_Hz * ((float32_t)(MIN_TON_LOW_NS / 1000000000.0));
	u32_dmin = u32_deadtim + u32_min_ton_low;
	u32_con_delay = gpt_fre_Hz * ((float32_t)(CON_DELAY_NS / 1000000000.0));

	// pwm and sampling period calc in internal timer ticks
	f32_pwm_fre_Hz = u16_fre_ratio * f32_sam_fre_Hz;
	u32_halfper = ((uint32_t)((0.5f * gpt_fre_Hz) / f32_pwm_fre_Hz));	// half pwm period in GPT ticks (int)
	u32_pwm_per_int = u32_halfper << 1;									// pwm period in GPT ticks (int)
	u32_sam_per_int = u16_fre_ratio * u32_pwm_per_int;					// sampling period in GPT ticks (int)

	// pwm and sampling freq. re-calculation
	f32_pwm_fre_Hz = gpt_fre_Hz / u32_pwm_per_int;						// true pwm frequency in Hz (float)
	f32_sam_fre_Hz = gpt_fre_Hz / u32_sam_per_int;						// true sampling frequency in Hz (float)
	f32_sam_per_s = 1.0f / f32_sam_fre_Hz;								// true sampling period in sec (float)

	// main loop timing
	u16_num_int = ((uint16_t)(f32_sam_fre_Hz / MLOOP_FRE_HZ));			// number of sampling periods in one main loop (int)
	f32_mloop_per_s = u16_num_int / f32_sam_fre_Hz;						// true main loop period in sec (float)

	// control parameters
	f32_comptime = K_COMPPH * f32_sam_per_s;							// phase compensation time

	// pwm management parameters
	u32_delmax = u32_halfper - u32_deadtim;								// max delta between max duty and min duty in GPT ticks (int)
	u32_delmax -= 1;
	u32_delmed = u32_halfper - u32_dmin;								// max delta between max duty and med duty in GPT ticks (int)
	u32_delmed -= 1;
	f32_u_halfper = 1.0f / u32_halfper;
	f32_k_vout = u32_delmax * f32_u_halfper * SQRT3D3 * K_OVERMOD;

	// polar couples related constants
	f32_krpmocp = f32_c_poli * KRPMO;
	f32_ukrpmocp = ((float32_t)(1.0f / f32_krpmocp));
#ifdef SPEEDPI_ERRCLAMP
	f32_maxerr = MAXERR_RPM * f32_krpmocp;
#endif	// ifdef SPEEDPI_ERRCLAMP

	// other useful quantities
	f32_sam_fre_kHz = f32_sam_fre_Hz * 0.001f;							// true sampling frequency in kHz (float)
	f32_sam_per_ms = 1000.0f * f32_sam_per_s;							// true sampling period in ms (float)

	// filters time constants
	f32_visu_ukt = 1.0f / (1.0f + (VISU_KT * f32_sam_fre_Hz));
	f32_vb_ukt_fast = 1.0f / (1.0f + (VB_KT_FAST * f32_sam_fre_Hz));
	f32_vb_ukt_slow = 1.0f / (1.0f + (VB_KT_SLOW * f32_sam_fre_Hz));
	f32_omr_ukt = 1.0f / (1.0f + (OMR_KT * f32_sam_fre_Hz));
	f32_off_ukt = 1.0f / (1.0f + (OFF_KT * f32_sam_fre_Hz));

	return;
}
// end of sampling frequency related constants calculation

ioport_level_t  debug_level;
/******************************************************************************
Function:       MC_Debug
Description:    debug function
Input:          nothing
Output:         nothing
******************************************************************************/
void MC_Debug(void)
{

#define EEP_SDA         ((ioport_port_pin_t)IOPORT_PORT_04_PIN_02)
#define EEP_SCL         ((ioport_port_pin_t)IOPORT_PORT_04_PIN_03)

    g_ioport.p_api->pinDirectionSet(EEP_SDA, IOPORT_DIRECTION_INPUT);
    g_ioport.p_api->pinRead(EEP_SDA, &debug_level);
    g_ioport.p_api->pinDirectionSet(EEP_SDA, IOPORT_DIRECTION_OUTPUT);
    g_ioport.p_api->pinWrite(EEP_SDA, IOPORT_LEVEL_HIGH);
    g_ioport.p_api->pinWrite(EEP_SDA, IOPORT_LEVEL_LOW);
    g_ioport.p_api->pinWrite(EEP_SDA, IOPORT_LEVEL_HIGH);
    g_ioport.p_api->pinDirectionSet(EEP_SDA, IOPORT_DIRECTION_INPUT);

    g_ioport.p_api->pinDirectionSet(EEP_SCL, IOPORT_DIRECTION_INPUT);
    g_ioport.p_api->pinRead(EEP_SCL, &debug_level);
    g_ioport.p_api->pinDirectionSet(EEP_SCL, IOPORT_DIRECTION_OUTPUT);
    g_ioport.p_api->pinWrite(EEP_SCL, IOPORT_LEVEL_HIGH);
    g_ioport.p_api->pinWrite(EEP_SCL, IOPORT_LEVEL_LOW);
    g_ioport.p_api->pinWrite(EEP_SCL, IOPORT_LEVEL_HIGH);
    g_ioport.p_api->pinDirectionSet(EEP_SCL, IOPORT_DIRECTION_INPUT);

}

/******************************************************************************
Function:       MC_WaitSync
Description:    time synchronization control function
Input:          nothing
Output:         0 if main loop time is elapsed, 1 if not
Modifies:       internal status variable
******************************************************************************/
uint16_t MC_WaitSync(void)
{
	if (0 != u16_cnt_int)
	{
		return( 1 );
	}
	u16_cnt_int = u16_num_int;
	return( 0 );
}

// current reading functions
#define OFF_CNV_NMB		( 32 )	// number of conversions to detect offsets
/******************************************************************************
Function:		MC_SetOff
Description:	A/D init and current offsets measurement
Input:			nothing
Output:			nothing
Modifies:		current reading offsets, conversion constants
******************************************************************************/
void MC_SetOff(void)
{
	uint16_t u16_cnv_cnt;

	// filters memories reset
	f32_iu_off = 0;
	f32_iv_off = 0;
	f32_iw_off = 0;
	f32_vb_fil = 0;

	// AD0 configuration for first conversions, used to detect A/D circuit offsets
    ssp_err = g_adc0_init.p_api->open(g_adc0_init.p_ctrl, g_adc0_init.p_cfg);
#ifdef FIRST_DEBUG
    if (SSP_SUCCESS != ssp_err) { while(1); }
#endif	// ifdef FIRST_DEBUG
    ssp_err = g_adc0_init.p_api->scanCfg(g_adc0_init.p_ctrl, g_adc0_init.p_channel_cfg);
#ifdef FIRST_DEBUG
    if (SSP_SUCCESS != ssp_err) { while(1); }
#endif	// ifdef FIRST_DEBUG

	// setting the PGA (still not included in A/D framework)
	//BYPASS_PGA_M0
	ENABLE_PGA_M0
	SET_PGA_GAIN_M0( PGA_GAIN )

	// repeated conversions, to detect offsets
    for (u16_cnv_cnt = 0; u16_cnv_cnt < OFF_CNV_NMB; u16_cnv_cnt++)
    {
        g_adc0_init.p_api->scanStart(g_adc0_init.p_ctrl);

#ifdef FIRST_DEBUG
        ssp_err = g_adc0_init.p_api->scanStatusGet(g_adc0_init.p_ctrl);
        while (SSP_ERR_IN_USE == ssp_err)
        {
            ssp_err = g_adc0_init.p_api->scanStatusGet(g_adc0_init.p_ctrl);
        }
        if (SSP_SUCCESS != ssp_err)
        {
            while(1);
        }
#else	// ifdef FIRST_DEBUG
        while (SSP_ERR_IN_USE == g_adc0_init.p_api->scanStatusGet(g_adc0_init.p_ctrl)) ;	// wait conversion end
#endif	// else ifdef FIRST_DEBUG

		g_adc0_init.p_api->read(g_adc0_init.p_ctrl, ADC_REG_CHANNEL_0, &adc_iu);
		g_adc0_init.p_api->read(g_adc0_init.p_ctrl, ADC_REG_CHANNEL_1, &adc_iv);
		g_adc0_init.p_api->read(g_adc0_init.p_ctrl, ADC_REG_CHANNEL_2, &adc_iw);
		g_adc0_init.p_api->read(g_adc0_init.p_ctrl, ADC_REG_CHANNEL_5, &adc_vb);
		f32_iu_off += adc_iu;
		f32_iv_off += adc_iv;
		f32_iw_off += adc_iw;
		f32_vb_fil += adc_vb;
    }
	f32_iu_off *= (float32_t)(1.0 / OFF_CNV_NMB);
	f32_iv_off *= (float32_t)(1.0 / OFF_CNV_NMB);
	f32_iw_off *= (float32_t)(1.0 / OFF_CNV_NMB);
	f32_vb_fil *= (float32_t)(1.0 / OFF_CNV_NMB);

	// close this ADC configuration (the closed-loop one will be automatically triggered)
    g_adc0_init.p_api->close(g_adc0_init.p_ctrl);

}

/******************************************************************************
Function:		MC_ADMan
Description:	closed loop management of triggered A/D conversions
Input:			nothing
Output:			nothing
Modifies:		global variables:
				f32_vbus, (bus voltage sample)
				f32_ium, f32_ivm, f32_iwm (current samples in u, v, w frame)
				f32_iam, f32_ibm (current samples in a, b frame)
******************************************************************************/
void MC_ADMan(void)
{
	uint32_t u32_rdu, u32_rdv, u32_rdw;

	// read A/D conversion results
    g_adc0_runtime.p_api->read(g_adc0_runtime.p_ctrl, ADC_REG_CHANNEL_0, &adc_iu);
    g_adc0_runtime.p_api->read(g_adc0_runtime.p_ctrl, ADC_REG_CHANNEL_1, &adc_iv);
    g_adc0_runtime.p_api->read(g_adc0_runtime.p_ctrl, ADC_REG_CHANNEL_2, &adc_iw);
    g_adc0_runtime.p_api->read(g_adc0_runtime.p_ctrl, ADC_REG_CHANNEL_5, &adc_vb);

	// phase currents calculation
	if (COM_ON)
	{

		// loading the actual duty-cycles values (they're the ones which have been used in
		// the last cycle) on which the A/D conversions are based
		g_mc_timer0.p_api->dutyCyclesGet(g_mc_timer0.p_ctrl, &dutycycle);
		u32_rdu = dutycycle.phases.du;             // pwm cycle U
		u32_rdv = dutycycle.phases.dv;             // pwm cycle V
		u32_rdw = dutycycle.phases.dw;             // pwm cycle W

		// finding duties order and minimum value, calculating current samples based on the two best readings
		// (low compare reg value -> high voltage -> short on time of the low arm -> bad reading)
		if (u32_rdu >= u32_rdv)
		{								// du >= dv
			if (u32_rdv >= u32_rdw)
			{							// du >= dv >= dw (w is the min -> we choose iu and iv)
				f32_ium = KADI * (adc_iu - f32_iu_off);
				f32_ivm = KADI * (adc_iv - f32_iv_off);
				f32_iwm = -f32_ium - f32_ivm;
			}
			else
			{							// du >= dv, dw > dv (v is the min -> we choose iw and iu)
				f32_iwm = KADI * (adc_iw - f32_iw_off);
				f32_ium = KADI * (adc_iu - f32_iu_off);
				f32_ivm = -f32_iwm - f32_ium;
			}
		}
		else
		{								// dv > du
			if (u32_rdu >= u32_rdw)
			{							// dv > du >= dw (w is the min -> we choose iu and iv)
				f32_ium = KADI * (adc_iu - f32_iu_off);
				f32_ivm = KADI * (adc_iv - f32_iv_off);
				f32_iwm = -f32_ium - f32_ivm;
			}
			else
			{							// dv > du, dw > du (u is the min -> we choose iv and iw)
				f32_ivm = KADI * (adc_iv - f32_iv_off);
				f32_iwm = KADI * (adc_iw - f32_iw_off);
				f32_ium = -f32_ivm - f32_iwm;
			}
		}

		// alpha, beta currents calc
		McrpLibf_uv_alphabeta(f32_ium, f32_ivm, &f32_iam, &f32_ibm);
	}
	else
	{
#ifdef OFF_REFRESH
		// current offsets refresh
		f32_iu_off += (adc_iu - f32_iu_off) * f32_off_ukt;
		f32_iv_off += (adc_iv - f32_iv_off) * f32_off_ukt;
		f32_iw_off += (adc_iw - f32_iw_off) * f32_off_ukt;
#endif  // ifdef OFF_REFRESH
	    f32_ium = f32_ivm = f32_iwm = 0.0f;
		f32_iam = f32_ibm = 0.0f;
	}

	// bus voltage calculation
	f32_vbus = KADV * adc_vb;			// dc bus voltage in Volt

}	// end of function MC_ADMan()
// end of current reading functions

// PWM management functions
/******************************************************************************
Function:		MC_StopOut
Description:	disable pwm outputs
Input:			nothing
Output:			nothing
******************************************************************************/
void MC_StopOut(void)
{
	g_mc_timer0.p_api->disableOut(g_mc_timer0.p_ctrl);
	RES_OUT_ENA
	RES_COM_ON
}

/******************************************************************************
Function:		MC_StartOut
Description:	enables pwm outputs and insert a loop for bootstrap
Input:			nothing
Output:			nothing
******************************************************************************/
void MC_StartOut(void)
{
	//bool enabled;
	//g_mc_timer0.p_api->outEnabled(g_mc_timer0.p_ctrl, &enabled);
	//if (enabled)
	if (OUT_ENA)
	{
		SET_COM_ON		// set the flag after one main loop (bootstrap)
	}
	else
	{
		RES_COM_ON
		g_mc_timer0.p_api->enableOut(g_mc_timer0.p_ctrl);
		SET_OUT_ENA
	}
}

/******************************************************************************
Function: 		MC_IniPWM
Description: 	This function initializes the GPT module and the closed loop
				A/D conversions, then links GPT with A/D converter
Input:			nothing
Output:			nothing
******************************************************************************/
void MC_IniPWM(void)
{

	// parameters are initialized: we can fill pwm timer parameters structure
	//g_mc_timer0_cfg.channels[0] = 1;
	//g_mc_timer0_cfg.channels[1] = 2;
	//g_mc_timer0_cfg.channels[2] = 3;
	g_mc_timer0_par_cfg.halfperiod = u32_halfper;
	g_mc_timer0_par_cfg.deadtime = u32_deadtim;
	g_mc_timer0_par_cfg.fre_ratio = u16_fre_ratio;
	g_mc_timer0_par_cfg.con_delay = u32_con_delay;

	// now open the timer
	ssp_err = g_mc_timer0.p_api->open(g_mc_timer0.p_ctrl, g_mc_timer0.p_cfg);
	if (SSP_SUCCESS != ssp_err)
	{
		while(1);
	}

	// open a new ADC0 configuration, automatically triggered
    ssp_err = g_adc0_runtime.p_api->open(g_adc0_runtime.p_ctrl, g_adc0_runtime.p_cfg);
#ifdef FIRST_DEBUG
    if (SSP_SUCCESS != ssp_err) { while(1); }
#endif	// ifdef FIRST_DEBUG
    ssp_err = g_adc0_runtime.p_api->scanCfg(g_adc0_runtime.p_ctrl, g_adc0_runtime.p_channel_cfg);
#ifdef FIRST_DEBUG
    if (SSP_SUCCESS != ssp_err) { while(1); }
#endif	// ifdef FIRST_DEBUG

	// setting the PGA (still not included in A/D framework)
	//BYPASS_PGA_M0
	ENABLE_PGA_M0
	SET_PGA_GAIN_M0( PGA_GAIN )

    // link GPT with ADC, using ELC
	// TODO: generalizzare a seconda dei canali usati!
    ssp_err = g_elc.p_api->linkSet(ELC_PERIPHERAL_ADC0, ELC_EVENT_GPT1_AD_TRIG_A);
#ifdef FIRST_DEBUG
    if (SSP_SUCCESS != ssp_err) { while(1); }
#endif	// ifdef FIRST_DEBUG
    ssp_err = g_elc.p_api->enable();
#ifdef FIRST_DEBUG
    if (SSP_SUCCESS != ssp_err) { while(1); }
#endif	// ifdef FIRST_DEBUG

}

/******************************************************************************
Function: 		MC_StartPWM
Description: 	This function starts GPT operations
Input:			nothing
Output:			nothing
******************************************************************************/
void MC_StartPWM(void)
{

	// enable ADC triggered conversions
    g_adc0_runtime.p_api->scanStart(g_adc0_runtime.p_ctrl);

	// Start GPT counting (simultaneous start for the three channels) (output pins still disabled!)
	g_mc_timer0.p_api->start(g_mc_timer0.p_ctrl);

	/* PWM outputs are still not enabled! */
}

/******************************************************************************
Function:		MC_CheckPOEAFlags
Description:	check POE flags and eventually set an alarm; in case reset
				the flags
Input:			nothing
Output:			nothing
******************************************************************************/
static void MC_CheckPOEAFlags(void)
{
	// TODO: gestire meglio!!
	if (0 != R_POEG->POEGGA_b.IOCF)
	{
		MC_StopOut();
		UIF_R.var.all = FAULT_ALL;
		SET_ALRM_ON
		R_POEG->POEGGA_b.IOCF = 0;	// B1 Real Time Overcurrent Detection Flag
	}
	else if (0 != R_POEG->POEGGA_b.PIDF)
    {
        MC_StopOut();
        UIF_R.var.all = OVC_ALL;
		SET_ALRM_ON
		R_POEG->POEGGA_b.PIDF = 0;	// B0 Port Input Detection Flag
    }
}

/******************************************************************************
Function:		MC_CheckPOEBFlags
Description:	check POE flags and eventually set an alarm; in case reset
				the flags
Input:			nothing
Output:			nothing
******************************************************************************/
static void MC_CheckPOEBFlags(void)
{
	// TODO: gestire meglio!!
	if (0 != R_POEG->POEGGB_b.IOCF)
	{
		MC_StopOut();
		UIF_R.var.all = FAULT1_ALL;
		SET_ALRM_ON
		R_POEG->POEGGB_b.IOCF = 0;	// B1 Real Time Overcurrent Detection Flag
	}
	else if (0 != R_POEG->POEGGB_b.PIDF)
    {
        MC_StopOut();
        UIF_R.var.all = OVC1_ALL;
		SET_ALRM_ON
		R_POEG->POEGGB_b.PIDF = 0;	// B0 Port Input Detection Flag
    }
}

/******************************************************************************
Function:		MC_PWMGen
Description:	output duties calc
Input:			nothing
Note:			uses global variables f32_vuc, f32_vvc and f32_vwc (requested voltages),
				global variable f32_vbus (bus voltage)
Output:			nothing
Modifies:		global variables s32_duty_u, s32_duty_v, s32_duty_w,
				f32_vuc, f32_vvc, f32_vwc
******************************************************************************/
void MC_PWMGen(void)
{
	float32_t	f32a;
	int32_t		s32a, s32b;
	int32_t		*as32_min, *as32_med, *as32_max;
	int32_t		s32_dmin, s32_hmed, s32_hmax;

	if (VBMIN > f32_vbus)
	{
		s32_duty_u = s32_duty_v = s32_duty_w = u32_halfper - 1;
		f32_vuc = f32_vvc = f32_vwc = 0.0f;
		return;
	}

	// duty-cycles calculation
	// note: no problems if halfper is summed to all the three duties
	f32a = u32_halfper / (-f32_vbus);				// f32a = u32_halfper / f32_vbus;
	s32_duty_u = f32a * f32_vuc;					// s32_duty_u = f32a * (f32_vbus - f32_vuc);
	s32_duty_v = f32a * f32_vvc;					// s32_duty_v = f32a * (f32_vbus - f32_vvc);
	s32_duty_w = f32a * f32_vwc;					// s32_duty_w = f32a * (f32_vbus - f32_vwc);

	// finding duties order and limits
	if (s32_duty_u >= s32_duty_v)
	{										// du >= dv
		if (s32_duty_v >= s32_duty_w)
		{									// du >= dv >= dw
			as32_min = (&s32_duty_w);
			as32_med = (&s32_duty_v);
			as32_max = (&s32_duty_u);
		}
		else
		{									// du >= dv, dw > dv
			as32_min = (&s32_duty_v);
			if (s32_duty_u >= s32_duty_w)
			{								// du >= dw > dv
				as32_med = (&s32_duty_w);
				as32_max = (&s32_duty_u);
			}
			else
			{								// dw > du > dv
				as32_med = (&s32_duty_u);
				as32_max = (&s32_duty_w);
			}
		}
	}
	else
	{										// dv > du
		if (s32_duty_u >= s32_duty_w)
		{									// dv > du >= dw
			as32_min = (&s32_duty_w);
			as32_med = (&s32_duty_u);
			as32_max = (&s32_duty_v);
		}
		else
		{									// dv > du, dw > du
			as32_min = (&s32_duty_u);
			if (s32_duty_v >= s32_duty_w)
			{								// dv > dw > du
				as32_med = (&s32_duty_w);
				as32_max = (&s32_duty_v);
			}
			else
			{								// dw > dv > du
				as32_med = (&s32_duty_v);
				as32_max = (&s32_duty_w);
			}
		}
	}

	// duties differences calculation
	s32_hmed = (*as32_med) - (*as32_min);
	s32_hmax = (*as32_max) - (*as32_min);

	// maximum limit control (and saturation)
	if (s32_hmax > u32_delmax)
	{
		s32a = s32_hmed * u32_delmax;
		s32_hmed = s32a / s32_hmax;
		s32_hmax = u32_delmax;
	}

	// medium limit control (and saturation)
	s32b = s32_hmax - s32_hmed;
	if (s32b > u32_delmed)
	{
		s32a = s32_hmax * u32_delmed;
		s32_hmax = s32a / s32b;
		s32a = s32_hmed * u32_delmed;
		s32_hmed = s32a / s32b;
	}

	// minimum duty (corresponding to maximum voltage) calculation
	s32_dmin = u32_halfper - 1 - s32_hmax;

// now we have min_duty equal to s32_dmin, med_duty equal to (s32_dmin + s32_hmed), and max_duty equal to (s32_dmin + s32_hmax)
// the maximum duty is at its maximum value u32_halfper - 1, that is, the modulation is clamped at the minimum voltage
// at least two duty cycles are greater than the minimum value needed for good current reading due to the previous clamping (u32_delmed)
// the duty corresponding to the voltage average (star-centre) is: s32_dmin + ((s32_hmed + s32_hmax) / 3)

	// duties update
	// keeping clamped position
	(*as32_max) = u32_halfper - 1;
	(*as32_med) = s32_dmin + s32_hmed;
	(*as32_min) = s32_dmin;

	// output voltages calculation after clamping
	f32a = f32_vbus * f32_u_halfper;
	f32_vux = (u32_halfper - s32_duty_u) * f32a;
	f32_vvx = (u32_halfper - s32_duty_v) * f32a;
	f32_vwx = (u32_halfper - s32_duty_w) * f32a;

}	// end of function MC_PWMGen()

// end of PWM management functions

/******************************************************************************
Function:		MC_VecComp
Description:	calculation of the second component of a vector (absolute value)
Input:			vector amplitude f32amp (greater than zero)
				first component f32cmp
Output:			second component
Note:			result is positive or zero
******************************************************************************/
float32_t MC_VecComp(float32_t f32amp, float32_t f32cmp)
{
	float32_t f32a;

	f32a = (f32amp * f32amp) - (f32cmp * f32cmp);
	if (0.0f < f32a)
	{
		return( sqrtf( f32a ) );
	}
	else
	{
		return( 0.0f );
	}
}

/******************************************************************************
Function:		MC_IniPar
Description:	internal constants initialization, using eeprom values
Input:			nothing
Output:			nothing
Modifies:		many internal variables, using eeprom parameters values
******************************************************************************/
void MC_IniPar(void)
{
    uint32_t    u32i;

#ifndef EEPROM_USED
    // default parameters loading
    for (u32i = 0; u32i < NUM_PAR; u32i++)
    {
        p_vec[u32i] = p_def[u32i];
    }
#endif	// ifndef EEPROM_USED

    // pwm constants setting, based on parameter values
	MC_SetPwmConst();
	UIF_R.var.pwmf = f32_pwm_fre_Hz;
	UIF_R.var.sf = f32_sam_fre_Hz;

#ifdef MACRO_DEBUG
	MC_ConstDebug();
#endif	// ifdef MACRO_DEBUG

	// additional features enabling
#ifdef CPI_TUN
	ENA_CURPI_TUN
#endif	// ifdef CPI_TUN
#ifdef CPI_AUT
	ENA_CURPI_AUT
#endif	// ifdef CPI_AUT
#ifdef	MOT_IDENT
	ENA_AUTO_IDEN
#endif	//	ifdef MOT_IDENT
#ifdef OSC_WIN
	ENA_OSCI_WIND
#endif	// ifdef OSC_WIN

	// rpm max
	RPM_MAX_CPY = RPM_MAX;
	f32_rpm_max = RPM_MAX_CPY;
	f32_max_speed = RPM_MAX_CPY * f32_krpmocp;	// max angular speed
	f32_max_s_trip = 1.5f * f32_max_speed;

	// rpm min
	RPM_MIN_CPY = RPM_MIN;
	f32_rpm_min = RPM_MIN_CPY;
	f32_min_speed = RPM_MIN_CPY * f32_krpmocp;	// min angular speed
	f32_min_s_trip = 0.5f * f32_min_speed;

	// startup time in internal units calculation, and delta speed every startup step
	SUP_TIM_CPY = SUP_TIM;
	f32_delta_om = f32_min_speed * (f32_sam_per_s / SUP_TIM_CPY);	// to be added to speed at every step

	// acceleration ramp
	R_ACC_CPY = R_ACC;				// rpm/s
	f32_r_acc = R_ACC_CPY * f32_mloop_per_s;	// rpm/one_loop_time

	// deceleration ramp
	R_DEC_CPY = R_DEC;				// rpm/s
	f32_r_dec = R_DEC_CPY * f32_mloop_per_s;	// rpm/one_loop_time

	// startup current
	I_START_CPY = I_START;
	f32_i_start = I_START_CPY;
	f32_is_rup = f32_i_start * f32_sam_per_s * ((float32_t)(1.0 / STUP_CUR_RT_S));
	f32_is_rdw = f32_i_start * f32_sam_per_s * ((float32_t)(1.0 / STUP_CUR_FT_S));

	// maximum current
	I_MAX_CPY = I_MAX;
	f32_i_max = I_MAX_CPY;
#ifdef FLUXWEAK
	f32_id_max = f32_i_max * K_MARGI_FW;
	f32_intid_rup = ((float32_t)DCUR_DT) * f32_sam_per_s;	// id integral variation ramp up in flux weakening
	f32_intid_rdw = ((float32_t)DCUR_DT) * f32_sam_per_s;	// id integral variation ramp dw in flux weakening
#endif	// ifdef FLUXWEAK

	// stator resistance
	R_STA_CPY = R_STA;
	f32_r_sta = R_STA_CPY;

	// synchronous inductance
	L_SQ_CPY = L_SQ;
	f32_l_sq = L_SQ_CPY;

	// permanent magnets flux
	PM_FLX_CPY = PM_FLX;
	f32_pm_fl = PM_FLX_CPY;

	// flux amplitude feedback gain (used in case of exact integration estimation algo)
	FB_GAIN_CPY = FB_GAIN;
	f32_fb_gain = FB_GAIN_CPY;

	// flux estimation filter time constant tau (used in case of approximate integration estimation algo)
	FL_FTAU_CPY = FL_FTAU;
	f32_fl_lpco_hz = (1.0f / (FTWOPI * FL_FTAU_CPY));

	// PI gains
	KP_CUR_CPY = KP_CUR;
	f32_kp_cur = KP_CUR_CPY;					// [Ohm]
	KI_CUR_CPY = KI_CUR;
	f32_ki_cur = KI_CUR_CPY * f32_sam_per_s;	// parameter referred to continuous time [Ohm/sec], internal value to discrete time [Ohm]
	KP_VEL_CPY = KP_VEL;
	f32_kp_vel = KP_VEL_CPY;					// [Ampere*sec/rad]
	KI_VEL_CPY = KI_VEL;
	f32_ki_vel = KI_VEL_CPY * f32_sam_per_s;	// parameter referred to continuous time [Ampere/rad], internal value to discrete time [Ampere*sec/rad]

	// flux and speed estimation functions init
#ifndef APP_INT
	McrpLibf_FluxEst_SetParam(f32_r_sta, f32_l_sq, f32_pm_fl, f32_fb_gain, f32_sam_per_s, SPEED_LP_CUTOFF_HZ);
#else	// ifndef APP_INT
	McrpLibf_FluxEstA_SetParam(f32_r_sta, f32_l_sq, f32_pm_fl, f32_fl_lpco_hz, f32_sam_per_s, SPEED_LP_CUTOFF_HZ);
#endif	// else ifndef APP_INT

	// speed ramp state machine
	ramp_status = STOP_0;
	SET_STOPPED
	RES_STARTED

	// phase offset
	f32_Phase_offset = 0.0f;

#ifdef MOT_IDENT
	// motor identification variables
	f32_ial_1 = f32_i_max * HIG_CUR_RATIO;
	f32_ial_2 = f32_i_max * MID_CUR_RATIO;
	f32_ial_3 = f32_i_max * LOW_CUR_RATIO;
	f32_ial_r = f32_ial_1 * f32_sam_per_ms * ((float32_t)(1.0 / CUR_RISE_TIME_MS));
	f32_iden_om = ((float32_t)(TWOPI * IDENT_SPEED_HZ));
	f32_iden_or = f32_iden_om * f32_sam_per_ms * ((float32_t)(1.0f / SPE_RISE_TIME_MS));
#endif	// ifdef MOT_IDENT

	// buffers
	for(u32i = 0; u32i < OUTBUF_SDIM; u32i++)
	{
		au16_rbuf[u32i] = 0;
		au16_wbuf[u32i] = 0;
	}

#ifdef OSC_WIN
	// oscilloscope window variables
	if(0.0001f < f32_i_max)
	{
		f32_imax1000 = 1000.0f / f32_i_max;
	}
	else
	{
		f32_imax1000 = 1000000.0f;
	}
	s16_selmem = 0;
	s16_bcnt0 = 0;
	s16_bcnt1 = 0;
	s16_bcntm = 0;
	f32_phmem = 0.0f;
	s16_sample = 0;
	RES_TRGED
	RES_RBUSY
	RES_CSELD
#endif	// ifdef OSC_WIN

	// mode
	SET_NORM_MODE

}

/******************************************************************************
Function:		MC_UpdPar
Description:	internal constants update, if parameters have been changed
Input:			nothing
Output:			nothing
Modifies:		many internal variables, using eeprom parameters values
******************************************************************************/
void MC_UpdPar(void)
{
	uint8_t	u8flg = 0;

	if (RPM_MAX_CPY != RPM_MAX)
    {
		RPM_MAX_CPY = RPM_MAX;
		f32_rpm_max = RPM_MAX_CPY;
		f32_max_speed = RPM_MAX_CPY * f32_krpmocp;	// max angular speed
		f32_max_s_trip = 1.5f * f32_max_speed;
	}

	if (RPM_MIN_CPY != RPM_MIN)
    {
		RPM_MIN_CPY = RPM_MIN;
		f32_rpm_min = RPM_MIN_CPY;
		f32_min_speed = RPM_MIN_CPY * f32_krpmocp;	// min angular speed
		f32_min_s_trip = 0.5f * f32_min_speed;
		SUP_TIM_CPY = SUP_TIM + 1;
	}

	if (SUP_TIM_CPY != SUP_TIM)
    {
		SUP_TIM_CPY = SUP_TIM;
		f32_delta_om = f32_min_speed * (f32_sam_per_s / SUP_TIM_CPY);	// to be added to speed at every step
	}

	if (R_ACC_CPY != R_ACC)
	{
		R_ACC_CPY = R_ACC;				// rpm/s
		f32_r_acc = R_ACC_CPY * f32_mloop_per_s;	// rpm/one_loop_time
	}

	if (R_DEC_CPY != R_DEC)
	{
		R_DEC_CPY = R_DEC;				// rpm/s
		f32_r_dec = R_DEC_CPY * f32_mloop_per_s;	// rpm/one_loop_time
	}

	if (I_START_CPY != I_START)
	{
		I_START_CPY = I_START;
		f32_i_start = I_START_CPY;
		f32_is_rup = f32_i_start * f32_sam_per_s * ((float32_t)(1.0 / STUP_CUR_RT_S));
		f32_is_rdw = f32_i_start * f32_sam_per_s * ((float32_t)(1.0 / STUP_CUR_FT_S));
	}

	if (I_MAX_CPY != I_MAX)
	{
		I_MAX_CPY = I_MAX;
		f32_i_max = I_MAX_CPY;
#ifdef FLUXWEAK
		f32_id_max = f32_i_max * K_MARGI_FW;
#endif	// ifdef FLUXWEAK
#ifdef MOT_IDENT
		f32_ial_1 = f32_i_max * HIG_CUR_RATIO;
		f32_ial_2 = f32_i_max * MID_CUR_RATIO;
		f32_ial_3 = f32_i_max * LOW_CUR_RATIO;
		f32_ial_r = f32_ial_1 * f32_sam_per_ms * ((float32_t)(1.0f / CUR_RISE_TIME_MS));
#endif	// ifdef MOT_IDENT
#ifdef OSC_WIN
	if(0.0001f < f32_i_max)
	{
		f32_imax1000 = 1000.0f / f32_i_max;
	}
	else
	{
		f32_imax1000 = 1000000.0f;
	}
#endif	// ifdef OSC_WIN
	}

	if (R_STA_CPY != R_STA)
	{
		R_STA_CPY = R_STA;
		f32_r_sta = R_STA_CPY;
		u8flg = 255;
	}

	if (L_SQ_CPY != L_SQ)
	{
		L_SQ_CPY = L_SQ;
		f32_l_sq = L_SQ_CPY;
		u8flg = 255;
	}

	if (PM_FLX_CPY != PM_FLX)
	{
		PM_FLX_CPY = PM_FLX;
		f32_pm_fl = PM_FLX_CPY;
		u8flg = 255;
	}

	if (FB_GAIN_CPY != FB_GAIN)
	{
		FB_GAIN_CPY = FB_GAIN;
		f32_fb_gain = FB_GAIN_CPY;
		u8flg = 255;
	}

	if (FL_FTAU_CPY != FL_FTAU)
	{
		FL_FTAU_CPY = FL_FTAU;
		f32_fl_lpco_hz = (1.0f / (FTWOPI * FL_FTAU_CPY));
		u8flg = 255;
	}

	// flux and speed estimation functions init
	if (0 != u8flg)
	{
#ifndef APP_INT
		McrpLibf_FluxEst_SetParam(f32_r_sta, f32_l_sq, f32_pm_fl, f32_fb_gain, f32_sam_per_s, SPEED_LP_CUTOFF_HZ);
#else	// ifndef APP_INT
		McrpLibf_FluxEstA_SetParam(f32_r_sta, f32_l_sq, f32_pm_fl, f32_fl_lpco_hz, f32_sam_per_s, SPEED_LP_CUTOFF_HZ);
#endif	// else ifndef APP_INT
	}

	if (KP_CUR_CPY != KP_CUR)
	{
		KP_CUR_CPY = KP_CUR;
		f32_kp_cur = KP_CUR_CPY;
	}

	if (KI_CUR_CPY != KI_CUR)
	{
		KI_CUR_CPY = KI_CUR;
		f32_ki_cur = KI_CUR_CPY * f32_sam_per_s;
	}

	if (KP_VEL_CPY != KP_VEL)
	{
		KP_VEL_CPY = KP_VEL;
		f32_kp_vel = KP_VEL_CPY;
	}

	if (KI_VEL_CPY != KI_VEL)
	{
		KI_VEL_CPY = KI_VEL;
		f32_ki_vel = KI_VEL_CPY * f32_sam_per_s;
	}

	// phase offset
	if(0.0f > f32_omrif)
	{
		f32_Phase_offset = PHA_OFF * (-FTWOPI / 360.0f);
	}
	else if(0.0f < f32_omrif)
	{
		f32_Phase_offset = PHA_OFF * (FTWOPI / 360.0f);
	}
	else
	{
		f32_Phase_offset = 0.0f;
	}

}

/******************************************************************************
Function:		MC_ConFil
Description:	management of measured values, to give them to the GUI
Input:			nothing
Output:			nothing
Modifies:		interface variables defined in userif.h
******************************************************************************/
void MC_ConFil(void)
{
    static uint16_t u16_blnk_cnt;

	// speed reference
	UIF_R.var.rif = f32_rpmrif_y;

	// mechanical speed
	f32_mec_rpm = f32_omf * f32_ukrpmocp;
	UIF_R.var.rpm = f32_mec_rpm;

	// imposed frequency
	f32_freq = f32_omf * ((float32_t) KOF);
	UIF_R.var.fre = f32_freq;				// [Hz]

	// direct current
	UIF_R.var.id = f32_idmf;				// [Apk]

	// quadrature current
	UIF_R.var.iq = f32_iqmf;				// [Apk]

	// direct voltage
	UIF_R.var.vd = f32_vdmf;				// [Vstar_pk]

	// quadrature voltage
	UIF_R.var.vq = f32_vqmf;				// [Vstar_pk]

	// bus voltage
	UIF_R.var.vb = f32_vb_fil;				// [V]

	// total current
	McrpLibf_xy_rt(f32_idmf, f32_iqmf, &f32_f_amp_i, &f32_f_arg_i);
	UIF_R.var.toti = f32_f_amp_i;			// [Apk]

	// total voltage
	McrpLibf_xy_rt(f32_vdmf, f32_vqmf, &f32_f_amp_v, &f32_f_arg_v);
	UIF_R.var.totv = f32_f_amp_v;			// [Vstar_pk]

	// dummy
#ifndef APP_INT
	f32_FluxAmp = McrpLibf_FluxEst_GetAmp();
#else	// APP_INT
	f32_FluxAmp = McrpLibf_FluxEstA_GetAmp();
#endif	// APP_INT
	UIF_R.var.du0 = f32_FluxAmp;			// [Web]
//	UIF_R.var.du0 = ((float32_t)(360 / TWOPI)) * McrpLibf_AngleNrm(f32_f_arg_v - f32_f_arg_i);

	// led blink
	if(0 < u16_blnk_cnt)
	{
	    --u16_blnk_cnt;
	}
	else
	{
#ifndef INTERRUPT_TIME_DEBUG
#ifndef MAINLOOP_TIME_DEBUG
	    TOGGLE_LED          // R_IOPORT3->PCNTR1 ^= 0x00800000;
#endif  // ifndef MAINLOOP_TIME_DEBUG
#endif  // ifndef INTERRUPT_TIME_DEBUG
	    if(0 == UIF_R.var.all)
	    {
	        u16_blnk_cnt = 40;  // slow (800ms period) blinking
	    }
	    else
	    {
	        u16_blnk_cnt = 10;  // fast (200ms period) blinking
	    }
	}

}

/******************************************************************************
Function:		MC_SpeedPI
Description:	Speed PI control
Input:			reference speed f32wr,
				measured speed f32wm,
				maximum output f32imx,
				integral term memory address af32i
Output:			control output
Modifies:		integral memory *af32i
******************************************************************************/
float32_t MC_SpeedPI(float32_t f32wr, float32_t f32wm, float32_t f32imx, float32_t *af32i)
{
	float32_t f32a = f32wr - f32wm;

#ifdef SPEEDPI_ERRCLAMP
	if(f32_maxerr < f32a)
	{
		f32a = f32_maxerr;
	}
	else if(-f32_maxerr > f32a)
	{
		f32a = -f32_maxerr;
	}
#endif	// ifdef SPEEDPI_ERRCLAMP

	return(McrpLibf_PIW(f32a, f32imx, -f32imx, f32_kp_vel, f32_ki_vel, af32i));
}

/******************************************************************************
Function:		MC_IdIqPI
Description:	Current PI control
Input:			reference direct current f32ird,
				measured direct current f32imd,
				reference quadrature current f32irq,
				measured quadrature current f32imq,
				maximum allowable output f32vmx,
				direct voltage output address af32vd,
				quadrature voltage output address af32vq,
				direct integral memory address af32iid,
				quadrature integral memory address af32iiq
Output:			nothing
Modifies:		control output voltage components *af32vd, *af32vq
				integral memories *af32iid, *af32iiq
******************************************************************************/
void MC_IdIqPI(	float32_t f32ird, float32_t f32imd, float32_t f32irq, float32_t f32imq, float32_t f32vmx,
				float32_t *af32vd, float32_t *af32vq, float32_t *af32iid, float32_t *af32iiq)
{
	float32_t f32a;

	if (0.0f >= f32vmx)
	{
		*af32vd = 0.0f;
		*af32vq = 0.0f;
		*af32iid = 0.0f;
		*af32iiq = 0.0f;
		return;
	}

	// id control
	f32a = f32vmx * K_MARGV_DQ;				// vmaxd
	(*af32vd) = McrpLibf_PIW((f32ird - f32imd), f32a, -f32a, f32_kp_cur, f32_ki_cur, af32iid);

	// iq control
	f32a = MC_VecComp(f32vmx, (*af32vd));	// vmaxq
	(*af32vq) = McrpLibf_PIW((f32irq - f32imq), f32a, -f32a, f32_kp_cur, f32_ki_cur, af32iiq);
}

#ifdef CPI_TUN
/******************************************************************************
Function:		MC_CurPITun
Description:	creation of a current step, to manually tune the current PIs
Input:			nothing
Output:			nothing
******************************************************************************/
void MC_CurPITun(void)
{
	if (MAX_SAM_CNT > u16_pit_c)
	{
		f32_idr = f32_i_max * f32_ratio;
		f32_iqr = 0.0f;
		if (50 > u16_pit_c)				// 50 samples	0, .., 49, and 50 interrupts
		{
			au16_rbuf[u16_pit_c++] = 1000.0f * f32_idm;
			f32_cur_ampf = 0.0f;
			u16_pit_s = 0;
		}
		else if (65 > u16_pit_c)			// 15 samples	50, .., 64, and 45 interrupts
		{
			if (3 > u16_pit_s)
			{
				u16_pit_s++;
				f32_cur_ampf = f32_cur_ampf + f32_idm;
			}
			else
			{
				au16_rbuf[u16_pit_c++] = 1000.0f * f32_cur_ampf / u16_pit_s;
				f32_cur_ampf = 0.0f;
				u16_pit_s = 0;
			}
		}
		else if (80 > u16_pit_c)			// 15 samples	65, .., 79, and 135 interrupts
		{
			if (9 > u16_pit_s)
			{
				u16_pit_s++;
				f32_cur_ampf = f32_cur_ampf + f32_idm;
			}
			else
			{
				au16_rbuf[u16_pit_c++] = 1000.0f * f32_cur_ampf / u16_pit_s;
				f32_cur_ampf = 0.0f;
				u16_pit_s = 0;
			}
		}
		else if (95 > u16_pit_c)			// 15 samples	80, .., 94, and 405 interrupts
		{
			if (27 > u16_pit_s)
			{
				u16_pit_s++;
				f32_cur_ampf = f32_cur_ampf + f32_idm;
			}
			else
			{
				au16_rbuf[u16_pit_c++] = 1000.0f * f32_cur_ampf / u16_pit_s;
				f32_cur_ampf = 0.0f;
				u16_pit_s = 0;
			}
		}
		else if (110 > u16_pit_c)		// 15 samples	95, .., 109, and 1215 interrupts
		{
			if (81 > u16_pit_s)
			{
				u16_pit_s++;
				f32_cur_ampf = f32_cur_ampf + f32_idm;
			}
			else
			{
				au16_rbuf[u16_pit_c++] = 1000.0f * f32_cur_ampf / u16_pit_s;
				f32_cur_ampf = 0.0f;
				u16_pit_s = 0;
			}
		}
		else /*if(MAX_SAM_CNT > u16_pit_c)*/	// remaining samples	110, .., 1XX, and YYYY interrupts
		{
			if (243 > u16_pit_s)
			{
				u16_pit_s++;
				f32_cur_ampf = f32_cur_ampf + f32_idm;
			}
			else
			{
				au16_rbuf[u16_pit_c++] = 1000.0f * f32_cur_ampf / u16_pit_s;
				f32_cur_ampf = 0.0f;
				u16_pit_s = 0;
			}
		}
	}		// total: MAX_SAM_CNT samples and less than 5495 interrupts (around 343ms @16kHz)
	else	// u16_pit_c >= MAX_SAM_CNT	(MAX_SAM_CNT <= 125)
	{
		f32_idr = 0.0f;
		f32_iqr = 0.0f;
		if (150 > u16_pit_c)
		{
			u16_pit_c++;
		}
	}
}
#endif	// ifdef CPI_TUN

#ifdef CPI_AUT
/******************************************************************************
Function:		MC_CurPIAuto
Description:	creation of a voltage profile, used to automatically tune
				the current PIs
Input:			nothing
Output:			nothing
******************************************************************************/
void MC_CurPIAuto(void)
{
	f32_idr = 0.0f;
	f32_iqr = 0.0f;
	PIT_SetCur(f32_idm, f32_iqm);
	switch (u16_pitun_status)
	{
		case R_UP:		// voltage ramp up till current reaches rated value
			if (PIT_RampUp(&f32_v_req))
			{
				u16_pitun_status = R_MS;
			}
			break;
		case R_MS:		// wait some time, measuring the current level (filtered)
			if (PIT_Measure(&f32_v_req))
			{
				u16_pitun_status = R_DW;
			}
			break;
		case R_DW:		// voltage ramp down
			if (PIT_RampDown(&f32_v_req))
			{
				u16_pitun_status = R_ST;
			}
			break;
		case R_ST:		// voltage step
			if (PIT_VoltStep(&f32_v_req))
			{
				u16_pitun_status = R_EN;
			}
			break;		// finished
		case R_EN:
		default:
			f32_v_req = 0.0f;
			break;
	}
	f32_vdc = f32_v_req;
	f32_vqc = 0.0f;
}
#endif	// ifdef CPI_AUT

#ifdef MOT_IDENT
/******************************************************************************
Function:		MC_MotIdent
Description:	motor identification sequence
Input:			nothing
Output:			nothing
******************************************************************************/
void MC_MotIdent(void)
{

	f32_iqr = 0.0f;
	MC_Ident_SetMea(f32_idm, f32_iqm, f32_vdm, f32_vqm, f32_omegae);

	if (0 > s32_x_cnt)
	{
		s32_x_cnt = 0;
	}
	else if (X_TIM1 >= s32_x_cnt)	// id ramp up
	{
		if (f32_idr < (f32_ial_1 - f32_ial_r))
		{
			f32_idr = f32_idr + f32_ial_r;
		}
		else
		{
			f32_idr = f32_ial_1;
		}
		f32_omegae = 0.0f;
		f32_SystemPhase = 0.0f;
	}
	else if (X_TIM2 > s32_x_cnt)		// first measurement cycle
	{
		MC_Ident_UpdAcc();
		f32_idr = f32_ial_1;
		f32_omegae = 0.0f;
		f32_SystemPhase = 0.0f;
	}
	else if (X_TIM2 == s32_x_cnt)	// first measurement acknowledge
	{
		MC_Ident_Step1();
		f32_idr = f32_ial_1;
		f32_omegae = 0.0f;
		f32_SystemPhase = 0.0f;
	}
	else if (X_TIM3 >= s32_x_cnt)	// id ramp dw
	{
		if (f32_idr > (f32_ial_2 + f32_ial_r))
		{
			f32_idr = f32_idr - f32_ial_r;
		}
		else
		{
			f32_idr = f32_ial_2;
		}
		f32_omegae = 0.0f;
		f32_SystemPhase = 0.0f;
	}
	else if (X_TIM4 > s32_x_cnt)		// second measurement cycle
	{
		MC_Ident_UpdAcc();
		f32_idr = f32_ial_2;
		f32_omegae = 0.0f;
		f32_SystemPhase = 0.0f;
	}
	else if (X_TIM4 == s32_x_cnt)	// second measurement acknowledge
	{
		MC_Ident_Step2();
		f32_idr = f32_ial_2;
		f32_omegae = 0.0f;
		f32_SystemPhase = 0.0f;
	}
	else if (X_TIM5 >= s32_x_cnt)	// speed ramp
	{
		f32_idr = f32_ial_2;
		if (f32_omegae < (f32_iden_om - f32_iden_or))
		{
			f32_omegae = f32_omegae + f32_iden_or;
		}
		else
		{
			f32_omegae = f32_iden_om;
		}
		f32_SystemPhase = McrpLibf_AngleNrm(f32_SystemPhase + (f32_omegae * f32_sam_per_s));
	}
	else if (X_TIM6 > s32_x_cnt)		// third measurement cycle
	{
		MC_Ident_UpdAcc();
		f32_idr = f32_ial_2;
		f32_omegae = f32_iden_om;
		f32_SystemPhase = McrpLibf_AngleNrm(f32_SystemPhase + (f32_omegae * f32_sam_per_s));
	}
	else if (X_TIM6 == s32_x_cnt)	// third measurement acknowledge
	{
		MC_Ident_Step3();
		f32_idr = f32_ial_2;
		f32_omegae = f32_iden_om;
		f32_SystemPhase = McrpLibf_AngleNrm(f32_SystemPhase + (f32_omegae * f32_sam_per_s));
	}
	else if (X_TIM7 >= s32_x_cnt)	// id ramp dw
	{
		if (f32_idr > (f32_ial_3 + f32_ial_r))
		{
			f32_idr = f32_idr - f32_ial_r;
		}
		else
		{
			f32_idr = f32_ial_3;
		}
		f32_omegae = f32_iden_om;
		f32_SystemPhase = McrpLibf_AngleNrm(f32_SystemPhase + (f32_omegae * f32_sam_per_s));
	}
	else if (X_TIM8 > s32_x_cnt)		// fourth measurement cycle
	{
		MC_Ident_UpdAcc();
		f32_idr = f32_ial_3;
		f32_omegae = f32_iden_om;
		f32_SystemPhase = McrpLibf_AngleNrm(f32_SystemPhase + (f32_omegae * f32_sam_per_s));
	}
	else if (X_TIM8 == s32_x_cnt)	// fourth measurement acknowledge
	{
		MC_Ident_Step4();
		f32_idr = 0.0f;
		f32_omegae = 0.0f;
		f32_SystemPhase = 0.0f;
	}
	else						// waiting end
	{
		f32_idr = 0.0f;
		f32_omegae = 0.0f;
		f32_SystemPhase = 0.0f;
	}
	if (0x7FFFFFFF > s32_x_cnt)
	{
		s32_x_cnt++;
	}
}
#endif	// ifdef MOT_IDENT

/******************************************************************************
Function:		MC_WorkMan
Description:	Working modes management function (speed ramps etc.)
Input:			nothing
Output:			nothing
Modifies:		internal speed reference
******************************************************************************/
void MC_WorkMan(void)
{
	static float32_t	f32_speed_ref_mem;
	float32_t 		    f32a;

	// new speed reference sampling
	f32_speed_ref = UIF_W.var.rif;

	// alarm management
	if (0 != UIF_R.var.all)
	{
		MC_StopOut();
		RES_STARTED
		SET_STOPPED
		RES_DECEL
		f32_rpmrif_x = 0.0f;
		f32_rpmrif_y = 0.0f;
		f32_omrif = 0.0f;
		f32_omrif_abs = 0.0f;
		ramp_status = STOP_0;
		if ((TRIP_ALL  == UIF_R.var.all)	||
			(FAULT_ALL == UIF_R.var.all)	||
			(OVC_ALL   == UIF_R.var.all)	)
		{
			if (((-0.9f < f32_speed_ref)	 && (0.9f > f32_speed_ref))	||
				(( 0.0f > f32_speed_ref_mem) && (0.0f < f32_speed_ref))	||
				(( 0.0f < f32_speed_ref_mem) && (0.0f > f32_speed_ref))	)
			{
				UIF_R.var.all = 0;
				RES_ALRM_ON
			}
		}
		f32_speed_ref_mem = f32_speed_ref;
		SET_NORM_MODE		// on alarm, normal mode is reestablished
		RES_STA_BSY
		SET_END_NOK
		RES_GUI_TRI
		RES_GUI_STP
		return;
	}
	f32_speed_ref_mem = f32_speed_ref;

	// working mode requests management
	if (NOT_OUT_ENA)	// all requests are taken into consideration only if COM_OFF
	{
#ifdef CPI_TUN
		if (CURPI_TUN_REQ)
		{
			SET_CURPI_TUN
		}
		else
#endif	// ifdef CPI_TUN
#ifdef CPI_AUT
		if (CURPI_AUT_REQ)
		{
			SET_CURPI_AUT
		}
		else
#endif	// ifdef CPI_AUT
#ifdef MOT_IDENT
		if (AUTO_IDEN_REQ)
		{
			SET_AUTO_IDEN
		}
#endif	// ifdef MOT_IDENT
		if (NOT_NORM_MODE)
		{
			RES_STARTED
			SET_STOPPED
			RES_DECEL
			f32_rpmrif_x = 0.0f;
			f32_rpmrif_y = 0.0f;
			f32_omrif = 0.0f;
			f32_omrif_abs = 0.0f;
			ramp_status = STOP_0;
			if (NORM_MODE_REQ)
			{
				UIF_R.var.kpi = KP_CUR;
				UIF_R.var.kii = KI_CUR;
				UIF_R.var.rs = R_STA;
				UIF_R.var.ls = L_SQ;
				UIF_R.var.fl = PM_FLX;
				RES_STA_BSY
				SET_NORM_MODE
			}
		}
	}

#ifdef CPI_TUN
	// Manual current PI tuning mode
	if (CURPI_TUN)
	{
		if (NOT_OUT_ENA)
		{
			if (GUI_TRI)
			{
				f32_ratio = UIF_W.var.cra;		// ratio of maximum current
				if (0.0f > f32_ratio)
				{
					f32_ratio = 0.0f;
				}
				else if (1.0f < f32_ratio)
				{
					f32_ratio = 1.0f;
				}
				au16_rbuf = ((uint16_t *)(outbuf.ss));
				au16_wbuf = ((uint16_t *)(outbuf1.ss));
				au16_pbuf = au16_rbuf;
				u16_pit_c = 0;
				SET_STA_BSY
				MC_StartOut();
			}
		}
		else if (GUI_STP)
		{
			MC_StopOut();
			SET_END_NOK
			RES_STA_BSY
		}
		else if (130 <= u16_pit_c)	// 130 surely > MAX_SAM_CNT
		{
			MC_StopOut();
			RES_END_NOK
			RES_STA_BSY
		}
		else
		{
			SET_STA_BSY
			MC_StartOut();
		}
		RES_GUI_TRI
		RES_GUI_STP
		return;
	}
#endif	// ifdef CPI_TUN

#ifdef CPI_AUT
	// Automatic current PI tuning mode
	if (CURPI_AUT)
	{
		if (NOT_OUT_ENA)
		{
			if (GUI_TRI)
			{
				PIT_IniVar(f32_i_max * I_PIT_RATIO, f32_vb_fil, f32_sam_per_s);
				f32_v_req = 0.0f;
				u16_pitun_status = R_UP;
				SET_STA_BSY
				MC_StartOut();
			}
		}
		else if (GUI_STP)
		{
			MC_StopOut();
			SET_END_NOK
			RES_STA_BSY
		}
		else if (R_EN <= u16_pitun_status)
		{
			MC_StopOut();
			UIF_R.var.kpi = PIT_ReadKP();
			UIF_R.var.kii = f32_sam_fre_Hz * PIT_ReadKI();	// identification returns directly the discrete time value in [Ohm], parameter is the continuous time value in [Ohm/sec]
			RES_END_NOK
			RES_STA_BSY
		}
		else
		{
			SET_STA_BSY
			MC_StartOut();
		}
		RES_GUI_TRI
		RES_GUI_STP
		return;
	}
#endif	// ifdef CPI_AUT

#ifdef MOT_IDENT
	// Motor parameters auto identification mode
	if (AUTO_IDEN)
	{
		if (NOT_OUT_ENA)
		{
			if (GUI_TRI)
			{
				INI_IDENT
				SET_STA_BSY
				MC_StartOut();
			}
		}
		else if (GUI_STP)
		{
			MC_StopOut();
			SET_END_NOK
			RES_STA_BSY
		}
		else if (IDENT_ENDED)
		{
			MC_StopOut();
			MC_Ident_CalMotPar(&f32_iden_rs, &f32_iden_ls, &f32_iden_fl);
			UIF_R.var.rs = f32_iden_rs;
			UIF_R.var.ls = f32_iden_ls;
			UIF_R.var.fl = f32_iden_fl;
			RES_END_NOK
			RES_STA_BSY
		}
		else
		{
			SET_STA_BSY
			MC_StartOut();
		}
		RES_GUI_TRI
		RES_GUI_STP
		return;
	}
#endif	// ifdef MOT_IDENT

	// From here normal mode management
	RES_END_NOK

	// preliminary reference management
#ifdef	ROT_REV
	f32a = -f32_speed_ref;
#else	// ROT_REV
	f32a = f32_speed_ref;
#endif	// ROT_REV
	if (f32a >= f32_rpm_min)
	{
		if (f32a > f32_rpm_max)
		{
			f32a = f32_rpm_max;
		}
	}
	else if (-f32_rpm_min >= f32a)
	{
		if (-f32_rpm_max > f32a)
		{
			f32a = -f32_rpm_max;
		}
	}
	else
	{
		f32a = 0.0f;
#ifdef DEBUG_OPENLOOP
		if (DEB_OPNLP)
		{
			MC_StopOut();
			f32_rpmrif_x = 0.0f;
			f32_rpmrif_y = 0.0f;
			f32_omrif = 0.0f;
			f32_omrif_abs = 0.0f;
			ramp_status = STOP_0;
			RES_STARTED
			SET_STOPPED
			RES_DECEL
			return;
		}
#endif	// ifdef DEBUG_OPENLOOP
	}

#ifdef DCCUR_SETTING
	// DC current vector setting
	if (DCCUR_SET)
	{
		f32_rpmrif_x = 0.0f;
		f32_rpmrif_y = 0.0f;
		f32_omrif = 0.0f;
		f32_omrif_abs = 0.0f;
		ramp_status = STOP_0;
		RES_STARTED
		SET_STOPPED
		RES_DECEL
		if ((-0.99f < f32a) && (0.99f > f32a))
		{
			MC_StopOut();
		}
		else
		{
			MC_StartOut();
		}
		return;
	}
#endif	// ifdef DCCUR_SETTING

	// ramp management state-machine
	switch (ramp_status)
	{
		case START_P:
			RES_STOPPED
			RES_DECEL
			MC_StartOut();
			f32_rpmrif_x = f32_rpm_min;
			f32_rpmrif_y = f32_rpm_min;
			if (f32a < f32_rpm_min)
			{
				ramp_status = STOP_W;
				if (-f32_rpm_min >= f32a)
				{
					f32_rpmrif_x = -f32_rpm_min;
				}
				else
				{
					f32_rpmrif_x = 0.0f;
				}
				break;
			}
			else if (NOT_STARTED)
			{
				break;
			}
			ramp_status = GO_P;
			break;
		case GO_P:
			RES_STOPPED
			MC_StartOut();
			if (f32a < f32_rpm_min)
			{
				if (f32_rpmrif_y <= f32_rpm_min)
				{
					f32_rpmrif_y = 0.0f;
					ramp_status = STOP_W;
					if (-f32_rpm_min >= f32a)
					{
						f32_rpmrif_x = -f32_rpm_min;
					}
					else
					{
						f32_rpmrif_x = 0.0f;
					}
					break;
				}
				f32_rpmrif_x = f32_rpm_min;
			}
			else
			{
				f32_rpmrif_x = f32a;
			}
			f32a = f32_rpmrif_y;
			if (f32a < (f32_rpmrif_x - f32_r_acc))
			{
				RES_DECEL
				f32a = f32a + f32_r_acc;
			}
			else if (f32a > (f32_rpmrif_x + f32_r_dec))
			{
				SET_DECEL
				if (NOT_RAMPLIM)
				{
				f32a = f32a - f32_r_dec;
				}
			}
			else
			{
				RES_DECEL
				f32a = f32_rpmrif_x;
			}
			f32_rpmrif_y = f32a;
			break;
		case START_N:
			RES_STOPPED
			RES_DECEL
			MC_StartOut();
			f32_rpmrif_x = -f32_rpm_min;
			f32_rpmrif_y = -f32_rpm_min;
			if (-f32_rpm_min < f32a)
			{
				ramp_status = STOP_W;
				if (f32a >= f32_rpm_min)
				{
					f32_rpmrif_x = f32_rpm_min;
				}
				else
				{
					f32_rpmrif_x = 0.0f;
				}
				break;
			}
			else if (NOT_STARTED)
			{
				break;
			}
			ramp_status = GO_N;
			break;
		case GO_N:
			RES_STOPPED
			MC_StartOut();
			if (-f32_rpm_min < f32a)
			{
				if (-f32_rpm_min <= f32_rpmrif_y)
				{
					f32_rpmrif_y = 0.0f;
					ramp_status = STOP_W;
					if (f32a >= f32_rpm_min)
					{
						f32_rpmrif_x = f32_rpm_min;
					}
					else
					{
						f32_rpmrif_x = 0.0f;
					}
					break;
				}
				f32_rpmrif_x = -f32_rpm_min;
			}
			else
			{
				f32_rpmrif_x = f32a;
			}
			f32a = f32_rpmrif_y;
			if (f32a > (f32_rpmrif_x + f32_r_acc))
			{
				RES_DECEL
				f32a = f32a - f32_r_acc;
			}
			else if (f32a < (f32_rpmrif_x - f32_r_dec))
			{
				SET_DECEL
				if (NOT_RAMPLIM)
				{
				f32a = f32a + f32_r_dec;
				}
			}
			else
			{
				RES_DECEL
				f32a = f32_rpmrif_x;
			}
			f32_rpmrif_y = f32a;
			break;
		case STOP_W:
			RES_STARTED
			RES_DECEL
			f32_rpmrif_y = 0.0f;
			MC_StartOut();
			if (f32a >= f32_rpm_min)
			{
				f32_rpmrif_x = f32_rpm_min;
				if (STOPPED)
				{
					ramp_status = START_P;
				}
			}
			else if (-f32_rpm_min >= f32a)
			{
				f32_rpmrif_x = -f32_rpm_min;
				if (STOPPED)
				{
					ramp_status = START_N;
				}
			}
			else
			{
				f32_rpmrif_x = 0.0f;
				if (STOPPED)
				{
					ramp_status = STOP_0;
				}
			}
			break;
		case STOP_0:
		default:
			MC_StopOut();
			SET_STOPPED
			RES_STARTED
			RES_DECEL
			f32_rpmrif_x = 0.0f;
			f32_rpmrif_y = 0.0f;
			if (f32a >= f32_rpm_min)
			{
				ramp_status = START_P;
			}
			else if (-f32_rpm_min >= f32a)
			{
				ramp_status = START_N;
			}
			else
			{
				ramp_status = STOP_0;
			}
			break;
	}

	// ramp output update and conversion in rad/sec
	f32_omrif = f32_rpmrif_y * f32_krpmocp;
	f32_omrif_abs = (0.0f > f32_omrif? -f32_omrif: f32_omrif);

}

#ifdef FLUXWEAK
/******************************************************************************
Function:		MC_FluxWeak
Description:	flux weakening algorithm
Input:			voltage margin f32vmf,
				maximum current direct f32imax,
				reference current direct f32iref
				new direct current reference address af32iint
Output:			nothing
Modifies:		new direct current reference *af32iint
******************************************************************************/
void MC_FluxWeak(float32_t f32vmf, float32_t f32imax, float32_t f32iref, float32_t *af32iint)
{

	if ((HVBUS_LIMH < f32_vbus) || (0.0f > f32vmf))
	{
		if (-f32imax < (*af32iint))
		{
			(*af32iint) =  (*af32iint) - f32_intid_rdw;
		}
	}
	else if ((*af32iint) > (f32iref + f32_intid_rdw))
	{
		(*af32iint) = (*af32iint) - f32_intid_rdw;
	}
	else if ((*af32iint) < (f32iref - f32_intid_rup))
	{
		(*af32iint) = (*af32iint) + f32_intid_rup;
	}
	else
	{
		(*af32iint) = f32iref;
	}

}
#endif	// ifdef FLUXWEAK

#ifdef TRIP_CONTROL
/******************************************************************************
Function:		MC_TripCon
Description:	loosing of control of the phase estimation algorithm check
Input:			nothing
Output:			nothing
Note:			uses some global variables
Modifies:		global alarm, if needed
******************************************************************************/
void MC_TripCon(uint32_t max_trip)
{
	if(0 < max_trip)
	{
		if (max_trip <= u16_trip_cnt)
		{
			MC_StopOut();
			UIF_R.var.all = TRIP_ALL;
			SET_ALRM_ON
			u16_trip_cnt = 0;
		}
		else if (GO_P == ramp_status)
		{
			if ((f32_Speed_est < f32_min_s_trip) || (f32_Speed_est > f32_max_s_trip))
			{
				u16_trip_cnt = u16_trip_cnt + 1;
			}
			else if (1 < u16_trip_cnt)
			{
				u16_trip_cnt = u16_trip_cnt - 2;
			}
		}
		else if (GO_N == ramp_status)
		{
			if ((-f32_min_s_trip < f32_Speed_est) || (-f32_max_s_trip > f32_Speed_est))
			{
				u16_trip_cnt = u16_trip_cnt + 1;
			}
			else if (1 < u16_trip_cnt)
			{
				u16_trip_cnt = u16_trip_cnt - 2;
			}
		}
	}
	else
	{
        u16_trip_cnt = 0;
	}
}
#endif	// ifdef TRIP_CONTROL

/******************************************************************************
Function:		g_adc0_runtime_callback
Description:	control interrupt
Input:			nothing
Output:			nothing
Note:			this function is called every sampling instant, and performs
           		the motor control
******************************************************************************/
void g_adc0_runtime_callback(adc_callback_args_t * p_args)
{
    float32_t   f32a;

	SSP_PARAMETER_NOT_USED(p_args);

	// time debug
#ifdef INTERRUPT_TIME_DEBUG
	TURN_ON_LED
#endif  // ifdef INTERRUPT_TIME_DEBUG

	// A/D conversion results management
	MC_ADMan();

	// check POE flags
	MC_CheckPOEAFlags();
	//MC_CheckPOEBFlags();

	if (COM_ON)
	{
		// current transformations (previous angle)
		McrpLibf_alphabeta_dq(f32_iam, f32_ibm, &f32_idm, &f32_iqm);

		// voltage measurements (related to currents measurements)
		f32_vam = f32_vax;
		f32_vbm = f32_vbx;
		f32_vdm = f32_vdx;
		f32_vqm = f32_vqx;
	}	// if(COM_ON)
	else
	{
		f32_iqm = f32_idm = 0.0f;
		f32_vbm = f32_vam = 0.0f;
		f32_vqm = f32_vdm = 0.0f;
	}

	// angle management
	McrpLibf_AngleSet(f32_SystemPhase);		// from here new angle (sin and cos) (conversions etc.)

	// voltages references and duty cycles calc
	if (COM_ON)
	{

#ifdef CPI_AUT
		if (CURPI_AUT)
		{
			MC_CurPIAuto();	// produces directly voltages references vdc, vqc
		}		// if(CURPI_AUT)
		else
#endif	// ifdef CPI_AUT
		{	// normal case: idr, iqr come from previous cycle

			// current PI controls
			MC_IdIqPI(f32_idr, f32_idm, f32_iqr, f32_iqm, f32_vfmax, &f32_vdc, &f32_vqc, &f32_idint, &f32_iqint);
		}	// end of normal case

		// voltage transformations (actual angle)
		McrpLibf_dq_alphabeta(f32_vdc, f32_vqc, &f32_vac, &f32_vbc);
		McrpLibf_alphabeta_uv(f32_vac, f32_vbc, &f32_vuc, &f32_vvc);
		f32_vwc = -f32_vuc - f32_vvc;

		// pwm output generation
		MC_PWMGen(); // uses vuc, vvc, vwc and vbus to calculate duty_u, duty_v, duty_w; updates vuc, vvc, vwc values

		// pwm output registers setting (HERE WE SHOULD BE BEFORE PWM CREST!)
		dutycycle.phases.du = s32_duty_u;
		dutycycle.phases.dv = s32_duty_v;
		dutycycle.phases.dw = s32_duty_w;
		g_mc_timer0.p_api->dutyCyclesSet(g_mc_timer0.p_ctrl, &dutycycle);

		// output voltages in (alpha, beta) frame update after modulation clamping
		McrpLibf_uvw_alphabeta(f32_vux, f32_vvx, f32_vwx, &f32_vax, &f32_vbx);
		McrpLibf_alphabeta_dq(f32_vax, f32_vbx, &f32_vdx, &f32_vqx);

	}
	else	// else of if(COM_ON)
	{
		f32_iqint = f32_idint = 0.0f;
		f32_vqc = f32_vdc = 0.0f;
		f32_vbc = f32_vac = 0.0f;
		f32_vwc = f32_vvc = f32_vuc = 0.0f;
		f32_vqx = f32_vdx = 0.0f;
		f32_vbx = f32_vax = 0.0f;
		f32_vwx = f32_vvx = f32_vux = 0.0f;
		s32_duty_u = s32_duty_v = s32_duty_w = u32_halfper - 1;

		// pwm output registers setting (if outputs enabled, this means the lower arm always on (used for BOOTSTRAP))
		dutycycle.phases.du = s32_duty_u;
		dutycycle.phases.dv = s32_duty_v;
		dutycycle.phases.dw = s32_duty_w;
		g_mc_timer0.p_api->dutyCyclesSet(g_mc_timer0.p_ctrl, &dutycycle);

	}

#ifdef VB_RAMP
	// ramp limitation depending on vbus
	if (DECEL)
	{
		if (HVBUS_LIMH <= f32_vbus)
		{
			SET_RAMPLIM
		}
		else if (HVBUS_LIML > f32_vbus)
		{
			RES_RAMPLIM
		}
	}
	else
	{
		RES_RAMPLIM
	}
#endif	// ifdef VB_RAMP

	// current references calculation, startup, position and speed management
	// the outputs of this block are: next system phase, next d and q current references
	if (COM_ON)
	{

#ifdef FLUXWEAK
		// measured current and voltages vectors amplitude
		//f32_amp_i = sqrtf((f32_iam * f32_iam) + (f32_ibm * f32_ibm));
		f32_amp_v = sqrtf((f32_vam * f32_vam) + (f32_vbm * f32_vbm));
#endif  // ifdef FLUXWEAK

#ifdef CPI_TUN
		if (CURPI_TUN)
		{
			MC_CurPITun();
		}
		else
#endif	// ifdef CPI_TUN

#ifdef MOT_IDENT
		if (AUTO_IDEN)
		{
			MC_MotIdent();
		}
		else
#endif	// ifdef MOT_IDENT

#ifdef DCCUR_SETTING
		if (DCCUR_SET)
		{
			f32_SystemPhase = McrpLibf_AngleNrm(PHA_OFF * ((float32_t)(TWOPI / 360)));
			f32_idr = f32_i_start;
			f32_iqr = 0.0f;
		}
		else
#endif	// ifdef DCCUR_SETTING

		{	// normal behaviour block beginning

#ifdef FLUXWEAK
			// voltage margin for flux weakening calc
			f32_vmarg = (f32_vfmax * K_MARGV_FW) - f32_amp_v;
			if ((0.0f < f32_omegae) && (0.0f >= f32_vqc))
			{
				f32_vmarg = f32_vmarg - f32_vqc;
			}
			else if ((0.0f > f32_omegae) && (0.0f <= f32_vqc))
			{
				f32_vmarg = f32_vmarg + f32_vqc;
			}
			if ((0.0f > f32_vmarg) && (	((0.0f < f32_omegae) && (0.0f >= f32_iqr)) ||
										((0.0f > f32_omegae) && (0.0f <= f32_iqr)) ))
			{
				f32_vmarg = 0.0f;
			}
#endif	// ifdef FLUXWEAK

			// speed reference filter
			f32_f_omrif = f32_f_omrif + ((f32_omrif - f32_f_omrif) * f32_omr_ukt);	// speed reference filtering

			// permanent magnets flux vector position and speed estimation
#ifndef APP_INT
			McrpLibf_FluxEst(f32_vam, f32_vbm, f32_iam, f32_ibm);
			f32_FluxPhase = McrpLibf_FluxEst_GetPhase();
			f32_Speed_est = McrpLibf_FluxEst_GetSpeed();
#else	// ifndef APP_INT
			McrpLibf_FluxEstA(f32_vam, f32_vbm, f32_iam, f32_ibm);
			f32_FluxPhase = McrpLibf_FluxEstA_GetPhase();
			f32_Speed_est = McrpLibf_FluxEstA_GetSpeed();
#endif	// else ifndef APP_INT

			// phase compensation and optional phase offset
			f32_Phase_est = McrpLibf_AngleNrm(f32_FluxPhase + (f32_comptime * f32_Speed_est) + f32_Phase_offset);

			// STARTUP
			if (STOP_W == ramp_status)	// stop control
			{
				f32_itr = f32_idr = f32_iqr = 0.0f;
				u16_startup_cnt = 0;
				SET_STOPPED
				f32_SystemPhase = 0.0f;
				f32_omf = f32_omegae = 0.0f;
			}
			else if (0 == u16_startup_cnt)	// alignment
			{
				f32_SystemPhase = 0.0f;
				f32_omf = f32_omegae = 0.0f;
				if (f32_itr < f32_i_start)
				{
					f32_itr = f32_itr + f32_is_rup;
				}
				else
				{
					f32_itr = f32_i_start;
					u16_startup_cnt = 1;
					f32_FluxPhase = f32_SystemPhase;
#ifndef APP_INT
					McrpLibf_FluxEst_SetFlux(f32_FluxPhase, f32_iam, f32_ibm, f32_omegae);
#else	// ifndef APP_INT
					McrpLibf_FluxEstA_SetFlux(f32_FluxPhase, f32_vam, f32_vbm, f32_iam, f32_ibm, f32_omegae);
#endif	// else ifndef APP_INT
				}
				if (0.0f > f32_omrif)
				{
					McrpLibf_rt_xy(f32_itr, -ISTART_PH, &f32_idr, &f32_iqr);
				}
				else
				{
					McrpLibf_rt_xy(f32_itr, ISTART_PH, &f32_idr, &f32_iqr);
				}
			}
			else if (1 == u16_startup_cnt)	// speed increasing till minimum speed
			{

				if (START_P == ramp_status)
				{
					if (f32_omegae < (f32_min_speed - f32_delta_om))
					{
						f32_omegae = f32_omegae + f32_delta_om;
					}
					else
					{
						f32_omegae = f32_min_speed;
						u16_startup_cnt = 2;
					}
				}
				else if (START_N == ramp_status)
				{
					if (f32_omegae > (f32_delta_om - f32_min_speed))
					{
						f32_omegae = f32_omegae - f32_delta_om;
					}
					else
					{
						f32_omegae = -f32_min_speed;
						u16_startup_cnt = 2;
					}
				}
				f32_omf = f32_omegae;

				// feed-forward angle management
				f32_delta_ph = f32_omegae * f32_sam_per_s;
				f32_SystemPhase = McrpLibf_AngleNrm(f32_SystemPhase + f32_delta_ph);
			}
#ifdef DEBUG_OPENLOOP
			else if (DEB_OPNLP)
			{
	    		f32_SystemPhase = McrpLibf_AngleNrm(f32_SystemPhase + f32_delta_ph);
			}
#endif	// ifdef DEBUG_OPENLOOP
			else if (2 == u16_startup_cnt)	// reference system change
			{
				u16_startup_cnt = 3;
				f32_f_omrif = f32_omf = f32_omegae;

				// control memories calc and setting
				McrpLibf_dq_alphabeta(f32_idint, f32_iqint, &f32_iaint, &f32_ibint);// current PI integral terms in old ref. frame
				McrpLibf_dq_alphabeta(f32_idr, f32_iqr, &f32_iar, &f32_ibr);		// references in old ref. frame
				McrpLibf_AngleSet(f32_Phase_est);									// phase update
				McrpLibf_alphabeta_dq(f32_iaint, f32_ibint, &f32_idint, &f32_iqint);// current PI integral terms in new ref. frame
				McrpLibf_alphabeta_dq(f32_iar, f32_ibr, &f32_idr, &f32_iqr);		// references in new ref. frame
				f32_errint = f32_iqr;												// speed PI integral term

				// phase alignment
				f32_SystemPhase = f32_Phase_est;
				u16_trip_cnt = 0;
				SET_STARTED
				// f32_itr is still equal to f32_i_start
			}
			else if (3 == u16_startup_cnt)	// id decreasing
			{

				// itr ramp down
				if (f32_itr > f32_is_rdw)
				{
					f32_itr = f32_itr - f32_is_rdw;
				}
				else
				{
					f32_itr = 0;
					u16_startup_cnt = 4;
				}

				// phase and speed update
				f32_SystemPhase = f32_Phase_est;
				f32_omegae = f32_Speed_est;

				// speed control
				f32_iqmax = f32_i_max;
				f32_iqr = MC_SpeedPI(f32_f_omrif, f32_omegae, f32_iqmax, &f32_errint);

				// idr calc
				f32a = MC_VecComp(f32_itr, f32_iqr);
				if (f32a < f32_idr)
				{
					f32_idr = f32a;
				}
				else if(-f32a > f32_idr)
				{
					f32_idr = -f32a;
				}
			}
			// STARTUP IS FINISHED, NORMAL CONTROL
			else		// normal sensorless control
			{

#ifdef TRIP_CONTROL
				// position loss control
				MC_TripCon(MAX_TRIP);
#endif	// ifdef TRIP_CONTROL

				// phase and speed update
				f32_SystemPhase = f32_Phase_est;
				f32_omegae = f32_Speed_est;

				// id reference calc
				f32_idrvar = 0.0f;

				// idr and iqmax calc
#ifdef FLUXWEAK
				MC_FluxWeak(f32_vmarg, f32_id_max, f32_idrvar, &f32_idr);
#else	// ifdef FLUXWEAK
				f32_idr = f32_idrvar;
#endif	// else ifdef FLUXWEAK
				f32_iqmax = MC_VecComp(f32_i_max, f32_idr);

				// speed control
#ifdef VB_RAMP
				if (RAMPLIM)
				{
					f32_errint = f32_errint * K_RAMPLIM;
				}
#endif	// ifdef VB_RAMP
				f32_iqr = MC_SpeedPI(f32_f_omrif, f32_omegae, f32_iqmax, &f32_errint);

			}	// end of normal control

		}	// end of startup and normal behaviour

		// visualization filters
		f32_idmf = f32_idmf + ((f32_idm - f32_idmf) * f32_visu_ukt);	// d current
		f32_iqmf = f32_iqmf + ((f32_iqm - f32_iqmf) * f32_visu_ukt);	// q current
		f32_vdmf = f32_vdmf + ((f32_vdm - f32_vdmf) * f32_visu_ukt);	// d voltage
		f32_vqmf = f32_vqmf + ((f32_vqm - f32_vqmf) * f32_visu_ukt);	// q voltage
		f32_omf = f32_omf + ((f32_omegae - f32_omf) * f32_visu_ukt);	// angular speed

	}	// if(COM_ON)
	else
	{

		// speed PI integral term reset
		f32_errint = 0.0f;

		// current references reset
		f32_itr = f32_iqr = f32_idr = 0.0f;

#ifdef FLUXWEAK
		f32_amp_v = 0.0f;
		//f32_arg_v = SystemPhase;
		//f32_amp_i = 0.0f;
		//f32_arg_i = SystemPhase;
		f32_vmarg = f32_vfmax;
#endif  // ifdef FLUXWEAK

		// position and speed reset
		f32_SystemPhase = 0.0f;
		f32_omegae = 0.0f;

		// phase detection variables reset
		f32_Phase_est = f32_FluxPhase = f32_SystemPhase;
		f32_Speed_est = f32_omegae;
#ifndef APP_INT
		McrpLibf_FluxEst_SetFlux(f32_FluxPhase, f32_iam, f32_ibm, f32_omegae);
#else	// APP_INT
		McrpLibf_FluxEstA_SetFlux(f32_FluxPhase, f32_vam, f32_vbm, f32_iam, f32_ibm, f32_omegae);
#endif	// APP_INT
		u16_trip_cnt = 0;

#ifdef TRIP_CONTROL
		// position loss control reset
		MC_TripCon(0);
#endif	// ifdef TRIP_CONTROL

		// startup variables and flags reset
		u16_startup_cnt = 0;
		f32_idrvar = 0.0f;
		f32_iqmax = 0.0f;

		// filters memories reset
		f32_f_omrif = 0.0f;	// speed reference
        f32_idmf = 0.0f;    // d current
        f32_iqmf = 0.0f;    // q current
        f32_vdmf = 0.0f;    // d voltage
        f32_vqmf = 0.0f;    // q voltage
        f32_omf = 0.0f;     // angular speed
	}	// else of if(COM_ON)

	// bus voltage filter and vfmax calc
	if (f32_vbus < f32_vbus_minf)
	{
		f32_vbus_minf = f32_vbus_minf + ((f32_vbus - f32_vbus_minf) * f32_vb_ukt_fast); // fast decrease
	}
	else if(f32_vbus > f32_vbus_minf)
	{
		f32_vbus_minf = f32_vbus_minf + ((f32_vbus - f32_vbus_minf) * f32_vb_ukt_slow);	// slow increase
	}
	f32_vfmax = f32_vbus_minf * f32_k_vout;

	// bus voltage filter (visualization)
	f32_vb_fil += (f32_vbus - f32_vb_fil) * f32_visu_ukt;

#ifdef OSC_WIN
	// management of oscilloscope window in GUI
	if (NORM_MODE)
	{
		if (CSELD)
		{
			s16_sample = f32_iam * f32_imax1000;
		}
		else
		{
			s16_sample = f32_SystemPhase * 1000.0f;
		}
		if (COM_ON)
		{
			if (NOT_TRGED)
			{
				if (s16_selmem != ((uint16_t)UIF_W.var.sel))
				{
					s16_selmem = (uint16_t)UIF_W.var.sel;
					s16_bcntm = s16_selmem >> 8;
					if(0 == (s16_selmem & 0x00FF))
					{
						RES_CSELD
					}
					else
					{
						SET_CSELD
					}
				}
				if (((0.0f < f32_omegae) && (0.0f > f32_phmem) && (0.0f <= f32_SystemPhase)) ||
					((0.0f > f32_omegae) && (0.0f < f32_phmem) && (0.0f >= f32_SystemPhase)) )
				{
					SET_TRGED
					s16_bcnt1 = 1;
					s16_bcnt0 = 1;
					au16_wbuf[0] = s16_sample;
				}
			}
			else if (MAX_SAM_CNT > s16_bcnt0)
			{
				if (s16_bcnt1 < s16_bcntm)
				{
					s16_bcnt1++;
				}
				else
				{
					s16_bcnt1 = 1;
					au16_wbuf[s16_bcnt0++] = s16_sample;
				}
			}
			else if (NOT_RBUSY)
			{
				au16_pbuf = au16_rbuf;
				au16_rbuf = au16_wbuf;
				au16_wbuf = au16_pbuf;
				RES_TRGED
			}
		}	// COM_ON
		else
		{
			RES_TRGED
			if (s16_selmem != ((uint16_t)UIF_W.var.sel))
			{
				s16_selmem = (uint16_t)UIF_W.var.sel;
				s16_bcntm = s16_selmem >> 8;
				if (0 == (s16_selmem & 0x00FF))
				{
					RES_CSELD
				}
				else
				{
					SET_CSELD
				}
			}
			if (MAX_SAM_CNT > s16_bcnt0)
			{
				au16_wbuf[s16_bcnt0++] = 0;
			}
			else if (NOT_RBUSY)
			{
				au16_pbuf = au16_rbuf;
				au16_rbuf = au16_wbuf;
				au16_wbuf = au16_pbuf;
				s16_bcnt0 = 0;
			}
		}	// COM_OFF
		f32_phmem = f32_SystemPhase;
	}	// NORM_MODE
#endif	// ifdef OSC_WIN

	// main loop synchronization
    if (0 < u16_cnt_int)
    {
        u16_cnt_int--;
    }

    // time debug
#ifdef INTERRUPT_TIME_DEBUG
    TURN_OFF_LED
#endif  // ifdef INTERRUPT_TIME_DEBUG

    // interrupt end
	return;

}

// EOF motorcontrol.c

